@@ -1,14 +1,9 @@
-import React from 'react';
-import config from 'app/core/config';
+import React, { PureComponent } from 'react';
 
-import { updateLocation } from 'app/core/actions';
-import { connect } from 'react-redux';
-import { StoreState } from 'app/types';
-import { PureComponent } from 'react';
-import { getBackendSrv } from '@grafana/runtime';
-import { hot } from 'react-hot-loader';
-import appEvents from 'app/core/app_events';
 import { AppEvents } from '@grafana/data';
+import { getBackendSrv } from '@grafana/runtime';
+import appEvents from 'app/core/app_events';
+import config from 'app/core/config';
 
 const isOauthEnabled = () => {
   return !!config.oauth && Object.keys(config.oauth).length > 0;
@@ -19,9 +14,10 @@ export interface FormModel {
   password: string;
   email: string;
 }
+
 interface Props {
-  routeParams?: any;
-  updateLocation?: typeof updateLocation;
+  resetCode?: string;
+
   children: (props: {
     isLoggingIn: boolean;
     changePassword: (pw: string) => void;
@@ -45,6 +41,7 @@ interface State {
 
 export class LoginCtrl extends PureComponent<Props, State> {
   result: any = {};
+
   constructor(props: Props) {
     super(props);
     this.state = {
@@ -63,7 +60,20 @@ export class LoginCtrl extends PureComponent<Props, State> {
       confirmNew: password,
       oldPassword: 'admin',
     };
-    if (!this.props.routeParams.code) {
+
+    if (this.props.resetCode) {
+      const resetModel = {
+        code: this.props.resetCode,
+        newPassword: password,
+        confirmPassword: password,
+      };
+
+      getBackendSrv()
+        .post('/api/user/password/reset', resetModel)
+        .then(() => {
+          this.toGrafana();
+        });
+    } else {
       getBackendSrv()
         .put('/api/user/password', pw)
         .then(() => {
@@ -71,18 +81,6 @@ export class LoginCtrl extends PureComponent<Props, State> {
         })
         .catch((err: any) => console.error(err));
     }
-
-    const resetModel = {
-      code: this.props.routeParams.code,
-      newPassword: password,
-      confirmPassword: password,
-    };
-
-    getBackendSrv()
-      .post('/api/user/password/reset', resetModel)
-      .then(() => {
-        this.toGrafana();
-      });
   };
 
   login = (formModel: FormModel) => {
@@ -118,12 +116,12 @@ export class LoginCtrl extends PureComponent<Props, State> {
     // Use window.location.href to force page reload
     if (this.result.redirectUrl) {
       if (config.appSubUrl !== '' && !this.result.redirectUrl.startsWith(config.appSubUrl)) {
-        window.location.href = config.appSubUrl + this.result.redirectUrl;
+        window.location.assign(config.appSubUrl + this.result.redirectUrl);
       } else {
-        window.location.href = this.result.redirectUrl;
+        window.location.assign(this.result.redirectUrl);
       }
     } else {
-      window.location.href = config.appSubUrl + '/';
+      window.location.assign(config.appSubUrl + '/');
     }
   };
 
@@ -154,10 +152,4 @@ export class LoginCtrl extends PureComponent<Props, State> {
   }
 }
 
-export const mapStateToProps = (state: StoreState) => ({
-  routeParams: state.location.routeParams,
-});
-
-const mapDispatchToProps = { updateLocation };
-
-export default hot(module)(connect(mapStateToProps, mapDispatchToProps)(LoginCtrl));
+export default LoginCtrl;
