@@ -1,9 +1,12 @@
+import { css } from '@emotion/css';
 import React, { FC, ReactElement } from 'react';
+
 import { selectors } from '@grafana/e2e-selectors';
+import { Button, Form, Input, Field } from '@grafana/ui';
+
+import { PasswordField } from '../PasswordField/PasswordField';
 
 import { FormModel } from './LoginCtrl';
-import { Button, Form, Input, Field } from '@grafana/ui';
-import { css } from 'emotion';
 
 interface Props {
   children: ReactElement;
@@ -31,24 +34,27 @@ export const LoginForm: FC<Props> = ({ children, onSubmit, isLoggingIn, password
           <>
             <Field label="Email or username" invalid={!!errors.user} error={errors.user?.message}>
               <Input
+                {...register('user', { required: 'Email or username is required' })}
                 autoFocus
-                name="user"
                 autoCapitalize="none"
-                ref={register({ required: 'Email or username is required' })}
                 placeholder={loginHint}
                 aria-label={selectors.pages.Login.username}
               />
             </Field>
             <Field label="Password" invalid={!!errors.password} error={errors.password?.message}>
-              <Input
-                name="password"
-                type="password"
-                placeholder={passwordHint}
-                ref={register({ required: 'Password is required' })}
-                aria-label={selectors.pages.Login.password}
+              <PasswordField
+                id="current-password"
+                autoComplete="current-password"
+                passwordHint={passwordHint}
+                {...register('password', { required: 'Password is required' })}
               />
             </Field>
-            <Button aria-label={selectors.pages.Login.submit} className={submitButton} disabled={isLoggingIn}>
+            <Button
+              type="submit"
+              aria-label={selectors.pages.Login.submit}
+              className={submitButton}
+              disabled={isLoggingIn}
+            >
               {isLoggingIn ? 'Logging in...' : 'Log in'}
             </Button>
             {children}
