@@ -1,9 +1,12 @@
+import { css } from '@emotion/css';
 import React, { FC } from 'react';
-import { css } from 'emotion';
-import { Tab, TabsBar, Icon, IconName, useStyles } from '@grafana/ui';
-import { NavModel, NavModelItem, NavModelBreadcrumb, GrafanaTheme } from '@grafana/data';
+
+import { NavModel, NavModelItem, NavModelBreadcrumb, GrafanaTheme2 } from '@grafana/data';
+import { Tab, TabsBar, Icon, IconName, useStyles2 } from '@grafana/ui';
 import { PanelHeaderMenuItem } from 'app/features/dashboard/dashgrid/PanelHeader/PanelHeaderMenuItem';
 
+import { ProBadge } from '../Upgrade/ProBadge';
+
 export interface Props {
   model: NavModel;
 }
@@ -62,6 +65,7 @@ const Navigation = ({ children }: { children: NavModelItem[] }) => {
                 key={`${child.url}-${index}`}
                 icon={child.icon as IconName}
                 href={child.url}
+                suffix={child.tabSuffix}
               />
             )
           );
@@ -72,7 +76,7 @@ const Navigation = ({ children }: { children: NavModelItem[] }) => {
 };
 
 export const PageHeader: FC<Props> = ({ model }) => {
-  const styles = useStyles(getStyles);
+  const styles = useStyles2(getStyles);
 
   if (!model) {
     return null;
@@ -80,7 +84,6 @@ export const PageHeader: FC<Props> = ({ model }) => {
 
   const main = model.main;
   const children = main.children;
-
   return (
     <div className={styles.headerCanvas}>
       <div className="page-container">
@@ -104,27 +107,39 @@ function renderHeaderTitle(main: NavModelItem) {
       </span>
 
       <div className="page-header__info-block">
-        {renderTitle(main.text, main.breadcrumbs ?? [])}
+        {renderTitle(main.text, main.breadcrumbs ?? [], main.highlightText)}
         {main.subTitle && <div className="page-header__sub-title">{main.subTitle}</div>}
       </div>
     </div>
   );
 }
 
-function renderTitle(title: string, breadcrumbs: NavModelBreadcrumb[]) {
+function renderTitle(title: string, breadcrumbs: NavModelBreadcrumb[], highlightText: NavModelItem['highlightText']) {
   if (!title && (!breadcrumbs || breadcrumbs.length === 0)) {
     return null;
   }
 
   if (!breadcrumbs || breadcrumbs.length === 0) {
-    return <h1 className="page-header__title">{title}</h1>;
+    return (
+      <h1 className="page-header__title">
+        {title}
+        {highlightText && (
+          <ProBadge
+            text={highlightText}
+            className={css`
+              vertical-align: middle;
+            `}
+          />
+        )}
+      </h1>
+    );
   }
 
   const breadcrumbsResult = [];
   for (const bc of breadcrumbs) {
     if (bc.url) {
       breadcrumbsResult.push(
-        <a className="text-link" key={breadcrumbsResult.length} href={bc.url}>
+        <a className="page-header__link" key={breadcrumbsResult.length} href={bc.url}>
           {bc.title}
         </a>
       );
@@ -137,10 +152,9 @@ function renderTitle(title: string, breadcrumbs: NavModelBreadcrumb[]) {
   return <h1 className="page-header__title">{breadcrumbsResult}</h1>;
 }
 
-const getStyles = (theme: GrafanaTheme) => ({
+const getStyles = (theme: GrafanaTheme2) => ({
   headerCanvas: css`
-    background: ${theme.colors.bg2};
-    border-bottom: 1px solid ${theme.colors.border1};
+    background: ${theme.colors.background.canvas};
   `,
 });
 
