@@ -1,6 +1,6 @@
+import { render, screen } from '@testing-library/react';
 import React from 'react';
-// @ts-ignore
-import renderer from 'react-test-renderer';
+
 import { TeamPicker } from './TeamPicker';
 
 jest.mock('@grafana/runtime', () => ({
@@ -14,11 +14,11 @@ jest.mock('@grafana/runtime', () => ({
 }));
 
 describe('TeamPicker', () => {
-  it('renders correctly', () => {
+  it('renders correctly', async () => {
     const props = {
       onSelected: () => {},
     };
-    const tree = renderer.create(<TeamPicker {...props} />).toJSON();
-    expect(tree).toMatchSnapshot();
+    render(<TeamPicker {...props} />);
+    expect(await screen.findByTestId('teamPicker')).toBeInTheDocument();
   });
 });
