@@ -1,26 +1,26 @@
 import React from 'react';
+
 import { TagBadge } from './TagBadge';
 
-export interface IProps {
+export interface Props {
   value: any;
   className: any;
   onClick: any;
   onRemove: any;
 }
 
-export class TagValue extends React.Component<IProps, any> {
-  constructor(props) {
+export class TagValue extends React.Component<Props, any> {
+  constructor(props: Props) {
     super(props);
     this.onClick = this.onClick.bind(this);
   }
 
-  onClick(event) {
+  onClick(event: React.SyntheticEvent) {
     this.props.onRemove(this.props.value, event);
   }
 
   render() {
     const { value } = this.props;
-
-    return <TagBadge label={value.label} removeIcon={true} count={0} onClick={this.onClick} />;
+    return <TagBadge label={value.label} removeIcon={false} count={0} onClick={this.onClick} />;
   }
 }
