@@ -1,21 +1,22 @@
-import store from 'app/core/store';
-import _ from 'lodash';
+import { filter, isArray, isNumber } from 'lodash';
+
 import config from 'app/core/config';
+import store from 'app/core/store';
 
 export class ImpressionSrv {
   constructor() {}
 
-  addDashboardImpression(dashboardId) {
-    var impressionsKey = this.impressionKey(config);
-    var impressions = [];
+  addDashboardImpression(dashboardId: number) {
+    const impressionsKey = this.impressionKey();
+    let impressions = [];
     if (store.exists(impressionsKey)) {
       impressions = JSON.parse(store.get(impressionsKey));
-      if (!_.isArray(impressions)) {
+      if (!isArray(impressions)) {
         impressions = [];
       }
     }
 
-    impressions = impressions.filter(imp => {
+    impressions = impressions.filter((imp) => {
       return dashboardId !== imp;
     });
 
@@ -27,19 +28,20 @@ export class ImpressionSrv {
     store.set(impressionsKey, JSON.stringify(impressions));
   }
 
-  getDashboardOpened() {
-    var impressions = store.get(this.impressionKey(config)) || '[]';
+  /** Returns an array of internal (numeric) dashboard IDs */
+  getDashboardOpened(): number[] {
+    let impressions = store.get(this.impressionKey()) || '[]';
 
     impressions = JSON.parse(impressions);
 
-    impressions = _.filter(impressions, el => {
-      return _.isNumber(el);
+    impressions = filter(impressions, (el) => {
+      return isNumber(el);
     });
 
     return impressions;
   }
 
-  impressionKey(config) {
+  impressionKey() {
     return 'dashboard_impressions-' + config.bootData.user.orgId;
   }
 }
