@@ -1,4 +1,4 @@
-import _ from 'lodash';
+import { isNumber } from 'lodash';
 
 const versionPattern = /^(\d+)(?:\.(\d+))?(?:\.(\d+))?(?:-([0-9A-Za-z\.]+))?/;
 
@@ -9,6 +9,10 @@ export class SemVersion {
   meta: string;
 
   constructor(version: string) {
+    this.major = 0;
+    this.minor = 0;
+    this.patch = 0;
+    this.meta = '';
     const match = versionPattern.exec(version);
     if (match) {
       this.major = Number(match[1]);
@@ -33,7 +37,7 @@ export class SemVersion {
   }
 
   isValid(): boolean {
-    return _.isNumber(this.major);
+    return isNumber(this.major);
   }
 
   get comparable() {
