@@ -1,5 +1,7 @@
+import { isString } from 'lodash';
 import { Observable, of, OperatorFunction } from 'rxjs';
 import { map, mergeMap } from 'rxjs/operators';
+
 import {
   AnnotationEvent,
   AnnotationEventFieldSource,
@@ -7,6 +9,7 @@ import {
   AnnotationQuery,
   AnnotationSupport,
   DataFrame,
+  DataSourceApi,
   Field,
   FieldType,
   getFieldDisplayName,
@@ -14,8 +17,6 @@ import {
   standardTransformers,
 } from '@grafana/data';
 
-import isString from 'lodash/isString';
-
 export const standardAnnotationSupport: AnnotationSupport = {
   /**
    * Assume the stored value is standard model.
@@ -26,6 +27,7 @@ export const standardAnnotationSupport: AnnotationSupport = {
       return {
         ...rest,
         target: {
+          refId: 'annotation_query',
           query,
         },
         mappings: {},
@@ -35,14 +37,12 @@ export const standardAnnotationSupport: AnnotationSupport = {
   },
 
   /**
-   * Convert the stored JSON model and environment to a standard datasource query object.
-   * This query will be executed in the datasource and the results converted into events.
-   * Returning an undefined result will quietly skip query execution
+   * Default will just return target from the annotation.
    */
   prepareQuery: (anno: AnnotationQuery) => anno.target,
 
   /**
-   * When the standard frame > event processing is insufficient, this allows explicit control of the mappings
+   * Provides default processing from dataFrame to annotation events.
    */
   processEvents: (anno: AnnotationQuery, data: DataFrame[]) => {
     return getAnnotationsFromData(data, anno.mappings);
@@ -50,7 +50,7 @@ export const standardAnnotationSupport: AnnotationSupport = {
 };
 
 /**
- * Flatten all panel data into a single frame
+ * Flatten all frames into a single frame with mergeTransformer.
  */
 
 export function singleFrameFromPanelData(): OperatorFunction<DataFrame[], DataFrame | undefined> {
@@ -90,6 +90,7 @@ export interface AnnotationFieldInfo {
   help?: string;
 }
 
+// These fields get added to the standard UI
 export const annotationEventNames: AnnotationFieldInfo[] = [
   {
     key: 'time',
@@ -106,9 +107,24 @@ export const annotationEventNames: AnnotationFieldInfo[] = [
     placeholder: 'text, or the first text field',
   },
   { key: 'tags', split: ',', help: 'The results will be split on comma (,)' },
-  // { key: 'userId' },
-  // { key: 'login' },
-  // { key: 'email' },
+  {
+    key: 'id',
+  },
+];
+
+// Given legacy infrastructure, alert events are passed though the same annotation
+// pipeline, but include fields that should not be exposed generally
+const alertEventAndAnnotationFields: AnnotationFieldInfo[] = [
+  ...annotationEventNames,
+  { key: 'userId' },
+  { key: 'login' },
+  { key: 'email' },
+  { key: 'prevState' },
+  { key: 'newState' },
+  { key: 'data' as any },
+  { key: 'panelId' },
+  { key: 'alertId' },
+  { key: 'dashboardId' },
 ];
 
 export function getAnnotationsFromData(
@@ -137,7 +153,7 @@ export function getAnnotationsFromData(
 
       const fields: AnnotationEventFieldSetter[] = [];
 
-      for (const evts of annotationEventNames) {
+      for (const evts of alertEventAndAnnotationFields) {
         const opt = options[evts.key] || {}; //AnnotationEventFieldMapping
 
         if (opt.source === AnnotationEventFieldSource.Skip) {
@@ -210,3 +226,29 @@ export function getAnnotationsFromData(
     })
   );
 }
+
+// These opt outs are here only for quicker and easier migration to react based annotations editors and because
+// annotation support API needs some work to support less "standard" editors like prometheus and here it is not
+// polluting public API.
+
+/**
+ * Opt out of using the default mapping functionality on frontend.
+ */
+export function shouldUseMappingUI(datasource: DataSourceApi): boolean {
+  return (
+    datasource.type !== 'prometheus' &&
+    datasource.type !== 'grafana-opensearch-datasource' &&
+    datasource.type !== 'grafana-splunk-datasource'
+  );
+}
+
+/**
+ * Use legacy runner. Used only as an escape hatch for easier transition to React based annotation editor.
+ */
+export function shouldUseLegacyRunner(datasource: DataSourceApi): boolean {
+  return (
+    datasource.type === 'prometheus' ||
+    datasource.type === 'grafana-opensearch-datasource' ||
+    datasource.type === 'grafana-splunk-datasource'
+  );
+}
