@@ -1,11 +1,12 @@
 import React from 'react';
+
 import { HorizontalGroup, Tooltip, Button } from '@grafana/ui';
 
 type VersionsButtonsType = {
   hasMore: boolean;
   canCompare: boolean;
   getVersions: (append: boolean) => void;
-  getDiff: (diff: string) => void;
+  getDiff: () => void;
   isLastPage: boolean;
 };
 export const VersionsHistoryButtons: React.FC<VersionsButtonsType> = ({
@@ -21,8 +22,8 @@ export const VersionsHistoryButtons: React.FC<VersionsButtonsType> = ({
         Show more versions
       </Button>
     )}
-    <Tooltip content="Select 2 versions to start comparing" placement="bottom">
-      <Button type="button" disabled={canCompare} onClick={() => getDiff('basic')} icon="code-branch">
+    <Tooltip content="Select two versions to start comparing" placement="bottom">
+      <Button type="button" disabled={!canCompare} onClick={getDiff} icon="code-branch">
         Compare versions
       </Button>
     </Tooltip>
