@@ -1,7 +1,11 @@
-import _ from 'lodash';
-import { DashboardModel } from '../state/DashboardModel';
+import { compact, flattenDeep, map, uniq } from 'lodash';
 import { expect } from 'test/lib/common';
+
+import { DashboardPanelsChangedEvent } from 'app/types/events';
+
 import { getDashboardModel } from '../../../../test/helpers/getDashboardModel';
+import { DashboardModel } from '../state/DashboardModel';
+
 import { PanelModel } from './PanelModel';
 
 jest.mock('app/core/services/context_srv', () => ({}));
@@ -19,6 +23,7 @@ describe('given dashboard with panel repeat', () => {
         list: [
           {
             name: 'apps',
+            type: 'custom',
             current: {
               text: 'se1, se2, se3',
               value: ['se1', 'se2', 'se3'],
@@ -74,6 +79,7 @@ describe('given dashboard with panel repeat in horizontal direction', () => {
         list: [
           {
             name: 'apps',
+            type: 'custom',
             current: {
               text: 'se1, se2, se3',
               value: ['se1', 'se2', 'se3'],
@@ -204,6 +210,7 @@ describe('given dashboard with panel repeat in vertical direction', () => {
         list: [
           {
             name: 'apps',
+            type: 'custom',
             current: {
               text: 'se1, se2, se3',
               value: ['se1', 'se2', 'se3'],
@@ -246,6 +253,7 @@ describe('given dashboard with row repeat and panel repeat in horizontal directi
         list: [
           {
             name: 'region',
+            type: 'custom',
             current: {
               text: 'reg1, reg2',
               value: ['reg1', 'reg2'],
@@ -257,6 +265,7 @@ describe('given dashboard with row repeat and panel repeat in horizontal directi
           },
           {
             name: 'app',
+            type: 'custom',
             current: {
               text: 'se1, se2, se3, se4, se5, se6',
               value: ['se1', 'se2', 'se3', 'se4', 'se5', 'se6'],
@@ -278,7 +287,7 @@ describe('given dashboard with row repeat and panel repeat in horizontal directi
   });
 
   it('should panels in self row', () => {
-    const panelTypes = _.map(dashboard.panels, 'type');
+    const panelTypes = map(dashboard.panels, 'type');
     expect(panelTypes).toEqual([
       'row',
       'graph',
@@ -339,6 +348,7 @@ describe('given dashboard with row repeat', () => {
         list: [
           {
             name: 'apps',
+            type: 'custom',
             current: {
               text: 'se1, se2',
               value: ['se1', 'se2'],
@@ -357,7 +367,7 @@ describe('given dashboard with row repeat', () => {
   });
 
   it('should not repeat only row', () => {
-    const panelTypes = _.map(dashboard.panels, 'type');
+    const panelTypes = map(dashboard.panels, 'type');
     expect(panelTypes).toEqual(['row', 'graph', 'graph', 'row', 'graph', 'graph', 'row', 'graph']);
   });
 
@@ -373,8 +383,8 @@ describe('given dashboard with row repeat', () => {
       apps: { text: 'se2', value: 'se2' },
     });
 
-    const scopedVars = _.compact(
-      _.map(dashboard.panels, (panel) => {
+    const scopedVars = compact(
+      map(dashboard.panels, (panel) => {
         return panel.scopedVars ? panel.scopedVars.apps.value : null;
       })
     );
@@ -406,7 +416,7 @@ describe('given dashboard with row repeat', () => {
     dashboard = getDashboardModel(dashboardJSON);
     dashboard.processRepeats();
 
-    const panelTypes = _.map(dashboard.panels, 'type');
+    const panelTypes = map(dashboard.panels, 'type');
     expect(panelTypes).toEqual(['row', 'row', 'row', 'graph']);
     expect(dashboard.panels[0].panels).toHaveLength(2);
     expect(dashboard.panels[1].panels).toHaveLength(2);
@@ -435,6 +445,7 @@ describe('given dashboard with row repeat', () => {
     ];
     dashboardJSON.templating.list.push({
       name: 'hosts',
+      type: 'custom',
       current: {
         text: 'backend01, backend02',
         value: ['backend01', 'backend02'],
@@ -448,7 +459,7 @@ describe('given dashboard with row repeat', () => {
     dashboard = getDashboardModel(dashboardJSON);
     dashboard.processRepeats();
 
-    const panelTypes = _.map(dashboard.panels, 'type');
+    const panelTypes = map(dashboard.panels, 'type');
     expect(panelTypes).toEqual([
       'row',
       'graph',
@@ -495,17 +506,17 @@ describe('given dashboard with row repeat', () => {
     dashboard = getDashboardModel(dashboardJSON);
     dashboard.processRepeats();
 
-    const panelIds = _.flattenDeep(
-      _.map(dashboard.panels, (panel) => {
+    const panelIds = flattenDeep(
+      map(dashboard.panels, (panel) => {
         let ids = [];
         if (panel.panels && panel.panels.length) {
-          ids = _.map(panel.panels, 'id');
+          ids = map(panel.panels, 'id');
         }
         ids.push(panel.id);
         return ids;
       })
     );
-    expect(panelIds.length).toEqual(_.uniq(panelIds).length);
+    expect(panelIds.length).toEqual(uniq(panelIds).length);
   });
 
   it('should place new panels in proper order', () => {
@@ -518,15 +529,15 @@ describe('given dashboard with row repeat', () => {
     dashboard = getDashboardModel(dashboardJSON);
     dashboard.processRepeats();
 
-    const panelTypes = _.map(dashboard.panels, 'type');
+    const panelTypes = map(dashboard.panels, 'type');
     expect(panelTypes).toEqual(['row', 'graph', 'graph', 'graph', 'row', 'graph', 'graph', 'graph']);
-    const panelYPositions = _.map(dashboard.panels, (p) => p.gridPos.y);
+    const panelYPositions = map(dashboard.panels, (p) => p.gridPos.y);
     expect(panelYPositions).toEqual([0, 1, 1, 5, 7, 8, 8, 12]);
   });
 });
 
 describe('given dashboard with row and panel repeat', () => {
-  let dashboard: any, dashboardJSON: any;
+  let dashboard: DashboardModel, dashboardJSON: any;
 
   beforeEach(() => {
     dashboardJSON = {
@@ -543,6 +554,7 @@ describe('given dashboard with row and panel repeat', () => {
         list: [
           {
             name: 'region',
+            type: 'custom',
             current: {
               text: 'reg1, reg2',
               value: ['reg1', 'reg2'],
@@ -555,6 +567,7 @@ describe('given dashboard with row and panel repeat', () => {
           },
           {
             name: 'app',
+            type: 'custom',
             current: {
               text: 'se1, se2',
               value: ['se1', 'se2'],
@@ -573,10 +586,18 @@ describe('given dashboard with row and panel repeat', () => {
   });
 
   it('should repeat row and panels for each row', () => {
-    const panelTypes = _.map(dashboard.panels, 'type');
+    const panelTypes = map(dashboard.panels, 'type');
     expect(panelTypes).toEqual(['row', 'graph', 'graph', 'row', 'graph', 'graph']);
   });
 
+  it('Row repeat should create new panel keys every repeat cycle', () => {
+    // This is the first repeated panel inside the second repeated row
+    // Since we create a new panel model every time (and new panel events bus) we need to create a new key here to trigger a re-mount & re-subscribe
+    const key1 = dashboard.panels[3].key;
+    dashboard.processRepeats();
+    expect(key1).not.toEqual(dashboard.panels[3].key);
+  });
+
   it('should clean up old repeated panels', () => {
     dashboardJSON.panels = [
       {
@@ -596,11 +617,16 @@ describe('given dashboard with row and panel repeat', () => {
       },
       { id: 12, type: 'graph', repeatPanelId: 2, repeatIteration: 101, gridPos: { x: 0, y: 3, h: 1, w: 6 } },
     ];
+
+    let panelChangedEvents: DashboardPanelsChangedEvent[] = [];
     dashboard = getDashboardModel(dashboardJSON);
+    dashboard.events.subscribe(DashboardPanelsChangedEvent, (evt) => panelChangedEvents.push(evt));
     dashboard.processRepeats();
 
-    const panelTypes = _.map(dashboard.panels, 'type');
+    const panelTypes = map(dashboard.panels, 'type');
     expect(panelTypes).toEqual(['row', 'graph', 'graph', 'row', 'graph', 'graph']);
+    // Make sure only a single DashboardPanelsChangedEvent event is emitted when processing repeats
+    expect(panelChangedEvents.length).toBe(1);
   });
 
   it('should set scopedVars for each row', () => {
@@ -673,6 +699,111 @@ describe('given dashboard with row and panel repeat', () => {
   });
 });
 
+// fix for https://github.com/grafana/grafana/issues/38805
+describe('given dashboard with row and repeats on same row', () => {
+  it('should set correct gridPos when row is expanding', () => {
+    const ROW1 = 1;
+    const GAUGE1 = 2;
+    const REPEAT1 = 3;
+    const GAUGE2 = 4;
+    const REPEAT2 = 5;
+    const GAUGE3 = 6;
+    const dashboardJSON = {
+      panels: [
+        {
+          collapsed: true,
+          datasource: null,
+          gridPos: { h: 1, w: 24, x: 0, y: 0 },
+          id: ROW1,
+          panels: [
+            { gridPos: { h: 5, w: 4, x: 0, y: 1 }, id: GAUGE1, type: 'gauge' },
+            {
+              gridPos: { h: 5, w: 4, x: 4, y: 1 },
+              id: REPEAT1,
+              repeat: 'abc',
+              repeatDirection: 'v',
+              type: 'gauge',
+            },
+            { gridPos: { h: 5, w: 4, x: 8, y: 1 }, id: GAUGE2, type: 'gauge' },
+            {
+              gridPos: { h: 5, w: 4, x: 12, y: 1 },
+              id: REPEAT2,
+              repeat: 'abc',
+              repeatDirection: 'v',
+              type: 'gauge',
+            },
+            { gridPos: { h: 5, w: 4, x: 16, y: 1 }, id: GAUGE3, type: 'gauge' },
+          ],
+          title: 'Row title',
+          type: 'row',
+        },
+      ],
+      templating: {
+        list: [
+          {
+            allValue: null,
+            current: { selected: true, text: ['All'], value: ['$__all'] },
+            includeAll: true,
+            name: 'abc',
+            options: [
+              { selected: true, text: 'All', value: '$__all' },
+              { selected: false, text: 'a', value: 'a' },
+              { selected: false, text: 'b', value: 'b' },
+              { selected: false, text: 'c', value: 'c' },
+              { selected: false, text: 'd', value: 'd' },
+              { selected: false, text: 'e', value: 'e' },
+              { selected: false, text: 'f', value: 'f' },
+              { selected: false, text: 'g', value: 'g' },
+            ],
+            type: 'custom',
+          },
+        ],
+      },
+    };
+    const dashboard = getDashboardModel(dashboardJSON);
+
+    // toggle row
+    dashboard.toggleRow(dashboard.panels[0]);
+
+    // correct number of panels
+    expect(dashboard.panels.length).toBe(18);
+
+    // check row
+    const rowPanel = dashboard.panels.find((p) => p.id === ROW1);
+    expect(rowPanel?.gridPos).toEqual({ x: 0, y: 0, w: 24, h: 1 });
+
+    // check the gridPos of all the top level panels that are next to each other
+    const firstGauge = dashboard.panels.find((p) => p.id === GAUGE1);
+    const secondGauge = dashboard.panels.find((p) => p.id === GAUGE2);
+    const thirdGauge = dashboard.panels.find((p) => p.id === GAUGE3);
+    const firstVerticalRepeatingGauge = dashboard.panels.find((p) => p.id === REPEAT1);
+    const secondVerticalRepeatingGauge = dashboard.panels.find((p) => p.id === REPEAT2);
+    expect(firstGauge?.gridPos).toEqual({ x: 0, y: 1, w: 4, h: 5 });
+    expect(secondGauge?.gridPos).toEqual({ x: 8, y: 1, w: 4, h: 5 });
+    expect(thirdGauge?.gridPos).toEqual({ x: 16, y: 1, w: 4, h: 5 });
+    expect(firstVerticalRepeatingGauge?.gridPos).toEqual({ x: 4, y: 1, w: 4, h: 5 });
+    expect(secondVerticalRepeatingGauge?.gridPos).toEqual({ x: 12, y: 1, w: 4, h: 5 });
+
+    // check the gridPos of all first vertical repeats children
+    const { x, h, w } = firstVerticalRepeatingGauge!.gridPos;
+    expect(dashboard.panels[6].gridPos).toEqual({ x, y: 6, w, h });
+    expect(dashboard.panels[8].gridPos).toEqual({ x, y: 11, w, h });
+    expect(dashboard.panels[10].gridPos).toEqual({ x, y: 16, w, h });
+    expect(dashboard.panels[12].gridPos).toEqual({ x, y: 21, w, h });
+    expect(dashboard.panels[14].gridPos).toEqual({ x, y: 26, w, h });
+    expect(dashboard.panels[16].gridPos).toEqual({ x, y: 31, w, h });
+
+    // check the gridPos of all second vertical repeats children
+    const { x: x2, h: h2, w: w2 } = secondVerticalRepeatingGauge!.gridPos;
+    expect(dashboard.panels[7].gridPos).toEqual({ x: x2, y: 6, w: w2, h: h2 });
+    expect(dashboard.panels[9].gridPos).toEqual({ x: x2, y: 11, w: w2, h: h2 });
+    expect(dashboard.panels[11].gridPos).toEqual({ x: x2, y: 16, w: w2, h: h2 });
+    expect(dashboard.panels[13].gridPos).toEqual({ x: x2, y: 21, w: w2, h: h2 });
+    expect(dashboard.panels[15].gridPos).toEqual({ x: x2, y: 26, w: w2, h: h2 });
+    expect(dashboard.panels[17].gridPos).toEqual({ x: x2, y: 31, w: w2, h: h2 });
+  });
+});
+
 describe('given panel is in view mode', () => {
   let dashboard: any;
 
@@ -690,6 +821,7 @@ describe('given panel is in view mode', () => {
         list: [
           {
             name: 'apps',
+            type: 'custom',
             current: {
               text: 'se1, se2, se3',
               value: ['se1', 'se2', 'se3'],
