@@ -1,16 +1,19 @@
-import React from 'react';
 import { shallow } from 'enzyme';
-import { DataSourceDashboards, Props } from './DataSourceDashboards';
-import { DataSourceSettings } from '@grafana/data';
-import { NavModel } from '@grafana/data';
+import React from 'react';
+
+import { DataSourceSettings, NavModel } from '@grafana/data';
+import { getRouteComponentProps } from 'app/core/navigation/__mocks__/routeProps';
 import { PluginDashboard } from 'app/types';
 
+import { DataSourceDashboards, Props } from './DataSourceDashboards';
+
 const setup = (propOverrides?: object) => {
   const props: Props = {
+    ...getRouteComponentProps(),
     navModel: {} as NavModel,
     dashboards: [] as PluginDashboard[],
     dataSource: {} as DataSourceSettings,
-    pageId: 1,
+    dataSourceId: 'x',
     importDashboard: jest.fn(),
     loadDataSource: jest.fn(),
     loadPluginDashboards: jest.fn(),
