@@ -1,12 +1,20 @@
-import React from 'react';
 import { shallow } from 'enzyme';
-import { DataSourceSettings, NavModel } from '@grafana/data';
+import React from 'react';
+
+import { DataSourceSettings, NavModel, LayoutModes } from '@grafana/data';
 
 import { DataSourcesListPage, Props } from './DataSourcesListPage';
-import { LayoutModes } from '../../core/components/LayoutSelector/LayoutSelector';
 import { getMockDataSources } from './__mocks__/dataSourcesMocks';
 import { setDataSourcesLayoutMode, setDataSourcesSearchQuery } from './state/reducers';
 
+jest.mock('app/core/core', () => {
+  return {
+    contextSrv: {
+      hasPermission: () => true,
+    },
+  };
+});
+
 const setup = (propOverrides?: object) => {
   const props: Props = {
     dataSources: [] as DataSourceSettings[],
