@@ -1,30 +1,31 @@
 import React from 'react';
+import { connect, ConnectedProps } from 'react-redux';
+
+import { CoreApp, TimeZone } from '@grafana/data';
 import { TabbedContainer, TabConfig } from '@grafana/ui';
-import { PanelData, TimeZone } from '@grafana/data';
-import { runQueries } from './state/query';
-import { StoreState, ExploreItemState, ExploreId } from 'app/types';
-import { hot } from 'react-hot-loader';
-import { connect } from 'react-redux';
 import { ExploreDrawer } from 'app/features/explore/ExploreDrawer';
-import { InspectJSONTab } from 'app/features/inspector/InspectJSONTab';
-import { QueryInspector } from 'app/features/inspector/QueryInspector';
-import { InspectStatsTab } from 'app/features/inspector/InspectStatsTab';
 import { InspectDataTab } from 'app/features/inspector/InspectDataTab';
+import { InspectErrorTab } from 'app/features/inspector/InspectErrorTab';
+import { InspectJSONTab } from 'app/features/inspector/InspectJSONTab';
+import { InspectStatsTab } from 'app/features/inspector/InspectStatsTab';
+import { QueryInspector } from 'app/features/inspector/QueryInspector';
+import { StoreState, ExploreItemState, ExploreId } from 'app/types';
+
+import { runQueries } from './state/query';
 
 interface DispatchProps {
-  runQueries: typeof runQueries;
-}
-interface Props extends DispatchProps {
-  loading: boolean;
   width: number;
   exploreId: ExploreId;
-  queryResponse?: PanelData;
+  timeZone: TimeZone;
   onClose: () => void;
 }
 
+type Props = DispatchProps & ConnectedProps<typeof connector>;
+
 export function ExploreQueryInspector(props: Props) {
-  const { loading, width, onClose, queryResponse } = props;
+  const { loading, width, onClose, queryResponse, timeZone } = props;
   const dataFrames = queryResponse?.series || [];
+  const error = queryResponse?.error;
 
   const statsTab: TabConfig = {
     label: 'Stats',
@@ -49,20 +50,31 @@ export function ExploreQueryInspector(props: Props) {
         data={dataFrames}
         isLoading={loading}
         options={{ withTransforms: false, withFieldConfig: false }}
+        timeZone={timeZone}
+        app={CoreApp.Explore}
       />
     ),
   };
 
-  const queryInspectorTab: TabConfig = {
-    label: 'Query Inspector',
-    value: 'query_inspector',
+  const queryTab: TabConfig = {
+    label: 'Query',
+    value: 'query',
     icon: 'info-circle',
     content: <QueryInspector data={dataFrames} onRefreshQuery={() => props.runQueries(props.exploreId)} />,
   };
 
-  const tabs = [statsTab, queryInspectorTab, jsonTab, dataTab];
+  const tabs = [statsTab, queryTab, jsonTab, dataTab];
+  if (error) {
+    const errorTab: TabConfig = {
+      label: 'Error',
+      value: 'error',
+      icon: 'exclamation-triangle',
+      content: <InspectErrorTab error={error} />,
+    };
+    tabs.push(errorTab);
+  }
   return (
-    <ExploreDrawer width={width} onResize={() => {}}>
+    <ExploreDrawer width={width}>
       <TabbedContainer tabs={tabs} onClose={onClose} closeIconTooltip="Close query inspector" />
     </ExploreDrawer>
   );
@@ -79,8 +91,10 @@ function mapStateToProps(state: StoreState, { exploreId }: { exploreId: ExploreI
   };
 }
 
-const mapDispatchToProps: DispatchProps = {
+const mapDispatchToProps = {
   runQueries,
 };
 
-export default hot(module)(connect(mapStateToProps, mapDispatchToProps)(ExploreQueryInspector));
+const connector = connect(mapStateToProps, mapDispatchToProps);
+
+export default connector(ExploreQueryInspector);
