@@ -1,58 +1,58 @@
+import { css } from '@emotion/css';
 import React, { PureComponent } from 'react';
-import { hot } from 'react-hot-loader';
 import { connect, ConnectedProps } from 'react-redux';
+
+import {
+  AbsoluteTimeRange,
+  Field,
+  hasLogsContextSupport,
+  LoadingState,
+  LogRowModel,
+  RawTimeRange,
+} from '@grafana/data';
 import { Collapse } from '@grafana/ui';
-
-import { AbsoluteTimeRange, Field, LogLevel, LogRowModel, LogsDedupStrategy, RawTimeRange } from '@grafana/data';
-
-import { ExploreId, ExploreItemState } from 'app/types/explore';
 import { StoreState } from 'app/types';
+import { ExploreId, ExploreItemState } from 'app/types/explore';
 
-import { splitOpen } from './state/main';
-import { updateTimeRange } from './state/time';
-import { toggleLogLevelAction, changeDedupStrategy } from './state/explorePane';
-import { deduplicatedRowsSelector } from './state/selectors';
 import { getTimeZone } from '../profile/state/selectors';
+
 import { LiveLogsWithTheme } from './LiveLogs';
 import { Logs } from './Logs';
-import { LogsCrossFadeTransition } from './utils/LogsCrossFadeTransition';
+import { splitOpen } from './state/main';
+import { addResultsToCache, clearCache } from './state/query';
+import { updateTimeRange } from './state/time';
 import { LiveTailControls } from './useLiveTailControls';
+import { LogsCrossFadeTransition } from './utils/LogsCrossFadeTransition';
 import { getFieldLinksForExplore } from './utils/links';
 
-interface LogsContainerProps {
+interface LogsContainerProps extends PropsFromRedux {
+  width: number;
   exploreId: ExploreId;
   scanRange?: RawTimeRange;
-  width: number;
   syncedTimes: boolean;
+  loadingState: LoadingState;
   onClickFilterLabel?: (key: string, value: string) => void;
   onClickFilterOutLabel?: (key: string, value: string) => void;
   onStartScanning: () => void;
   onStopScanning: () => void;
 }
 
-export class LogsContainer extends PureComponent<PropsFromRedux & LogsContainerProps> {
+class LogsContainer extends PureComponent<LogsContainerProps> {
   onChangeTime = (absoluteRange: AbsoluteTimeRange) => {
     const { exploreId, updateTimeRange } = this.props;
     updateTimeRange({ exploreId, absoluteRange });
   };
 
-  handleDedupStrategyChange = (dedupStrategy: LogsDedupStrategy) => {
-    this.props.changeDedupStrategy(this.props.exploreId, dedupStrategy);
-  };
-
-  handleToggleLogLevel = (hiddenLogLevels: LogLevel[]) => {
-    const { exploreId } = this.props;
-    this.props.toggleLogLevelAction({
-      exploreId,
-      hiddenLogLevels,
-    });
-  };
-
   getLogRowContext = async (row: LogRowModel, options?: any): Promise<any> => {
-    const { datasourceInstance } = this.props;
+    const { datasourceInstance, logsQueries } = this.props;
 
-    if (datasourceInstance?.getLogRowContext) {
-      return datasourceInstance.getLogRowContext(row, options);
+    if (hasLogsContextSupport(datasourceInstance)) {
+      // we need to find the query, and we need to be very sure that
+      // it's a query from this datasource
+      const query = (logsQueries ?? []).find(
+        (q) => q.refId === row.dataFrame.refId && q.datasource != null && q.datasource.type === datasourceInstance.type
+      );
+      return datasourceInstance.getLogRowContext(row, options, query);
     }
 
     return [];
@@ -61,7 +61,7 @@ export class LogsContainer extends PureComponent<PropsFromRedux & LogsContainerP
   showContextToggle = (row?: LogRowModel): boolean => {
     const { datasourceInstance } = this.props;
 
-    if (datasourceInstance?.showContextToggle) {
+    if (hasLogsContextSupport(datasourceInstance)) {
       return datasourceInstance.showContextToggle(row);
     }
 
@@ -76,11 +76,11 @@ export class LogsContainer extends PureComponent<PropsFromRedux & LogsContainerP
   render() {
     const {
       loading,
-      logsHighlighterExpressions,
+      loadingState,
       logRows,
       logsMeta,
       logsSeries,
-      dedupedRows,
+      logsQueries,
       onClickFilterLabel,
       onClickFilterOutLabel,
       onStartScanning,
@@ -93,12 +93,24 @@ export class LogsContainer extends PureComponent<PropsFromRedux & LogsContainerP
       width,
       isLive,
       exploreId,
+      addResultsToCache,
+      clearCache,
     } = this.props;
 
     if (!logRows) {
       return null;
     }
 
+    // We need to override css overflow of divs in Collapse element to enable sticky Logs navigation
+    const styleOverridesForStickyNavigation = css`
+      & > div {
+        overflow: visible;
+        & > div {
+          overflow: visible;
+        }
+      }
+    `;
+
     return (
       <>
         <LogsCrossFadeTransition visible={isLive}>
@@ -118,31 +130,32 @@ export class LogsContainer extends PureComponent<PropsFromRedux & LogsContainerP
           </Collapse>
         </LogsCrossFadeTransition>
         <LogsCrossFadeTransition visible={!isLive}>
-          <Collapse label="Logs" loading={loading} isOpen>
+          <Collapse label="Logs" loading={loading} isOpen className={styleOverridesForStickyNavigation}>
             <Logs
-              dedupStrategy={this.props.dedupStrategy || LogsDedupStrategy.none}
+              exploreId={exploreId}
+              datasourceType={this.props.datasourceInstance?.type}
               logRows={logRows}
               logsMeta={logsMeta}
               logsSeries={logsSeries}
-              dedupedRows={dedupedRows}
-              highlighterExpressions={logsHighlighterExpressions}
+              logsQueries={logsQueries}
+              width={width}
               loading={loading}
+              loadingState={loadingState}
               onChangeTime={this.onChangeTime}
               onClickFilterLabel={onClickFilterLabel}
               onClickFilterOutLabel={onClickFilterOutLabel}
               onStartScanning={onStartScanning}
               onStopScanning={onStopScanning}
-              onDedupStrategyChange={this.handleDedupStrategyChange}
-              onToggleLogLevel={this.handleToggleLogLevel}
               absoluteRange={absoluteRange}
               visibleRange={visibleRange}
               timeZone={timeZone}
               scanning={scanning}
               scanRange={range.raw}
               showContextToggle={this.showContextToggle}
-              width={width}
               getRowContext={this.getLogRowContext}
               getFieldLinks={this.getFieldLinks}
+              addResultsToCache={() => addResultsToCache(exploreId)}
+              clearCache={() => clearCache(exploreId)}
             />
           </Collapse>
         </LogsCrossFadeTransition>
@@ -156,7 +169,6 @@ function mapStateToProps(state: StoreState, { exploreId }: { exploreId: string }
   // @ts-ignore
   const item: ExploreItemState = explore[exploreId];
   const {
-    logsHighlighterExpressions,
     logsResult,
     loading,
     scanning,
@@ -165,38 +177,38 @@ function mapStateToProps(state: StoreState, { exploreId }: { exploreId: string }
     isPaused,
     range,
     absoluteRange,
-    dedupStrategy,
+    logsVolumeDataProvider,
+    logsVolumeData,
   } = item;
-  const dedupedRows = deduplicatedRowsSelector(item) || undefined;
   const timeZone = getTimeZone(state.user);
 
   return {
     loading,
-    logsHighlighterExpressions,
     logRows: logsResult?.rows,
     logsMeta: logsResult?.meta,
     logsSeries: logsResult?.series,
+    logsQueries: logsResult?.queries,
     visibleRange: logsResult?.visibleRange,
     scanning,
     timeZone,
-    dedupStrategy,
-    dedupedRows,
     datasourceInstance,
     isLive,
     isPaused,
     range,
     absoluteRange,
+    logsVolumeDataProvider,
+    logsVolumeData,
   };
 }
 
 const mapDispatchToProps = {
-  changeDedupStrategy,
-  toggleLogLevelAction,
   updateTimeRange,
   splitOpen,
+  addResultsToCache,
+  clearCache,
 };
 
 const connector = connect(mapStateToProps, mapDispatchToProps);
 type PropsFromRedux = ConnectedProps<typeof connector>;
 
-export default hot(module)(connector(LogsContainer));
+export default connector(LogsContainer);
