@@ -1,10 +1,19 @@
-import { act, renderHook } from '@testing-library/react-hooks';
-import { useDetailState } from './useDetailState';
 import { TraceLog } from '@jaegertracing/jaeger-ui-components/src/types/trace';
+import { act, renderHook } from '@testing-library/react-hooks';
+
+import { DataFrame } from '@grafana/data';
+
+import { useDetailState } from './useDetailState';
+
+const sampleFrame: DataFrame = {
+  name: 'trace',
+  fields: [],
+  length: 0,
+};
 
 describe('useDetailState', () => {
   it('toggles detail', async () => {
-    const { result } = renderHook(() => useDetailState());
+    const { result } = renderHook(() => useDetailState(sampleFrame));
     expect(result.current.detailStates.size).toBe(0);
 
     act(() => result.current.toggleDetail('span1'));
@@ -16,7 +25,7 @@ describe('useDetailState', () => {
   });
 
   it('toggles logs and logs items', async () => {
-    const { result } = renderHook(() => useDetailState());
+    const { result } = renderHook(() => useDetailState(sampleFrame));
     act(() => result.current.toggleDetail('span1'));
     act(() => result.current.detailLogsToggle('span1'));
     expect(result.current.detailStates.get('span1')?.logs.isOpen).toBe(true);
@@ -27,28 +36,28 @@ describe('useDetailState', () => {
   });
 
   it('toggles warnings', async () => {
-    const { result } = renderHook(() => useDetailState());
+    const { result } = renderHook(() => useDetailState(sampleFrame));
     act(() => result.current.toggleDetail('span1'));
     act(() => result.current.detailWarningsToggle('span1'));
     expect(result.current.detailStates.get('span1')?.isWarningsOpen).toBe(true);
   });
 
   it('toggles references', async () => {
-    const { result } = renderHook(() => useDetailState());
+    const { result } = renderHook(() => useDetailState(sampleFrame));
     act(() => result.current.toggleDetail('span1'));
     act(() => result.current.detailReferencesToggle('span1'));
-    expect(result.current.detailStates.get('span1')?.isReferencesOpen).toBe(true);
+    expect(result.current.detailStates.get('span1')?.references.isOpen).toBe(true);
   });
 
   it('toggles processes', async () => {
-    const { result } = renderHook(() => useDetailState());
+    const { result } = renderHook(() => useDetailState(sampleFrame));
     act(() => result.current.toggleDetail('span1'));
     act(() => result.current.detailProcessToggle('span1'));
     expect(result.current.detailStates.get('span1')?.isProcessOpen).toBe(true);
   });
 
   it('toggles tags', async () => {
-    const { result } = renderHook(() => useDetailState());
+    const { result } = renderHook(() => useDetailState(sampleFrame));
     act(() => result.current.toggleDetail('span1'));
     act(() => result.current.detailTagsToggle('span1'));
     expect(result.current.detailStates.get('span1')?.isTagsOpen).toBe(true);
