@@ -1,20 +1,23 @@
-import { serializeStateToUrlParam } from '@grafana/data/src/utils/url';
-import { exploreReducer, navigateToExplore, splitCloseAction } from './main';
 import { thunkTester } from 'test/core/thunk/thunkTester';
+
+import { ExploreUrlState } from '@grafana/data';
+import { serializeStateToUrlParam } from '@grafana/data/src/utils/url';
+import { locationService } from '@grafana/runtime';
 import { PanelModel } from 'app/features/dashboard/state';
-import { updateLocation } from '../../../core/actions';
+
+import { reducerTester } from '../../../../test/core/redux/reducerTester';
 import { MockDataSourceApi } from '../../../../test/mocks/datasource_srv';
 import { ExploreId, ExploreItemState, ExploreState } from '../../../types';
-import { reducerTester } from '../../../../test/core/redux/reducerTester';
-import { ExploreUrlState } from '@grafana/data';
+
+import { exploreReducer, navigateToExplore, splitCloseAction } from './main';
 
 const getNavigateToExploreContext = async (openInNewWindow?: (url: string) => void) => {
-  const url = 'http://www.someurl.com';
+  const url = '/explore';
   const panel: Partial<PanelModel> = {
-    datasource: 'mocked datasource',
+    datasource: { uid: 'mocked datasource' },
     targets: [{ refId: 'A' }],
   };
-  const datasource = new MockDataSourceApi(panel.datasource!);
+  const datasource = new MockDataSourceApi(panel.datasource!.uid!);
   const get = jest.fn().mockResolvedValue(datasource);
   const getDataSourceSrv = jest.fn().mockReturnValue({ get });
   const getTimeSrv = jest.fn();
@@ -27,7 +30,6 @@ const getNavigateToExploreContext = async (openInNewWindow?: (url: string) => vo
   return {
     url,
     panel,
-    datasource,
     get,
     getDataSourceSrv,
     getTimeSrv,
@@ -39,11 +41,10 @@ const getNavigateToExploreContext = async (openInNewWindow?: (url: string) => vo
 describe('navigateToExplore', () => {
   describe('when navigateToExplore thunk is dispatched', () => {
     describe('and openInNewWindow is undefined', () => {
-      const openInNewWindow: (url: string) => void = (undefined as unknown) as (url: string) => void;
+      const openInNewWindow: (url: string) => void = undefined as unknown as (url: string) => void;
       it('then it should dispatch correct actions', async () => {
-        const { dispatchedActions, url } = await getNavigateToExploreContext(openInNewWindow);
-
-        expect(dispatchedActions).toEqual([updateLocation({ path: url, query: {} })]);
+        const { url } = await getNavigateToExploreContext(openInNewWindow);
+        expect(locationService.getLocation().pathname).toEqual(url);
       });
 
       it('then getDataSourceSrv should have been once', async () => {
@@ -52,13 +53,6 @@ describe('navigateToExplore', () => {
         expect(getDataSourceSrv).toHaveBeenCalledTimes(1);
       });
 
-      it('then getDataSourceSrv.get should have been called with correct arguments', async () => {
-        const { get, panel } = await getNavigateToExploreContext(openInNewWindow);
-
-        expect(get).toHaveBeenCalledTimes(1);
-        expect(get).toHaveBeenCalledWith(panel.datasource);
-      });
-
       it('then getTimeSrv should have been called once', async () => {
         const { getTimeSrv } = await getNavigateToExploreContext(openInNewWindow);
 
@@ -66,15 +60,13 @@ describe('navigateToExplore', () => {
       });
 
       it('then getExploreUrl should have been called with correct arguments', async () => {
-        const { getExploreUrl, panel, datasource, getDataSourceSrv, getTimeSrv } = await getNavigateToExploreContext(
+        const { getExploreUrl, panel, getDataSourceSrv, getTimeSrv } = await getNavigateToExploreContext(
           openInNewWindow
         );
 
         expect(getExploreUrl).toHaveBeenCalledTimes(1);
         expect(getExploreUrl).toHaveBeenCalledWith({
           panel,
-          panelTargets: panel.targets,
-          panelDatasource: datasource,
           datasourceSrv: getDataSourceSrv(),
           timeSrv: getTimeSrv(),
         });
@@ -95,13 +87,6 @@ describe('navigateToExplore', () => {
         expect(getDataSourceSrv).toHaveBeenCalledTimes(1);
       });
 
-      it('then getDataSourceSrv.get should have been called with correct arguments', async () => {
-        const { get, panel } = await getNavigateToExploreContext(openInNewWindow);
-
-        expect(get).toHaveBeenCalledTimes(1);
-        expect(get).toHaveBeenCalledWith(panel.datasource);
-      });
-
       it('then getTimeSrv should have been called once', async () => {
         const { getTimeSrv } = await getNavigateToExploreContext(openInNewWindow);
 
@@ -109,15 +94,13 @@ describe('navigateToExplore', () => {
       });
 
       it('then getExploreUrl should have been called with correct arguments', async () => {
-        const { getExploreUrl, panel, datasource, getDataSourceSrv, getTimeSrv } = await getNavigateToExploreContext(
+        const { getExploreUrl, panel, getDataSourceSrv, getTimeSrv } = await getNavigateToExploreContext(
           openInNewWindow
         );
 
         expect(getExploreUrl).toHaveBeenCalledTimes(1);
         expect(getExploreUrl).toHaveBeenCalledWith({
           panel,
-          panelTargets: panel.targets,
-          panelDatasource: datasource,
           datasourceSrv: getDataSourceSrv(),
           timeSrv: getTimeSrv(),
         });
@@ -138,50 +121,50 @@ describe('Explore reducer', () => {
   describe('split view', () => {
     describe('split close', () => {
       it('should keep right pane as left when left is closed', () => {
-        const leftItemMock = ({
+        const leftItemMock = {
           containerWidth: 100,
-        } as unknown) as ExploreItemState;
+        } as unknown as ExploreItemState;
 
-        const rightItemMock = ({
+        const rightItemMock = {
           containerWidth: 200,
-        } as unknown) as ExploreItemState;
+        } as unknown as ExploreItemState;
 
-        const initialState = ({
+        const initialState = {
           left: leftItemMock,
           right: rightItemMock,
-        } as unknown) as ExploreState;
+        } as unknown as ExploreState;
 
         // closing left item
         reducerTester<ExploreState>()
           .givenReducer(exploreReducer, initialState)
           .whenActionIsDispatched(splitCloseAction({ itemId: ExploreId.left }))
-          .thenStateShouldEqual(({
+          .thenStateShouldEqual({
             left: rightItemMock,
             right: undefined,
-          } as unknown) as ExploreState);
+          } as unknown as ExploreState);
       });
       it('should reset right pane when it is closed ', () => {
-        const leftItemMock = ({
+        const leftItemMock = {
           containerWidth: 100,
-        } as unknown) as ExploreItemState;
+        } as unknown as ExploreItemState;
 
-        const rightItemMock = ({
+        const rightItemMock = {
           containerWidth: 200,
-        } as unknown) as ExploreItemState;
+        } as unknown as ExploreItemState;
 
-        const initialState = ({
+        const initialState = {
           left: leftItemMock,
           right: rightItemMock,
-        } as unknown) as ExploreState;
+        } as unknown as ExploreState;
 
         // closing left item
         reducerTester<ExploreState>()
           .givenReducer(exploreReducer, initialState)
           .whenActionIsDispatched(splitCloseAction({ itemId: ExploreId.right }))
-          .thenStateShouldEqual(({
+          .thenStateShouldEqual({
             left: leftItemMock,
             right: undefined,
-          } as unknown) as ExploreState);
+          } as unknown as ExploreState);
       });
     });
   });
@@ -198,9 +181,9 @@ export const setup = (urlStateOverrides?: any) => {
   };
   const urlState: ExploreUrlState = { ...urlStateDefaults, ...urlStateOverrides };
   const serializedUrlState = serializeStateToUrlParam(urlState);
-  const initialState = ({
+  const initialState = {
     split: false,
-  } as unknown) as ExploreState;
+  } as unknown as ExploreState;
 
   return {
     initialState,
