@@ -1,7 +1,16 @@
 import { FolderDTO, FolderState, OrgRole, PermissionLevel } from 'app/types';
-import { folderReducer, initialState, loadFolder, loadFolderPermissions, setFolderTitle } from './reducers';
+
 import { reducerTester } from '../../../../test/core/redux/reducerTester';
 
+import {
+  folderReducer,
+  initialState,
+  loadFolder,
+  loadFolderPermissions,
+  setCanViewFolderPermissions,
+  setFolderTitle,
+} from './reducers';
+
 function getTestFolder(): FolderDTO {
   return {
     id: 1,
@@ -9,6 +18,9 @@ function getTestFolder(): FolderDTO {
     uid: 'asd',
     url: 'url',
     canSave: true,
+    canEdit: true,
+    canAdmin: true,
+    canDelete: true,
     version: 0,
   };
 }
@@ -140,4 +152,16 @@ describe('folder reducer', () => {
         });
     });
   });
+
+  describe('setCanViewFolderPermissions', () => {
+    it('should set the canViewFolderPermissions value', () => {
+      reducerTester<FolderState>()
+        .givenReducer(folderReducer, { ...initialState })
+        .whenActionIsDispatched(setCanViewFolderPermissions(true))
+        .thenStateShouldEqual({
+          ...initialState,
+          canViewFolderPermissions: true,
+        });
+    });
+  });
 });
