@@ -1,7 +1,8 @@
+import { css } from '@emotion/css';
 import React, { FC } from 'react';
-import { useStyles } from '@grafana/ui';
+
 import { GrafanaTheme } from '@grafana/data';
-import { css } from 'emotion';
+import { useStyles } from '@grafana/ui';
 
 const getStyles = (theme: GrafanaTheme) => css`
   margin: 0;
