@@ -1,14 +1,35 @@
-import React from 'react';
 import { shallow } from 'enzyme';
+import React from 'react';
+import { mockToolkitActionCreator } from 'test/core/redux/mocks';
+
+import { NavModel } from '@grafana/data';
+
+import { Organization } from '../../types';
+
 import { OrgDetailsPage, Props } from './OrgDetailsPage';
-import { NavModel, Organization } from '../../types';
+import { setOrganizationName } from './state/reducers';
+
+jest.mock('app/core/core', () => {
+  return {
+    contextSrv: {
+      hasPermission: () => true,
+    },
+  };
+});
 
 const setup = (propOverrides?: object) => {
   const props: Props = {
     organization: {} as Organization,
-    navModel: {} as NavModel,
+    navModel: {
+      main: {
+        text: 'Configuration',
+      },
+      node: {
+        text: 'Org details',
+      },
+    } as NavModel,
     loadOrganization: jest.fn(),
-    setOrganizationName: jest.fn(),
+    setOrganizationName: mockToolkitActionCreator(setOrganizationName),
     updateOrganization: jest.fn(),
   };
 
