@@ -1,44 +1,58 @@
-import React, { ChangeEvent, FormEvent, PureComponent } from 'react';
-import { css } from 'emotion';
-import { MapDispatchToProps, MapStateToProps } from 'react-redux';
+import { css } from '@emotion/css';
+import React, { FormEvent, PureComponent } from 'react';
+import { connect, ConnectedProps } from 'react-redux';
+
+import { DataSourceInstanceSettings, getDataSourceRef, LoadingState, SelectableValue } from '@grafana/data';
+import { selectors } from '@grafana/e2e-selectors';
+import { DataSourcePicker, getTemplateSrv } from '@grafana/runtime';
 import { InlineField, InlineFieldRow, VerticalGroup } from '@grafana/ui';
-import { selectors } from '@grafana/e2e-selectors';
-import { getTemplateSrv } from '@grafana/runtime';
-import { DataSourceInstanceSettings, LoadingState, SelectableValue } from '@grafana/data';
 
-import { SelectionOptionsEditor } from '../editor/SelectionOptionsEditor';
-import { QueryVariableModel, VariableRefresh, VariableSort, VariableWithMultiSupport } from '../types';
-import { QueryVariableEditorState } from './reducer';
-import { changeQueryVariableDataSource, changeQueryVariableQuery, initQueryVariableEditor } from './actions';
-import { VariableEditorState } from '../editor/reducer';
-import { OnPropChangeArguments, VariableEditorProps } from '../editor/types';
 import { StoreState } from '../../../types';
-import { connectWithStore } from '../../../core/utils/connectWithReduxStore';
-import { toVariableIdentifier } from '../state/types';
-import { changeVariableMultiValue } from '../state/actions';
 import { getTimeSrv } from '../../dashboard/services/TimeSrv';
-import { isLegacyQueryEditor, isQueryEditor } from '../guard';
+import { SelectionOptionsEditor } from '../editor/SelectionOptionsEditor';
 import { VariableSectionHeader } from '../editor/VariableSectionHeader';
 import { VariableTextField } from '../editor/VariableTextField';
-import { VariableSwitchField } from '../editor/VariableSwitchField';
+import { initialVariableEditorState } from '../editor/reducer';
+import { getQueryVariableEditorState } from '../editor/selectors';
+import { OnPropChangeArguments, VariableEditorProps } from '../editor/types';
+import { isLegacyQueryEditor, isQueryEditor } from '../guard';
+import { changeVariableMultiValue } from '../state/actions';
+import { getVariablesState } from '../state/selectors';
+import { QueryVariableModel, VariableRefresh, VariableSort, VariableWithMultiSupport } from '../types';
+import { toKeyedVariableIdentifier } from '../utils';
+
 import { QueryVariableRefreshSelect } from './QueryVariableRefreshSelect';
 import { QueryVariableSortSelect } from './QueryVariableSortSelect';
-import { DataSourcePicker } from 'app/core/components/Select/DataSourcePicker';
+import { changeQueryVariableDataSource, changeQueryVariableQuery, initQueryVariableEditor } from './actions';
+
+const mapStateToProps = (state: StoreState, ownProps: OwnProps) => {
+  const { rootStateKey } = ownProps.variable;
+  if (!rootStateKey) {
+    console.error('QueryVariableEditor: variable has no rootStateKey');
+    return {
+      extended: getQueryVariableEditorState(initialVariableEditorState),
+    };
+  }
+
+  const { editor } = getVariablesState(rootStateKey, state);
+
+  return {
+    extended: getQueryVariableEditorState(editor),
+  };
+};
+
+const mapDispatchToProps = {
+  initQueryVariableEditor,
+  changeQueryVariableDataSource,
+  changeQueryVariableQuery,
+  changeVariableMultiValue,
+};
+
+const connector = connect(mapStateToProps, mapDispatchToProps);
 
 export interface OwnProps extends VariableEditorProps<QueryVariableModel> {}
 
-interface ConnectedProps {
-  editor: VariableEditorState<QueryVariableEditorState>;
-}
-
-interface DispatchProps {
-  initQueryVariableEditor: typeof initQueryVariableEditor;
-  changeQueryVariableDataSource: typeof changeQueryVariableDataSource;
-  changeQueryVariableQuery: typeof changeQueryVariableQuery;
-  changeVariableMultiValue: typeof changeVariableMultiValue;
-}
-
-export type Props = OwnProps & ConnectedProps & DispatchProps;
+export type Props = OwnProps & ConnectedProps<typeof connector>;
 
 export interface State {
   regex: string | null;
@@ -54,29 +68,28 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
   };
 
   async componentDidMount() {
-    await this.props.initQueryVariableEditor(toVariableIdentifier(this.props.variable));
+    await this.props.initQueryVariableEditor(toKeyedVariableIdentifier(this.props.variable));
   }
 
   componentDidUpdate(prevProps: Readonly<Props>): void {
     if (prevProps.variable.datasource !== this.props.variable.datasource) {
       this.props.changeQueryVariableDataSource(
-        toVariableIdentifier(this.props.variable),
+        toKeyedVariableIdentifier(this.props.variable),
         this.props.variable.datasource
       );
     }
   }
 
   onDataSourceChange = (dsSettings: DataSourceInstanceSettings) => {
-    this.props.onPropChange({ propName: 'query', propValue: '' });
     this.props.onPropChange({
       propName: 'datasource',
-      propValue: dsSettings.isDefault ? null : dsSettings.name,
+      propValue: dsSettings.isDefault ? null : getDataSourceRef(dsSettings),
     });
   };
 
   onLegacyQueryChange = async (query: any, definition: string) => {
     if (this.props.variable.query !== query) {
-      this.props.changeQueryVariableQuery(toVariableIdentifier(this.props.variable), query, definition);
+      this.props.changeQueryVariableQuery(toKeyedVariableIdentifier(this.props.variable), query, definition);
     }
   };
 
@@ -88,7 +101,7 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
         definition = query.query;
       }
 
-      this.props.changeQueryVariableQuery(toVariableIdentifier(this.props.variable), query, definition);
+      this.props.changeQueryVariableQuery(toKeyedVariableIdentifier(this.props.variable), query, definition);
     }
   };
 
@@ -103,28 +116,6 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
     }
   };
 
-  onTagsQueryChange = async (event: FormEvent<HTMLInputElement>) => {
-    this.setState({ tagsQuery: event.currentTarget.value });
-  };
-
-  onTagsQueryBlur = async (event: FormEvent<HTMLInputElement>) => {
-    const tagsQuery = event.currentTarget.value;
-    if (this.props.variable.tagsQuery !== tagsQuery) {
-      this.props.onPropChange({ propName: 'tagsQuery', propValue: tagsQuery, updateOptions: true });
-    }
-  };
-
-  onTagValuesQueryChange = async (event: FormEvent<HTMLInputElement>) => {
-    this.setState({ tagValuesQuery: event.currentTarget.value });
-  };
-
-  onTagValuesQueryBlur = async (event: FormEvent<HTMLInputElement>) => {
-    const tagValuesQuery = event.currentTarget.value;
-    if (this.props.variable.tagValuesQuery !== tagValuesQuery) {
-      this.props.onPropChange({ propName: 'tagValuesQuery', propValue: tagValuesQuery, updateOptions: true });
-    }
-  };
-
   onRefreshChange = (option: SelectableValue<VariableRefresh>) => {
     this.props.onPropChange({ propName: 'refresh', propValue: option.value });
   };
@@ -137,19 +128,16 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
     this.props.onPropChange({ propName, propValue, updateOptions: true });
   };
 
-  onUseTagsChange = async (event: ChangeEvent<HTMLInputElement>) => {
-    this.props.onPropChange({ propName: 'useTags', propValue: event.target.checked, updateOptions: true });
-  };
-
   renderQueryEditor = () => {
-    const { editor, variable } = this.props;
-    if (!editor.extended || !editor.extended.dataSource || !editor.extended.VariableQueryEditor) {
+    const { extended, variable } = this.props;
+
+    if (!extended || !extended.dataSource || !extended.VariableQueryEditor) {
       return null;
     }
 
     const query = variable.query;
-    const datasource = editor.extended.dataSource;
-    const VariableQueryEditor = editor.extended.VariableQueryEditor;
+    const datasource = extended.dataSource;
+    const VariableQueryEditor = extended.VariableQueryEditor;
 
     if (isLegacyQueryEditor(VariableQueryEditor, datasource)) {
       return (
@@ -189,7 +177,7 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
         <VerticalGroup spacing="lg">
           <VerticalGroup spacing="none">
             <InlineFieldRow>
-              <InlineField label="Data source" labelWidth={20}>
+              <InlineField label="Data source" labelWidth={20} htmlFor="data-source-picker">
                 <DataSourcePicker
                   current={this.props.variable.datasource}
                   onChange={this.onDataSourceChange}
@@ -213,11 +201,13 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
               onChange={this.onRegExChange}
               onBlur={this.onRegExBlur}
               labelWidth={20}
+              interactive={true}
               tooltip={
                 <div>
                   Optional, if you want to extract part of a series name or metric node segment. Named capture groups
                   can be used to separate the display text and value (
                   <a
+                    className="external-link"
                     href="https://grafana.com/docs/grafana/latest/variables/filter-variables-with-regex#filter-and-modify-using-named-text-and-value-capture-groups"
                     target="__blank"
                   >
@@ -226,7 +216,7 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
                   ).
                 </div>
               }
-              ariaLabel={selectors.pages.Dashboard.Settings.Variables.Edit.QueryVariable.queryOptionsRegExInput}
+              testId={selectors.pages.Dashboard.Settings.Variables.Edit.QueryVariable.queryOptionsRegExInputV2}
               grow
             />
             <QueryVariableSortSelect onChange={this.onSortChange} sort={this.props.variable.sort} />
@@ -237,65 +227,10 @@ export class QueryVariableEditorUnConnected extends PureComponent<Props, State>
             onPropChange={this.onSelectionOptionsChange}
             onMultiChanged={this.props.changeVariableMultiValue}
           />
-
-          <VerticalGroup spacing="none">
-            <h5>Value group tags</h5>
-            <em className="muted p-b-1">Experimental feature, will be deprecated in Grafana v8.</em>
-
-            <VariableSwitchField
-              value={this.props.variable.useTags}
-              name="Enabled"
-              onChange={this.onUseTagsChange}
-              ariaLabel={selectors.pages.Dashboard.Settings.Variables.Edit.QueryVariable.valueGroupsTagsEnabledSwitch}
-            />
-            {this.props.variable.useTags ? (
-              <VerticalGroup spacing="none">
-                <VariableTextField
-                  value={this.state.tagsQuery ?? this.props.variable.tagsQuery}
-                  name="Tags query"
-                  placeholder="metric name or tags query"
-                  onChange={this.onTagsQueryChange}
-                  onBlur={this.onTagsQueryBlur}
-                  ariaLabel={
-                    selectors.pages.Dashboard.Settings.Variables.Edit.QueryVariable.valueGroupsTagsTagsQueryInput
-                  }
-                  labelWidth={20}
-                  grow
-                />
-                <VariableTextField
-                  value={this.state.tagValuesQuery ?? this.props.variable.tagValuesQuery}
-                  name="Tag values query"
-                  placeholder="apps.$tag.*"
-                  onChange={this.onTagValuesQueryChange}
-                  onBlur={this.onTagValuesQueryBlur}
-                  ariaLabel={
-                    selectors.pages.Dashboard.Settings.Variables.Edit.QueryVariable.valueGroupsTagsTagsValuesQueryInput
-                  }
-                  labelWidth={20}
-                  grow
-                />
-              </VerticalGroup>
-            ) : null}
-          </VerticalGroup>
         </VerticalGroup>
       </VerticalGroup>
     );
   }
 }
 
-const mapStateToProps: MapStateToProps<ConnectedProps, OwnProps, StoreState> = (state, ownProps) => ({
-  editor: state.templating.editor as VariableEditorState<QueryVariableEditorState>,
-});
-
-const mapDispatchToProps: MapDispatchToProps<DispatchProps, OwnProps> = {
-  initQueryVariableEditor,
-  changeQueryVariableDataSource,
-  changeQueryVariableQuery,
-  changeVariableMultiValue,
-};
-
-export const QueryVariableEditor = connectWithStore(
-  QueryVariableEditorUnConnected,
-  mapStateToProps,
-  mapDispatchToProps
-);
+export const QueryVariableEditor = connector(QueryVariableEditorUnConnected);
