@@ -1,34 +1,41 @@
 import { dateTime, TimeRange } from '@grafana/data';
 
-import { TemplateSrv } from '../../templating/template_srv';
-import { onTimeRangeUpdated, OnTimeRangeUpdatedDependencies, setOptionAsCurrent } from './actions';
-import { DashboardModel } from '../../dashboard/state';
+import { reduxTester } from '../../../../test/core/redux/reduxTester';
+import { silenceConsoleOutput } from '../../../../test/core/utils/silenceConsoleOutput';
+import { expect } from '../../../../test/lib/common';
+import { appEvents } from '../../../core/core';
+import { notifyApp } from '../../../core/reducers/appNotification';
 import { DashboardState } from '../../../types';
-import { createIntervalVariableAdapter } from '../interval/adapter';
+import { DashboardModel } from '../../dashboard/state';
+import { TemplateSrv } from '../../templating/template_srv';
 import { variableAdapters } from '../adapters';
 import { createConstantVariableAdapter } from '../constant/adapter';
-import { VariableRefresh } from '../types';
+import { createIntervalVariableAdapter } from '../interval/adapter';
+import { createIntervalOptions } from '../interval/reducer';
 import { constantBuilder, intervalBuilder } from '../shared/testing/builders';
-import { reduxTester } from '../../../../test/core/redux/reduxTester';
-import { TemplatingState } from './reducers';
-import { getRootReducer } from './helpers';
-import { toVariableIdentifier, toVariablePayload } from './types';
+import { VariableRefresh } from '../types';
+import { toKeyedVariableIdentifier, toVariablePayload } from '../utils';
+
+import { onTimeRangeUpdated, OnTimeRangeUpdatedDependencies, setOptionAsCurrent } from './actions';
+import { getPreloadedState, getRootReducer, RootReducerType } from './helpers';
+import { toKeyedAction } from './keyedVariablesReducer';
 import {
   setCurrentVariableValue,
   variableStateCompleted,
   variableStateFailed,
   variableStateFetching,
 } from './sharedReducer';
-import { createIntervalOptions } from '../interval/reducer';
-import { silenceConsoleOutput } from '../../../../test/core/utils/silenceConsoleOutput';
-import { notifyApp } from '../../../core/reducers/appNotification';
-import { expect } from '../../../../test/lib/common';
+import { variablesInitTransaction } from './transactionReducer';
 
 variableAdapters.setInit(() => [createIntervalVariableAdapter(), createConstantVariableAdapter()]);
 
-const getTestContext = () => {
+const getTestContext = (dashboard: DashboardModel) => {
+  jest.clearAllMocks();
+
+  const key = 'key';
   const interval = intervalBuilder()
     .withId('interval-0')
+    .withRootStateKey(key)
     .withName('interval-0')
     .withOptions('1m', '10m', '30m', '1h', '6h', '12h', '1d', '7d', '14d', '30d')
     .withCurrent('1m')
@@ -37,6 +44,7 @@ const getTestContext = () => {
 
   const constant = constantBuilder()
     .withId('constant-1')
+    .withRootStateKey(key)
     .withName('constant-1')
     .withOptions('a constant')
     .withCurrent('a constant')
@@ -51,30 +59,29 @@ const getTestContext = () => {
     },
   };
   const updateTimeRangeMock = jest.fn();
-  const templateSrvMock = ({ updateTimeRange: updateTimeRangeMock } as unknown) as TemplateSrv;
-  const dependencies: OnTimeRangeUpdatedDependencies = { templateSrv: templateSrvMock };
+  const templateSrvMock = { updateTimeRange: updateTimeRangeMock } as unknown as TemplateSrv;
+  const dependencies: OnTimeRangeUpdatedDependencies = { templateSrv: templateSrvMock, events: appEvents };
   const templateVariableValueUpdatedMock = jest.fn();
-  const dashboard = ({
-    getModel: () =>
-      (({
-        templateVariableValueUpdated: templateVariableValueUpdatedMock,
-        startRefresh: startRefreshMock,
-      } as unknown) as DashboardModel),
-  } as unknown) as DashboardState;
   const startRefreshMock = jest.fn();
+  dashboard.templateVariableValueUpdated = templateVariableValueUpdatedMock;
+  dashboard.startRefresh = startRefreshMock;
+  const dashboardState = {
+    getModel: () => dashboard,
+  } as unknown as DashboardState;
   const adapter = variableAdapters.get('interval');
-  const preloadedState = {
-    dashboard,
-    location: { query: '' },
-    templating: ({
-      variables: {
-        'interval-0': { ...interval },
-        'constant-1': { ...constant },
-      },
-    } as unknown) as TemplatingState,
+  const templatingState = {
+    variables: {
+      'interval-0': { ...interval },
+      'constant-1': { ...constant },
+    },
   };
+  const preloadedState = {
+    dashboard: dashboardState,
+    ...getPreloadedState(key, templatingState),
+  } as unknown as RootReducerType;
 
   return {
+    key,
     interval,
     range,
     dependencies,
@@ -90,28 +97,34 @@ describe('when onTimeRangeUpdated is dispatched', () => {
   describe('and options are changed by update', () => {
     it('then correct actions are dispatched and correct dependencies are called', async () => {
       const {
+        key,
         preloadedState,
         range,
         dependencies,
         updateTimeRangeMock,
         templateVariableValueUpdatedMock,
         startRefreshMock,
-      } = getTestContext();
+      } = getTestContext(getDashboardModel());
 
-      const tester = await reduxTester<{ templating: TemplatingState }>({ preloadedState })
+      const tester = await reduxTester<RootReducerType>({ preloadedState })
         .givenRootReducer(getRootReducer())
-        .whenAsyncActionIsDispatched(onTimeRangeUpdated(range, dependencies));
+        .whenActionIsDispatched(toKeyedAction(key, variablesInitTransaction({ uid: key })))
+        .whenAsyncActionIsDispatched(onTimeRangeUpdated(key, range, dependencies));
 
       tester.thenDispatchedActionsShouldEqual(
-        variableStateFetching(toVariablePayload({ type: 'interval', id: 'interval-0' })),
-        createIntervalOptions(toVariablePayload({ type: 'interval', id: 'interval-0' })),
-        setCurrentVariableValue(
-          toVariablePayload(
-            { type: 'interval', id: 'interval-0' },
-            { option: { text: '1m', value: '1m', selected: false } }
+        toKeyedAction(key, variablesInitTransaction({ uid: key })),
+        toKeyedAction(key, variableStateFetching(toVariablePayload({ type: 'interval', id: 'interval-0' }))),
+        toKeyedAction(key, createIntervalOptions(toVariablePayload({ type: 'interval', id: 'interval-0' }))),
+        toKeyedAction(
+          key,
+          setCurrentVariableValue(
+            toVariablePayload(
+              { type: 'interval', id: 'interval-0' },
+              { option: { text: '1m', value: '1m', selected: false } }
+            )
           )
         ),
-        variableStateCompleted(toVariablePayload({ type: 'interval', id: 'interval-0' }))
+        toKeyedAction(key, variableStateCompleted(toVariablePayload({ type: 'interval', id: 'interval-0' })))
       );
 
       expect(updateTimeRangeMock).toHaveBeenCalledTimes(1);
@@ -124,6 +137,7 @@ describe('when onTimeRangeUpdated is dispatched', () => {
   describe('and options are not changed by update', () => {
     it('then correct actions are dispatched and correct dependencies are called', async () => {
       const {
+        key,
         interval,
         preloadedState,
         range,
@@ -131,24 +145,30 @@ describe('when onTimeRangeUpdated is dispatched', () => {
         updateTimeRangeMock,
         templateVariableValueUpdatedMock,
         startRefreshMock,
-      } = getTestContext();
+      } = getTestContext(getDashboardModel());
 
-      const base = await reduxTester<{ templating: TemplatingState }>({ preloadedState })
+      const base = await reduxTester<RootReducerType>({ preloadedState })
         .givenRootReducer(getRootReducer())
-        .whenAsyncActionIsDispatched(setOptionAsCurrent(toVariableIdentifier(interval), interval.options[0], false));
+        .whenActionIsDispatched(toKeyedAction(key, variablesInitTransaction({ uid: key })))
+        .whenAsyncActionIsDispatched(
+          setOptionAsCurrent(toKeyedVariableIdentifier(interval), interval.options[0], false)
+        );
 
-      const tester = await base.whenAsyncActionIsDispatched(onTimeRangeUpdated(range, dependencies), true);
+      const tester = await base.whenAsyncActionIsDispatched(onTimeRangeUpdated(key, range, dependencies), true);
 
       tester.thenDispatchedActionsShouldEqual(
-        variableStateFetching(toVariablePayload({ type: 'interval', id: 'interval-0' })),
-        createIntervalOptions(toVariablePayload({ type: 'interval', id: 'interval-0' })),
-        setCurrentVariableValue(
-          toVariablePayload(
-            { type: 'interval', id: 'interval-0' },
-            { option: { text: '1m', value: '1m', selected: false } }
+        toKeyedAction(key, variableStateFetching(toVariablePayload({ type: 'interval', id: 'interval-0' }))),
+        toKeyedAction(key, createIntervalOptions(toVariablePayload({ type: 'interval', id: 'interval-0' }))),
+        toKeyedAction(
+          key,
+          setCurrentVariableValue(
+            toVariablePayload(
+              { type: 'interval', id: 'interval-0' },
+              { option: { text: '1m', value: '1m', selected: false } }
+            )
           )
         ),
-        variableStateCompleted(toVariablePayload({ type: 'interval', id: 'interval-0' }))
+        toKeyedAction(key, variableStateCompleted(toVariablePayload({ type: 'interval', id: 'interval-0' })))
       );
 
       expect(updateTimeRangeMock).toHaveBeenCalledTimes(1);
@@ -162,6 +182,7 @@ describe('when onTimeRangeUpdated is dispatched', () => {
     silenceConsoleOutput();
     it('then correct actions are dispatched and correct dependencies are called', async () => {
       const {
+        key,
         adapter,
         preloadedState,
         range,
@@ -169,21 +190,25 @@ describe('when onTimeRangeUpdated is dispatched', () => {
         updateTimeRangeMock,
         templateVariableValueUpdatedMock,
         startRefreshMock,
-      } = getTestContext();
+      } = getTestContext(getDashboardModel());
 
       adapter.updateOptions = jest.fn().mockRejectedValue(new Error('Something broke'));
 
-      const tester = await reduxTester<{ templating: TemplatingState }>({ preloadedState, debug: true })
+      const tester = await reduxTester<RootReducerType>({ preloadedState, debug: true })
         .givenRootReducer(getRootReducer())
-        .whenAsyncActionIsDispatched(onTimeRangeUpdated(range, dependencies), true);
+        .whenActionIsDispatched(toKeyedAction(key, variablesInitTransaction({ uid: key })))
+        .whenAsyncActionIsDispatched(onTimeRangeUpdated(key, range, dependencies), true);
 
       tester.thenDispatchedActionsPredicateShouldEqual((dispatchedActions) => {
         expect(dispatchedActions[0]).toEqual(
-          variableStateFetching(toVariablePayload({ type: 'interval', id: 'interval-0' }))
+          toKeyedAction(key, variableStateFetching(toVariablePayload({ type: 'interval', id: 'interval-0' })))
         );
         expect(dispatchedActions[1]).toEqual(
-          variableStateFailed(
-            toVariablePayload({ type: 'interval', id: 'interval-0' }, { error: new Error('Something broke') })
+          toKeyedAction(
+            key,
+            variableStateFailed(
+              toVariablePayload({ type: 'interval', id: 'interval-0' }, { error: new Error('Something broke') })
+            )
           )
         );
         expect(dispatchedActions[2].type).toEqual(notifyApp.type);
@@ -200,3 +225,7 @@ describe('when onTimeRangeUpdated is dispatched', () => {
     });
   });
 });
+
+function getDashboardModel(): DashboardModel {
+  return new DashboardModel({ schemaVersion: 9999 }); // ignore any schema migrations
+}
