@@ -1,19 +1,15 @@
-import { createAction, PayloadAction } from '@reduxjs/toolkit';
+import { createAction } from '@reduxjs/toolkit';
+import { AnyAction } from 'redux';
+
 import { variableAdapters } from '../adapters';
-import { sharedReducer } from './sharedReducer';
 import { VariableModel } from '../types';
-import { VariablePayload } from './types';
 
-export interface VariablesState extends Record<string, VariableModel> {}
-
-export const initialVariablesState: VariablesState = {};
+import { sharedReducer } from './sharedReducer';
+import { initialVariablesState, VariablesState } from './types';
 
 export const cleanVariables = createAction<undefined>('templating/cleanVariables');
 
-export const variablesReducer = (
-  state: VariablesState = initialVariablesState,
-  action: PayloadAction<VariablePayload>
-): VariablesState => {
+export const variablesReducer = (state: VariablesState = initialVariablesState, action: AnyAction): VariablesState => {
   if (cleanVariables.match(action)) {
     const globalVariables = Object.values(state).filter((v) => v.global);
     if (!globalVariables) {
