@@ -1,11 +1,13 @@
-import React, { ComponentProps, useState } from 'react';
-import { InlineField, Input } from '@grafana/ui';
-import { useDispatch } from '../../../../hooks/useStatelessReducer';
-import { changeMetricSetting } from '../state/actions';
-import { ChangeMetricSettingAction } from '../state/types';
-import { SettingKeyOf } from '../../../types';
-import { MetricAggregationWithSettings } from '../aggregations';
 import { uniqueId } from 'lodash';
+import React, { ComponentProps, useState } from 'react';
+
+import { InlineField, Input } from '@grafana/ui';
+import { getScriptValue } from 'app/plugins/datasource/elasticsearch/utils';
+
+import { useDispatch } from '../../../../hooks/useStatelessReducer';
+import { SettingKeyOf } from '../../../types';
+import { MetricAggregationWithInlineScript, MetricAggregationWithSettings } from '../aggregations';
+import { changeMetricSetting } from '../state/actions';
 
 interface Props<T extends MetricAggregationWithSettings, K extends SettingKeyOf<T>> {
   label: string;
@@ -22,17 +24,23 @@ export function SettingField<T extends MetricAggregationWithSettings, K extends
   placeholder,
   tooltip,
 }: Props<T, K>) {
-  const dispatch = useDispatch<ChangeMetricSettingAction<T>>();
+  const dispatch = useDispatch();
   const [id] = useState(uniqueId(`es-field-id-`));
   const settings = metric.settings;
 
+  let defaultValue = settings?.[settingName as keyof typeof settings] || '';
+
+  if (settingName === 'script') {
+    defaultValue = getScriptValue(metric as MetricAggregationWithInlineScript);
+  }
+
   return (
     <InlineField label={label} labelWidth={16} tooltip={tooltip}>
       <Input
         id={id}
         placeholder={placeholder}
-        onBlur={(e) => dispatch(changeMetricSetting(metric, settingName, e.target.value as any))}
-        defaultValue={settings?.[settingName as keyof typeof settings]}
+        onBlur={(e) => dispatch(changeMetricSetting({ metric, settingName, newValue: e.target.value }))}
+        defaultValue={defaultValue}
       />
     </InlineField>
   );
