@@ -1,8 +1,10 @@
-import _ from 'lodash';
+import { map } from 'lodash';
 import { Observable, of, throwError } from 'rxjs';
+
 import {
   ArrayVector,
   CoreApp,
+  DataLink,
   DataQueryRequest,
   DataSourceInstanceSettings,
   DataSourcePluginMeta,
@@ -15,17 +17,18 @@ import {
   toUtc,
 } from '@grafana/data';
 import { BackendSrvRequest, FetchResponse } from '@grafana/runtime';
-
-import { ElasticDatasource, enhanceDataFrame } from './datasource';
 import { backendSrv } from 'app/core/services/backend_srv'; // will use the version in __mocks__
-import { ElasticsearchOptions, ElasticsearchQuery } from './types';
-import { Filters } from './components/QueryEditor/BucketAggregationsEditor/aggregations';
+
 import { createFetchResponse } from '../../../../test/helpers/createFetchResponse';
 
+import { Filters } from './components/QueryEditor/BucketAggregationsEditor/aggregations';
+import { ElasticDatasource, enhanceDataFrame } from './datasource';
+import { ElasticsearchOptions, ElasticsearchQuery } from './types';
+
 const ELASTICSEARCH_MOCK_URL = 'http://elasticsearch.local';
 
 jest.mock('@grafana/runtime', () => ({
-  ...((jest.requireActual('@grafana/runtime') as unknown) as object),
+  ...(jest.requireActual('@grafana/runtime') as unknown as object),
   getBackendSrv: () => backendSrv,
   getDataSourceSrv: () => {
     return {
@@ -68,8 +71,8 @@ function getTestContext({
   fetchMock.mockImplementation(mockImplementation ?? defaultMock);
 
   const templateSrv: any = {
-    replace: jest.fn((text) => {
-      if (text.startsWith('$')) {
+    replace: jest.fn((text?: string) => {
+      if (text?.startsWith('$')) {
         return `resolvedVariable`;
       } else {
         return text;
@@ -99,6 +102,7 @@ function getTestContext({
     name: 'test-elastic',
     type: 'type',
     uid: 'uid',
+    access: 'proxy',
     url: ELASTICSEARCH_MOCK_URL,
     database,
     jsonData,
@@ -112,7 +116,7 @@ function getTestContext({
 describe('ElasticDatasource', function (this: any) {
   describe('When testing datasource with index pattern', () => {
     it('should translate index pattern to current day', () => {
-      const { ds, fetchMock } = getTestContext({ jsonData: { interval: 'Daily', esVersion: 2 } });
+      const { ds, fetchMock } = getTestContext({ jsonData: { interval: 'Daily', esVersion: '7.10.0' } });
 
       ds.testDatasource();
 
@@ -150,7 +154,7 @@ describe('ElasticDatasource', function (this: any) {
           },
         ],
       };
-      const { ds, fetchMock } = getTestContext({ jsonData: { interval: 'Daily', esVersion: 2 }, data });
+      const { ds, fetchMock } = getTestContext({ jsonData: { interval: 'Daily', esVersion: '7.10.0' }, data });
 
       let result: any = {};
       await expect(ds.query(query)).toEmitValuesWith((received) => {
@@ -203,7 +207,7 @@ describe('ElasticDatasource', function (this: any) {
     async function setupDataSource(jsonData?: Partial<ElasticsearchOptions>) {
       jsonData = {
         interval: 'Daily',
-        esVersion: 2,
+        esVersion: '7.10.0',
         timeField: '@timestamp',
         ...(jsonData || {}),
       };
@@ -226,9 +230,8 @@ describe('ElasticDatasource', function (this: any) {
                 id: '2',
               },
             ],
-            metrics: [{ type: 'count', id: '1' }],
+            metrics: [{ type: 'logs', id: '1' }],
             query: 'escape\\:test',
-            isLogsQuery: true,
             timeField: '@timestamp',
           },
         ],
@@ -256,14 +259,16 @@ describe('ElasticDatasource', function (this: any) {
           {
             field: 'host',
             url: 'http://localhost:3000/${__value.raw}',
+            urlDisplayLabel: 'Custom Label',
           },
         ],
       });
 
       expect(response.data.length).toBe(1);
-      const links = response.data[0].fields.find((field: Field) => field.name === 'host').config.links;
+      const links: DataLink[] = response.data[0].fields.find((field: Field) => field.name === 'host').config.links;
       expect(links.length).toBe(1);
       expect(links[0].url).toBe('http://localhost:3000/${__value.raw}');
+      expect(links[0].title).toBe('Custom Label');
     });
   });
 
@@ -274,7 +279,7 @@ describe('ElasticDatasource', function (this: any) {
       const query: any = { range, targets };
       const data = { responses: [] };
 
-      const { ds, fetchMock } = getTestContext({ jsonData: { esVersion: 2 }, data, database: 'test' });
+      const { ds, fetchMock } = getTestContext({ jsonData: { esVersion: '7.10.0' }, data, database: 'test' });
 
       await expect(ds.query(query)).toEmitValuesWith((received) => {
         expect(received.length).toBe(1);
@@ -317,7 +322,7 @@ describe('ElasticDatasource', function (this: any) {
 
     it('should process it properly', async () => {
       const { ds } = getTestContext({
-        jsonData: { interval: 'Daily', esVersion: 7 },
+        jsonData: { interval: 'Daily', esVersion: '7.10.0' },
         data: {
           took: 1,
           responses: [
@@ -335,7 +340,7 @@ describe('ElasticDatasource', function (this: any) {
         data: '{\n    "reason": "all shards failed"\n}',
         message: 'all shards failed',
         config: {
-          url: 'http://localhost:3000/api/tsdb/query',
+          url: 'http://localhost:3000/api/ds/query',
         },
       };
 
@@ -345,9 +350,42 @@ describe('ElasticDatasource', function (this: any) {
       });
     });
 
+    it('should properly throw an error with just a message', async () => {
+      const response: FetchResponse = {
+        data: {
+          error: 'Bad Request',
+          message: 'Authentication to data source failed',
+        },
+        status: 400,
+        url: 'http://localhost:3000/api/ds/query',
+        config: { url: 'http://localhost:3000/api/ds/query' },
+        type: 'basic',
+        statusText: 'Bad Request',
+        redirected: false,
+        headers: {} as unknown as Headers,
+        ok: false,
+      };
+
+      const { ds } = getTestContext({
+        mockImplementation: () => throwError(response),
+        from: undefined,
+        jsonData: { esVersion: '7.10.0' },
+      });
+
+      const errObject = {
+        error: 'Bad Request',
+        message: 'Elasticsearch error: Authentication to data source failed',
+      };
+
+      await expect(ds.query(query)).toEmitValuesWith((received) => {
+        expect(received.length).toBe(1);
+        expect(received[0]).toEqual(errObject);
+      });
+    });
+
     it('should properly throw an unknown error', async () => {
       const { ds } = getTestContext({
-        jsonData: { interval: 'Daily', esVersion: 7 },
+        jsonData: { interval: 'Daily', esVersion: '7.10.0' },
         data: {
           took: 1,
           responses: [
@@ -363,7 +401,7 @@ describe('ElasticDatasource', function (this: any) {
         data: '{}',
         message: 'Unknown elastic error response',
         config: {
-          url: 'http://localhost:3000/api/tsdb/query',
+          url: 'http://localhost:3000/api/ds/query',
         },
       };
 
@@ -374,101 +412,101 @@ describe('ElasticDatasource', function (this: any) {
     });
   });
 
-  describe('When getting fields', () => {
-    const data = {
-      metricbeat: {
-        mappings: {
-          metricsets: {
-            _all: {},
-            _meta: {
-              test: 'something',
-            },
-            properties: {
-              '@timestamp': { type: 'date' },
-              __timestamp: { type: 'date' },
-              '@timestampnano': { type: 'date_nanos' },
-              beat: {
-                properties: {
-                  name: {
-                    fields: { raw: { type: 'keyword' } },
-                    type: 'string',
-                  },
-                  hostname: { type: 'string' },
-                },
-              },
-              system: {
-                properties: {
-                  cpu: {
-                    properties: {
-                      system: { type: 'float' },
-                      user: { type: 'float' },
-                    },
-                  },
-                  process: {
-                    properties: {
-                      cpu: {
-                        properties: {
-                          total: { type: 'float' },
-                        },
-                      },
-                      name: { type: 'string' },
-                    },
-                  },
-                },
-              },
-            },
-          },
-        },
-      },
-    };
+  // describe('When getting fields', () => {
+  // const data = {
+  //   metricbeat: {
+  //     mappings: {
+  //       metricsets: {
+  //         _all: {},
+  //         _meta: {
+  //           test: 'something',
+  //         },
+  //         properties: {
+  //           '@timestamp': { type: 'date' },
+  //           __timestamp: { type: 'date' },
+  //           '@timestampnano': { type: 'date_nanos' },
+  //           beat: {
+  //             properties: {
+  //               name: {
+  //                 fields: { raw: { type: 'keyword' } },
+  //                 type: 'string',
+  //               },
+  //               hostname: { type: 'string' },
+  //             },
+  //           },
+  //           system: {
+  //             properties: {
+  //               cpu: {
+  //                 properties: {
+  //                   system: { type: 'float' },
+  //                   user: { type: 'float' },
+  //                 },
+  //               },
+  //               process: {
+  //                 properties: {
+  //                   cpu: {
+  //                     properties: {
+  //                       total: { type: 'float' },
+  //                     },
+  //                   },
+  //                   name: { type: 'string' },
+  //                 },
+  //               },
+  //             },
+  //           },
+  //         },
+  //       },
+  //     },
+  //   },
+  // };
 
-    it('should return nested fields', async () => {
-      const { ds } = getTestContext({ data, jsonData: { esVersion: 50 }, database: 'metricbeat' });
+  // it('should return nested fields', async () => {
+  //   const { ds } = getTestContext({ data, jsonData: { esVersion: 50 }, database: 'metricbeat' });
 
-      await expect(ds.getFields()).toEmitValuesWith((received) => {
-        expect(received.length).toBe(1);
-        const fieldObjects = received[0];
-        const fields = _.map(fieldObjects, 'text');
+  //   await expect(ds.getFields()).toEmitValuesWith((received) => {
+  //     expect(received.length).toBe(1);
+  //     const fieldObjects = received[0];
+  //     const fields = map(fieldObjects, 'text');
 
-        expect(fields).toEqual([
-          '@timestamp',
-          '__timestamp',
-          '@timestampnano',
-          'beat.name.raw',
-          'beat.name',
-          'beat.hostname',
-          'system.cpu.system',
-          'system.cpu.user',
-          'system.process.cpu.total',
-          'system.process.name',
-        ]);
-      });
-    });
+  //     expect(fields).toEqual([
+  //       '@timestamp',
+  //       '__timestamp',
+  //       '@timestampnano',
+  //       'beat.name.raw',
+  //       'beat.name',
+  //       'beat.hostname',
+  //       'system.cpu.system',
+  //       'system.cpu.user',
+  //       'system.process.cpu.total',
+  //       'system.process.name',
+  //     ]);
+  //   });
+  // });
 
-    it('should return number fields', async () => {
-      const { ds } = getTestContext({ data, jsonData: { esVersion: 50 }, database: 'metricbeat' });
+  // it('should return number fields', async () => {
+  //   const { ds } = getTestContext({ data, jsonData: { esVersion: 50 }, database: 'metricbeat' });
 
-      await expect(ds.getFields('number')).toEmitValuesWith((received) => {
-        expect(received.length).toBe(1);
-        const fieldObjects = received[0];
-        const fields = _.map(fieldObjects, 'text');
+  //   await expect(ds.getFields(['number'])).toEmitValuesWith((received) => {
+  //     expect(received.length).toBe(1);
+  //     const fieldObjects = received[0];
+  //     const fields = map(fieldObjects, 'text');
 
-        expect(fields).toEqual(['system.cpu.system', 'system.cpu.user', 'system.process.cpu.total']);
-      });
-    });
+  //     expect(fields).toEqual(['system.cpu.system', 'system.cpu.user', 'system.process.cpu.total']);
+  //   });
+  // });
 
-    it('should return date fields', async () => {
-      const { ds } = getTestContext({ data, jsonData: { esVersion: 50 }, database: 'metricbeat' });
+  //   it('should return date fields', async () => {
+  //     const { ds } = getTestContext({ data, jsonData: { esVersion: 50 }, database: 'metricbeat' });
 
-      await expect(ds.getFields('date')).toEmitValuesWith((received) => {
-        expect(received.length).toBe(1);
-        const fieldObjects = received[0];
-        const fields = _.map(fieldObjects, 'text');
+  //     await expect(ds.getFields(['date'])).toEmitValuesWith((received) => {
+  //       expect(received.length).toBe(1);
+  //       const fieldObjects = received[0];
+  //       const fields = map(fieldObjects, 'text');
 
-        expect(fields).toEqual(['@timestamp', '__timestamp', '@timestampnano']);
-      });
-    });
-  });
+  //       expect(fields).toEqual(['@timestamp', '__timestamp', '@timestampnano']);
+  //     });
+  //   });
+  // });
 
   describe('When getting field mappings on indices with gaps', () => {
     const basicResponse = {
@@ -489,54 +527,54 @@ describe('ElasticDatasource', function (this: any) {
       },
     };
 
-    const alternateResponse = {
-      metricbeat: {
-        mappings: {
-          metricsets: {
-            _all: {},
-            properties: {
-              '@timestamp': { type: 'date' },
-            },
-          },
-        },
-      },
-    };
+    // const alternateResponse = {
+    //   metricbeat: {
+    //     mappings: {
+    //       metricsets: {
+    //         _all: {},
+    //         properties: {
+    //           '@timestamp': { type: 'date' },
+    //         },
+    //       },
+    //     },
+    //   },
+    // };
 
-    it('should return fields of the newest available index', async () => {
-      const twoDaysBefore = toUtc().subtract(2, 'day').format('YYYY.MM.DD');
-      const threeDaysBefore = toUtc().subtract(3, 'day').format('YYYY.MM.DD');
-      const baseUrl = `${ELASTICSEARCH_MOCK_URL}/asd-${twoDaysBefore}/_mapping`;
-      const alternateUrl = `${ELASTICSEARCH_MOCK_URL}/asd-${threeDaysBefore}/_mapping`;
+    // it('should return fields of the newest available index', async () => {
+    //   const twoDaysBefore = toUtc().subtract(2, 'day').format('YYYY.MM.DD');
+    //   const threeDaysBefore = toUtc().subtract(3, 'day').format('YYYY.MM.DD');
+    //   const baseUrl = `${ELASTICSEARCH_MOCK_URL}/asd-${twoDaysBefore}/_mapping`;
+    //   const alternateUrl = `${ELASTICSEARCH_MOCK_URL}/asd-${threeDaysBefore}/_mapping`;
 
-      const { ds, timeSrv } = getTestContext({
-        from: 'now-2w',
-        jsonData: { interval: 'Daily', esVersion: 50 },
-        mockImplementation: (options) => {
-          if (options.url === baseUrl) {
-            return of(createFetchResponse(basicResponse));
-          } else if (options.url === alternateUrl) {
-            return of(createFetchResponse(alternateResponse));
-          }
-          return throwError({ status: 404 });
-        },
-      });
+    //   const { ds, timeSrv } = getTestContext({
+    //     from: 'now-2w',
+    //     jsonData: { interval: 'Daily', esVersion: 50 },
+    //     mockImplementation: (options) => {
+    //       if (options.url === baseUrl) {
+    //         return of(createFetchResponse(basicResponse));
+    //       } else if (options.url === alternateUrl) {
+    //         return of(createFetchResponse(alternateResponse));
+    //       }
+    //       return throwError({ status: 404 });
+    //     },
+    //   });
 
-      const range = timeSrv.timeRange();
+    //   const range = timeSrv.timeRange();
 
-      await expect(ds.getFields(undefined, range)).toEmitValuesWith((received) => {
-        expect(received.length).toBe(1);
-        const fieldObjects = received[0];
-        const fields = _.map(fieldObjects, 'text');
-        expect(fields).toEqual(['@timestamp', 'beat.hostname']);
-      });
-    });
+    //   await expect(ds.getFields(undefined, range)).toEmitValuesWith((received) => {
+    //     expect(received.length).toBe(1);
+    //     const fieldObjects = received[0];
+    //     const fields = map(fieldObjects, 'text');
+    //     expect(fields).toEqual(['@timestamp', 'beat.hostname']);
+    //   });
+    // });
 
     it('should not retry when ES is down', async () => {
       const twoDaysBefore = toUtc().subtract(2, 'day').format('YYYY.MM.DD');
 
       const { ds, timeSrv, fetchMock } = getTestContext({
         from: 'now-2w',
-        jsonData: { interval: 'Daily', esVersion: 50 },
+        jsonData: { interval: 'Daily', esVersion: '7.10.0' },
         mockImplementation: (options) => {
           if (options.url === `${ELASTICSEARCH_MOCK_URL}/asd-${twoDaysBefore}/_mapping`) {
             return of(createFetchResponse(basicResponse));
@@ -557,7 +595,7 @@ describe('ElasticDatasource', function (this: any) {
     it('should not retry more than 7 indices', async () => {
       const { ds, timeSrv, fetchMock } = getTestContext({
         from: 'now-2w',
-        jsonData: { interval: 'Daily', esVersion: 50 },
+        jsonData: { interval: 'Daily', esVersion: '7.10.0' },
         mockImplementation: (options) => {
           return throwError({ status: 404 });
         },
@@ -656,14 +694,28 @@ describe('ElasticDatasource', function (this: any) {
       },
     };
 
+    const dateFields = ['@timestamp_millis'];
+    const numberFields = [
+      'justification_blob.overall_vote_score',
+      'justification_blob.shallow.jsi.sdb.dsel2.bootlegged-gille.botness',
+      'justification_blob.shallow.jsi.sdb.dsel2.bootlegged-gille.general_algorithm_score',
+      'justification_blob.shallow.jsi.sdb.dsel2.uncombed-boris.botness',
+      'justification_blob.shallow.jsi.sdb.dsel2.uncombed-boris.general_algorithm_score',
+      'overall_vote_score',
+    ];
+
     it('should return nested fields', async () => {
-      const { ds } = getTestContext({ data, database: 'genuine.es7._mapping.response', jsonData: { esVersion: 70 } });
+      const { ds } = getTestContext({
+        data,
+        database: 'genuine.es7._mapping.response',
+        jsonData: { esVersion: '7.10.0' },
+      });
 
       await expect(ds.getFields()).toEmitValuesWith((received) => {
         expect(received.length).toBe(1);
 
         const fieldObjects = received[0];
-        const fields = _.map(fieldObjects, 'text');
+        const fields = map(fieldObjects, 'text');
         expect(fields).toEqual([
           '@timestamp_millis',
           'classification_terms',
@@ -684,33 +736,34 @@ describe('ElasticDatasource', function (this: any) {
     });
 
     it('should return number fields', async () => {
-      const { ds } = getTestContext({ data, database: 'genuine.es7._mapping.response', jsonData: { esVersion: 70 } });
+      const { ds } = getTestContext({
+        data,
+        database: 'genuine.es7._mapping.response',
+        jsonData: { esVersion: '7.10.0' },
+      });
 
-      await expect(ds.getFields('number')).toEmitValuesWith((received) => {
+      await expect(ds.getFields(['number'])).toEmitValuesWith((received) => {
         expect(received.length).toBe(1);
 
         const fieldObjects = received[0];
-        const fields = _.map(fieldObjects, 'text');
-        expect(fields).toEqual([
-          'justification_blob.overall_vote_score',
-          'justification_blob.shallow.jsi.sdb.dsel2.bootlegged-gille.botness',
-          'justification_blob.shallow.jsi.sdb.dsel2.bootlegged-gille.general_algorithm_score',
-          'justification_blob.shallow.jsi.sdb.dsel2.uncombed-boris.botness',
-          'justification_blob.shallow.jsi.sdb.dsel2.uncombed-boris.general_algorithm_score',
-          'overall_vote_score',
-        ]);
+        const fields = map(fieldObjects, 'text');
+        expect(fields).toEqual(numberFields);
       });
     });
 
     it('should return date fields', async () => {
-      const { ds } = getTestContext({ data, database: 'genuine.es7._mapping.response', jsonData: { esVersion: 70 } });
+      const { ds } = getTestContext({
+        data,
+        database: 'genuine.es7._mapping.response',
+        jsonData: { esVersion: '7.10.0' },
+      });
 
-      await expect(ds.getFields('date')).toEmitValuesWith((received) => {
+      await expect(ds.getFields(['date'])).toEmitValuesWith((received) => {
         expect(received.length).toBe(1);
 
         const fieldObjects = received[0];
-        const fields = _.map(fieldObjects, 'text');
-        expect(fields).toEqual(['@timestamp_millis']);
+        const fields = map(fieldObjects, 'text');
+        expect(fields).toEqual(dateFields);
       });
     });
   });
@@ -729,7 +782,7 @@ describe('ElasticDatasource', function (this: any) {
       const query: any = { range, targets };
       const data = { responses: [] };
 
-      const { ds, fetchMock } = getTestContext({ jsonData: { esVersion: 5 }, data, database: 'test' });
+      const { ds, fetchMock } = getTestContext({ jsonData: { esVersion: '7.10.0' }, data, database: 'test' });
 
       await expect(ds.query(query)).toEmitValuesWith((received) => {
         expect(received.length).toBe(1);
@@ -777,7 +830,7 @@ describe('ElasticDatasource', function (this: any) {
         ],
       };
 
-      const { ds, fetchMock } = getTestContext({ jsonData: { esVersion: 5 }, data, database: 'test' });
+      const { ds, fetchMock } = getTestContext({ jsonData: { esVersion: '7.10.0' }, data, database: 'test' });
 
       const results = await ds.metricFindQuery('{"find": "terms", "field": "test"}');
 
@@ -819,7 +872,7 @@ describe('ElasticDatasource', function (this: any) {
 
   describe('query', () => {
     it('should replace range as integer not string', async () => {
-      const { ds } = getTestContext({ jsonData: { interval: 'Daily', esVersion: 2, timeField: '@time' } });
+      const { ds } = getTestContext({ jsonData: { interval: 'Daily', esVersion: '7.10.0', timeField: '@time' } });
       const postMock = jest.fn((url: string, data: any) => of(createFetchResponse({ responses: [] })));
       ds['post'] = postMock;
 
@@ -847,7 +900,7 @@ describe('ElasticDatasource', function (this: any) {
     expect((interpolatedQuery.bucketAggs![0] as Filters).settings!.filters![0].query).toBe('resolvedVariable');
   });
 
-  it('should correctly handle empty query strings', () => {
+  it('should correctly handle empty query strings in filters bucket aggregation', () => {
     const { ds } = getTestContext();
     const query: ElasticsearchQuery = {
       refId: 'A',
@@ -858,11 +911,32 @@ describe('ElasticDatasource', function (this: any) {
 
     const interpolatedQuery = ds.interpolateVariablesInQueries([query], {})[0];
 
-    expect(interpolatedQuery.query).toBe('*');
     expect((interpolatedQuery.bucketAggs![0] as Filters).settings!.filters![0].query).toBe('*');
   });
 });
 
+describe('getMultiSearchUrl', () => {
+  describe('When esVersion >= 7.10.0', () => {
+    it('Should add correct params to URL if "includeFrozen" is enabled', () => {
+      const { ds } = getTestContext({ jsonData: { esVersion: '7.10.0', includeFrozen: true, xpack: true } });
+
+      expect(ds.getMultiSearchUrl()).toMatch(/ignore_throttled=false/);
+    });
+
+    it('Should NOT add ignore_throttled if "includeFrozen" is disabled', () => {
+      const { ds } = getTestContext({ jsonData: { esVersion: '7.10.0', includeFrozen: false, xpack: true } });
+
+      expect(ds.getMultiSearchUrl()).not.toMatch(/ignore_throttled=false/);
+    });
+
+    it('Should NOT add ignore_throttled if "xpack" is disabled', () => {
+      const { ds } = getTestContext({ jsonData: { esVersion: '7.10.0', includeFrozen: true, xpack: false } });
+
+      expect(ds.getMultiSearchUrl()).not.toMatch(/ignore_throttled=false/);
+    });
+  });
+});
+
 describe('enhanceDataFrame', () => {
   it('adds links to dataframe', () => {
     const df = new MutableDataFrame({
@@ -883,28 +957,67 @@ describe('enhanceDataFrame', () => {
         field: 'urlField',
         url: 'someUrl',
       },
+      {
+        field: 'urlField',
+        url: 'someOtherUrl',
+      },
       {
         field: 'traceField',
         url: 'query',
-        datasourceUid: 'dsUid',
+        datasourceUid: 'ds1',
+      },
+      {
+        field: 'traceField',
+        url: 'otherQuery',
+        datasourceUid: 'ds2',
       },
     ]);
 
-    expect(df.fields[0].config.links!.length).toBe(1);
-    expect(df.fields[0].config.links![0]).toEqual({
+    expect(df.fields[0].config.links).toHaveLength(2);
+    expect(df.fields[0].config.links).toContainEqual({
       title: '',
       url: 'someUrl',
     });
-    expect(df.fields[1].config.links!.length).toBe(1);
-    expect(df.fields[1].config.links![0]).toEqual({
+    expect(df.fields[0].config.links).toContainEqual({
       title: '',
-      url: '',
-      internal: {
-        query: { query: 'query' },
-        datasourceName: 'elastic25',
-        datasourceUid: 'dsUid',
-      },
+      url: 'someOtherUrl',
     });
+
+    expect(df.fields[1].config.links).toHaveLength(2);
+    expect(df.fields[1].config.links).toContainEqual(
+      expect.objectContaining({
+        title: '',
+        url: '',
+        internal: expect.objectContaining({
+          query: { query: 'query' },
+          datasourceUid: 'ds1',
+        }),
+      })
+    );
+    expect(df.fields[1].config.links).toContainEqual(
+      expect.objectContaining({
+        title: '',
+        url: '',
+        internal: expect.objectContaining({
+          query: { query: 'otherQuery' },
+          datasourceUid: 'ds2',
+        }),
+      })
+    );
+  });
+
+  it('adds limit to dataframe', () => {
+    const df = new MutableDataFrame({
+      fields: [
+        {
+          name: 'someField',
+          values: new ArrayVector([]),
+        },
+      ],
+    });
+    enhanceDataFrame(df, [], 10);
+
+    expect(df.meta?.limit).toBe(10);
   });
 });
 
@@ -926,7 +1039,6 @@ const createElasticQuery = (): DataQueryRequest<ElasticsearchQuery> => {
     targets: [
       {
         refId: '',
-        isLogsQuery: false,
         bucketAggs: [{ type: 'date_histogram', field: '@timestamp', id: '2' }],
         metrics: [{ type: 'count', id: '' }],
         query: 'test',
