@@ -1,41 +1,54 @@
-import _ from 'lodash';
+import { cloneDeep, find, first as _first, isNumber, isObject, isString, map as _map } from 'lodash';
+import { generate, lastValueFrom, Observable, of, throwError } from 'rxjs';
+import { catchError, first, map, mergeMap, skipWhile, throwIfEmpty } from 'rxjs/operators';
+import { gte, lt, satisfies } from 'semver';
+
 import {
   DataFrame,
   DataLink,
-  DataQuery,
   DataQueryRequest,
   DataQueryResponse,
   DataSourceApi,
   DataSourceInstanceSettings,
+  DataSourceWithLogsContextSupport,
+  DataSourceWithQueryImportSupport,
+  DataSourceWithLogsVolumeSupport,
   DateTime,
   dateTime,
   Field,
   getDefaultTimeRange,
+  AbstractQuery,
+  getLogLevelFromKey,
+  LogLevel,
   LogRowModel,
   MetricFindValue,
-  PluginMeta,
   ScopedVars,
   TimeRange,
   toUtc,
 } from '@grafana/data';
-import LanguageProvider from './language_provider';
-import { ElasticResponse } from './elastic_response';
-import { IndexPattern } from './index_pattern';
-import { ElasticQueryBuilder } from './query_builder';
-import { defaultBucketAgg, hasMetricOfType } from './query_def';
-import { getBackendSrv, getDataSourceSrv } from '@grafana/runtime';
-import { getTemplateSrv, TemplateSrv } from 'app/features/templating/template_srv';
-import { DataLinkConfig, ElasticsearchOptions, ElasticsearchQuery } from './types';
+import { BackendSrvRequest, getBackendSrv, getDataSourceSrv } from '@grafana/runtime';
 import { RowContextOptions } from '@grafana/ui/src/components/Logs/LogRowContextProvider';
-import { metricAggregationConfig } from './components/QueryEditor/MetricAggregationsEditor/utils';
+import { queryLogsVolume } from 'app/core/logs_model';
+import { getTemplateSrv, TemplateSrv } from 'app/features/templating/template_srv';
+
+import {
+  BucketAggregation,
+  isBucketAggregationWithField,
+} from './components/QueryEditor/BucketAggregationsEditor/aggregations';
+import { bucketAggregationConfig } from './components/QueryEditor/BucketAggregationsEditor/utils';
 import {
   isMetricAggregationWithField,
   isPipelineAggregationWithMultipleBucketPaths,
+  Logs,
 } from './components/QueryEditor/MetricAggregationsEditor/aggregations';
-import { bucketAggregationConfig } from './components/QueryEditor/BucketAggregationsEditor/utils';
-import { isBucketAggregationWithField } from './components/QueryEditor/BucketAggregationsEditor/aggregations';
-import { generate, Observable, of, throwError } from 'rxjs';
-import { catchError, first, map, mergeMap, skipWhile, throwIfEmpty } from 'rxjs/operators';
+import { metricAggregationConfig } from './components/QueryEditor/MetricAggregationsEditor/utils';
+import { ElasticResponse } from './elastic_response';
+import { IndexPattern } from './index_pattern';
+import LanguageProvider from './language_provider';
+import { ElasticQueryBuilder } from './query_builder';
+import { defaultBucketAgg, hasMetricOfType } from './query_def';
+import { DataLinkConfig, ElasticsearchOptions, ElasticsearchQuery, TermsQuery } from './types';
+import { coerceESVersion, getScriptValue, isSupportedVersion } from './utils';
 
 // Those are metadata fields as defined in https://www.elastic.co/guide/en/elasticsearch/reference/current/mapping-fields.html#_identity_metadata_fields.
 // custom fields can start with underscores, therefore is not safe to exclude anything that starts with one.
@@ -51,14 +64,21 @@ const ELASTIC_META_FIELDS = [
   '_meta',
 ];
 
-export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, ElasticsearchOptions> {
+export class ElasticDatasource
+  extends DataSourceApi<ElasticsearchQuery, ElasticsearchOptions>
+  implements
+    DataSourceWithLogsContextSupport,
+    DataSourceWithQueryImportSupport<ElasticsearchQuery>,
+    DataSourceWithLogsVolumeSupport<ElasticsearchQuery>
+{
   basicAuth?: string;
   withCredentials?: boolean;
   url: string;
   name: string;
   index: string;
   timeField: string;
-  esVersion: number;
+  esVersion: string;
+  xpack: boolean;
   interval: string;
   maxConcurrentShardRequests?: number;
   queryBuilder: ElasticQueryBuilder;
@@ -67,6 +87,8 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
   logLevelField?: string;
   dataLinks: DataLinkConfig[];
   languageProvider: LanguageProvider;
+  includeFrozen: boolean;
+  isProxyAccess: boolean;
 
   constructor(
     instanceSettings: DataSourceInstanceSettings<ElasticsearchOptions>,
@@ -78,10 +100,12 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
     this.url = instanceSettings.url!;
     this.name = instanceSettings.name;
     this.index = instanceSettings.database ?? '';
+    this.isProxyAccess = instanceSettings.access === 'proxy';
     const settingsData = instanceSettings.jsonData || ({} as ElasticsearchOptions);
 
     this.timeField = settingsData.timeField;
-    this.esVersion = settingsData.esVersion;
+    this.esVersion = coerceESVersion(settingsData.esVersion);
+    this.xpack = Boolean(settingsData.xpack);
     this.indexPattern = new IndexPattern(this.index, settingsData.interval);
     this.interval = settingsData.timeInterval;
     this.maxConcurrentShardRequests = settingsData.maxConcurrentShardRequests;
@@ -92,6 +116,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
     this.logMessageField = settingsData.logMessageField || '';
     this.logLevelField = settingsData.logLevelField || '';
     this.dataLinks = settingsData.dataLinks || [];
+    this.includeFrozen = settingsData.includeFrozen ?? false;
 
     if (this.logMessageField === '') {
       this.logMessageField = undefined;
@@ -103,11 +128,31 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
     this.languageProvider = new LanguageProvider(this);
   }
 
-  private request(method: string, url: string, data?: undefined): Observable<any> {
-    const options: any = {
+  private request(
+    method: string,
+    url: string,
+    data?: undefined,
+    headers?: BackendSrvRequest['headers']
+  ): Observable<any> {
+    if (!this.isProxyAccess) {
+      const error = new Error(
+        'Browser access mode in the Elasticsearch datasource is no longer available. Switch to server access mode.'
+      );
+      return throwError(() => error);
+    }
+
+    if (!isSupportedVersion(this.esVersion)) {
+      const error = new Error(
+        'Support for Elasticsearch versions after their end-of-life (currently versions < 7.10) was removed.'
+      );
+      return throwError(() => error);
+    }
+
+    const options: BackendSrvRequest = {
       url: this.url + '/' + url,
-      method: method,
-      data: data,
+      method,
+      data,
+      headers,
     };
 
     if (this.basicAuth || this.withCredentials) {
@@ -127,9 +172,11 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
           return results.data;
         }),
         catchError((err) => {
-          if (err.data && err.data.error) {
+          if (err.data) {
+            const message = err.data.error?.reason ?? err.data.message ?? 'Unknown error';
+
             return throwError({
-              message: 'Elasticsearch error: ' + err.data.error.reason,
+              message: 'Elasticsearch error: ' + message,
               error: err.data.error,
             });
           }
@@ -139,8 +186,8 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
       );
   }
 
-  async importQueries(queries: DataQuery[], originMeta: PluginMeta): Promise<ElasticsearchQuery[]> {
-    return this.languageProvider.importQueries(queries, originMeta.id);
+  async importFromAbstractQueries(abstractQueries: AbstractQuery[]): Promise<ElasticsearchQuery[]> {
+    return abstractQueries.map((abstractQuery) => this.languageProvider.importFromAbstractQuery(abstractQuery));
   }
 
   /**
@@ -166,16 +213,16 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
     const maxTraversals = 7; // do not go beyond one week (for a daily pattern)
     const listLen = indexList.length;
 
-    return generate(
-      0,
-      (i) => i < Math.min(listLen, maxTraversals),
-      (i) => i + 1
-    ).pipe(
+    return generate({
+      initialState: 0,
+      condition: (i) => i < Math.min(listLen, maxTraversals),
+      iterate: (i) => i + 1,
+    }).pipe(
       mergeMap((index) => {
         // catch all errors and emit an object with an err property to simplify checks later in the pipeline
         return this.request('GET', indexList[listLen - index - 1]).pipe(catchError((err) => of({ err })));
       }),
-      skipWhile((resp) => resp.err && resp.err.status === 404), // skip all requests that fail because missing Elastic index
+      skipWhile((resp) => resp?.err?.status === 404), // skip all requests that fail because missing Elastic index
       throwIfEmpty(() => 'Could not find an available index for this time range.'), // when i === Math.min(listLen, maxTraversals) generate will complete but without emitting any values which means we didn't find a valid index
       first(), // take the first value that isn't skipped
       map((resp) => {
@@ -189,14 +236,14 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
   }
 
   private post(url: string, data: any): Observable<any> {
-    return this.request('POST', url, data);
+    return this.request('POST', url, data, { 'Content-Type': 'application/x-ndjson' });
   }
 
   annotationQuery(options: any): Promise<any> {
     const annotation = options.annotation;
     const timeField = annotation.timeField || '@timestamp';
     const timeEndField = annotation.timeEndField || null;
-    const queryString = annotation.query || '*';
+    const queryString = annotation.query;
     const tagsField = annotation.tagsField || 'tags';
     const textField = annotation.textField || null;
 
@@ -219,8 +266,8 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
       dateRanges.push({ range: rangeEnd });
     }
 
-    const queryInterpolated = this.templateSrv.replace(queryString, {}, 'lucene');
-    const query = {
+    const queryInterpolated = this.interpolateLuceneQuery(queryString);
+    const query: any = {
       bool: {
         filter: [
           {
@@ -229,22 +276,24 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
               minimum_should_match: 1,
             },
           },
-          {
-            query_string: {
-              query: queryInterpolated,
-            },
-          },
         ],
       },
     };
 
+    if (queryInterpolated) {
+      query.bool.filter.push({
+        query_string: {
+          query: queryInterpolated,
+        },
+      });
+    }
     const data: any = {
       query,
       size: 10000,
     };
 
     // fields field not supported on ES 5.x
-    if (this.esVersion < 5) {
+    if (lt(this.esVersion, '5.0.0')) {
       data['fields'] = [timeField, '_source'];
     }
 
@@ -262,8 +311,8 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
 
     const payload = JSON.stringify(header) + '\n' + JSON.stringify(data) + '\n';
 
-    return this.post('_msearch', payload)
-      .pipe(
+    return lastValueFrom(
+      this.post('_msearch', payload).pipe(
         map((res) => {
           const list = [];
           const hits = res.responses[0].hits.hits;
@@ -292,7 +341,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
             let time = getFieldFromSource(source, timeField);
             if (typeof hits[i].fields !== 'undefined') {
               const fields = hits[i].fields;
-              if (_.isString(fields[timeField]) || _.isNumber(fields[timeField])) {
+              if (isString(fields[timeField]) || isNumber(fields[timeField])) {
                 time = fields[timeField];
               }
             }
@@ -334,43 +383,55 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
           return list;
         })
       )
-      .toPromise();
+    );
   }
 
-  private interpolateLuceneQuery(queryString: string, scopedVars: ScopedVars) {
-    // Elasticsearch queryString should always be '*' if empty string
-    return this.templateSrv.replace(queryString, scopedVars, 'lucene') || '*';
+  private interpolateLuceneQuery(queryString: string, scopedVars?: ScopedVars) {
+    return this.templateSrv.replace(queryString, scopedVars, 'lucene');
   }
 
   interpolateVariablesInQueries(queries: ElasticsearchQuery[], scopedVars: ScopedVars): ElasticsearchQuery[] {
-    let expandedQueries = queries;
-    if (queries && queries.length > 0) {
-      expandedQueries = queries.map((query) => {
-        const expandedQuery = {
-          ...query,
-          datasource: this.name,
-          query: this.interpolateLuceneQuery(query.query || '', scopedVars),
+    // We need a separate interpolation format for lucene queries, therefore we first interpolate any
+    // lucene query string and then everything else
+    const interpolateBucketAgg = (bucketAgg: BucketAggregation): BucketAggregation => {
+      if (bucketAgg.type === 'filters') {
+        return {
+          ...bucketAgg,
+          settings: {
+            ...bucketAgg.settings,
+            filters: bucketAgg.settings?.filters?.map((filter) => ({
+              ...filter,
+              query: this.interpolateLuceneQuery(filter.query, scopedVars) || '*',
+            })),
+          },
         };
+      }
 
-        for (let bucketAgg of query.bucketAggs || []) {
-          if (bucketAgg.type === 'filters') {
-            for (let filter of bucketAgg.settings?.filters || []) {
-              filter.query = this.interpolateLuceneQuery(filter.query, scopedVars);
-            }
-          }
-        }
-        return expandedQuery;
-      });
-    }
-    return expandedQueries;
+      return bucketAgg;
+    };
+
+    const expandedQueries = queries.map(
+      (query): ElasticsearchQuery => ({
+        ...query,
+        datasource: this.getRef(),
+        query: this.interpolateLuceneQuery(query.query || '', scopedVars),
+        bucketAggs: query.bucketAggs?.map(interpolateBucketAgg),
+      })
+    );
+
+    const finalQueries: ElasticsearchQuery[] = JSON.parse(
+      this.templateSrv.replace(JSON.stringify(expandedQueries), scopedVars)
+    );
+
+    return finalQueries;
   }
 
   testDatasource() {
     // validate that the index exist and has date field
-    return this.getFields('date')
-      .pipe(
+    return lastValueFrom(
+      this.getFields(['date']).pipe(
         mergeMap((dateFields) => {
-          const timeField: any = _.find(dateFields, { text: this.timeField });
+          const timeField: any = find(dateFields, { text: this.timeField });
           if (!timeField) {
             return of({ status: 'error', message: 'No date field named ' + this.timeField + ' found' });
           }
@@ -385,7 +446,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
           }
         })
       )
-      .toPromise();
+    );
   }
 
   getQueryHeader(searchType: any, timeFrom?: DateTime, timeTo?: DateTime): string {
@@ -395,7 +456,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
       index: this.indexPattern.getIndexList(timeFrom, timeTo),
     };
 
-    if (this.esVersion >= 56 && this.esVersion < 70) {
+    if (satisfies(this.esVersion, '>=5.6.0 <7.0.0')) {
       queryHeader['max_concurrent_shard_requests'] = this.maxConcurrentShardRequests;
     }
 
@@ -423,7 +484,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
         text += metric.field;
       }
       if (isPipelineAggregationWithMultipleBucketPaths(metric)) {
-        text += metric.settings?.script?.replace(new RegExp('params.', 'g'), '');
+        text += getScriptValue(metric).replace(new RegExp('params.', 'g'), '');
       }
       text += '), ';
 
@@ -459,7 +520,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
    * search_after feature.
    */
   showContextToggle(): boolean {
-    return this.esVersion > 5;
+    return gte(this.esVersion, '5.0.0');
   }
 
   getLogRowContext = async (row: LogRowModel, options?: RowContextOptions): Promise<{ data: DataFrame[] }> => {
@@ -494,11 +555,11 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
     });
     const payload = [header, esQuery].join('\n') + '\n';
     const url = this.getMultiSearchUrl();
-    const response = await this.post(url, payload).toPromise();
-    const targets: ElasticsearchQuery[] = [{ refId: `${row.dataFrame.refId}`, metrics: [], isLogsQuery: true }];
+    const response = await lastValueFrom(this.post(url, payload));
+    const targets: ElasticsearchQuery[] = [{ refId: `${row.dataFrame.refId}`, metrics: [{ type: 'logs', id: '1' }] }];
     const elasticResponse = new ElasticResponse(targets, transformHitsBasedOnDirection(response, sort));
     const logResponse = elasticResponse.getLogs(this.logMessageField, this.logLevelField);
-    const dataFrame = _.first(logResponse.data);
+    const dataFrame = _first(logResponse.data);
     if (!dataFrame) {
       return { data: [] };
     }
@@ -524,37 +585,104 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
     return logResponse;
   };
 
+  getLogsVolumeDataProvider(request: DataQueryRequest<ElasticsearchQuery>): Observable<DataQueryResponse> | undefined {
+    const isLogsVolumeAvailable = request.targets.some((target) => {
+      return target.metrics?.length === 1 && target.metrics[0].type === 'logs';
+    });
+    if (!isLogsVolumeAvailable) {
+      return undefined;
+    }
+    const logsVolumeRequest = cloneDeep(request);
+    logsVolumeRequest.targets = logsVolumeRequest.targets.map((target) => {
+      const bucketAggs: BucketAggregation[] = [];
+      const timeField = this.timeField ?? '@timestamp';
+
+      if (this.logLevelField) {
+        bucketAggs.push({
+          id: '2',
+          type: 'terms',
+          settings: {
+            min_doc_count: '0',
+            size: '0',
+            order: 'desc',
+            orderBy: '_count',
+            missing: LogLevel.unknown,
+          },
+          field: this.logLevelField,
+        });
+      }
+      bucketAggs.push({
+        id: '3',
+        type: 'date_histogram',
+        settings: {
+          interval: 'auto',
+          min_doc_count: '0',
+          trimEdges: '0',
+        },
+        field: timeField,
+      });
+
+      const logsVolumeQuery: ElasticsearchQuery = {
+        refId: target.refId,
+        query: target.query,
+        metrics: [{ type: 'count', id: '1' }],
+        timeField,
+        bucketAggs,
+      };
+      return logsVolumeQuery;
+    });
+
+    return queryLogsVolume(this, logsVolumeRequest, {
+      range: request.range,
+      targets: request.targets,
+      extractLevel: (dataFrame) => getLogLevelFromKey(dataFrame.name || ''),
+    });
+  }
+
   query(options: DataQueryRequest<ElasticsearchQuery>): Observable<DataQueryResponse> {
     let payload = '';
-    const targets = this.interpolateVariablesInQueries(_.cloneDeep(options.targets), options.scopedVars);
+    const targets = this.interpolateVariablesInQueries(cloneDeep(options.targets), options.scopedVars);
     const sentTargets: ElasticsearchQuery[] = [];
+    let targetsContainsLogsQuery = targets.some((target) => hasMetricOfType(target, 'logs'));
 
     // add global adhoc filters to timeFilter
     const adhocFilters = this.templateSrv.getAdhocFilters(this.name);
 
+    const logLimits: Array<number | undefined> = [];
+
     for (const target of targets) {
       if (target.hide) {
         continue;
       }
 
       let queryObj;
-      if (target.isLogsQuery || hasMetricOfType(target, 'logs')) {
+      if (hasMetricOfType(target, 'logs')) {
+        // FIXME: All this logic here should be in the query builder.
+        // When moving to the BE-only implementation we should remove this and let the BE
+        // Handle this.
+        // TODO: defaultBucketAgg creates a dete_histogram aggregation without a field, so it fallbacks to
+        // the configured timeField. we should allow people to use a different time field here.
         target.bucketAggs = [defaultBucketAgg()];
+
+        const log = target.metrics?.find((m) => m.type === 'logs') as Logs;
+        const limit = log.settings?.limit ? parseInt(log.settings?.limit, 10) : 500;
+        logLimits.push(limit);
+
         target.metrics = [];
         // Setting this for metrics queries that are typed as logs
-        target.isLogsQuery = true;
-        queryObj = this.queryBuilder.getLogsQuery(target, adhocFilters, target.query);
+        queryObj = this.queryBuilder.getLogsQuery(target, limit, adhocFilters);
       } else {
+        logLimits.push();
         if (target.alias) {
-          target.alias = this.templateSrv.replace(target.alias, options.scopedVars, 'lucene');
+          target.alias = this.interpolateLuceneQuery(target.alias, options.scopedVars);
         }
 
-        queryObj = this.queryBuilder.build(target, adhocFilters, target.query);
+        queryObj = this.queryBuilder.build(target, adhocFilters);
       }
 
       const esQuery = JSON.stringify(queryObj);
 
-      const searchType = queryObj.size === 0 && this.esVersion < 5 ? 'count' : 'query_then_fetch';
+      const searchType = queryObj.size === 0 && lt(this.esVersion, '5.0.0') ? 'count' : 'query_then_fetch';
       const header = this.getQueryHeader(searchType, options.range.from, options.range.to);
       payload += header + '\n';
 
@@ -581,11 +709,13 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
       map((res) => {
         const er = new ElasticResponse(sentTargets, res);
 
-        if (sentTargets.some((target) => target.isLogsQuery)) {
+        // TODO: This needs to be revisited, it seems wrong to process ALL the sent queries as logs if only one of them was a log query
+        if (targetsContainsLogsQuery) {
           const response = er.getLogs(this.logMessageField, this.logLevelField);
-          for (const dataFrame of response.data) {
-            enhanceDataFrame(dataFrame, this.dataLinks);
-          }
+
+          response.data.forEach((dataFrame, index) => {
+            enhanceDataFrame(dataFrame, this.dataLinks, logLimits[index]);
+          });
           return response;
         }
 
@@ -599,34 +729,35 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
   }
 
   // TODO: instead of being a string, this could be a custom type representing all the elastic types
-  getFields(type?: string, range?: TimeRange): Observable<MetricFindValue[]> {
-    const configuredEsVersion = this.esVersion;
+  // FIXME: This doesn't seem to return actual MetricFindValues, we should either change the return type
+  // or fix the implementation.
+  getFields(type?: string[], range?: TimeRange): Observable<MetricFindValue[]> {
+    const typeMap: Record<string, string> = {
+      float: 'number',
+      double: 'number',
+      integer: 'number',
+      long: 'number',
+      date: 'date',
+      date_nanos: 'date',
+      string: 'string',
+      text: 'string',
+      scaled_float: 'number',
+      nested: 'nested',
+      histogram: 'number',
+    };
     return this.get('/_mapping', range).pipe(
       map((result) => {
-        const typeMap: any = {
-          float: 'number',
-          double: 'number',
-          integer: 'number',
-          long: 'number',
-          date: 'date',
-          date_nanos: 'date',
-          string: 'string',
-          text: 'string',
-          scaled_float: 'number',
-          nested: 'nested',
-        };
-
         const shouldAddField = (obj: any, key: string) => {
           if (this.isMetadataField(key)) {
             return false;
           }
 
-          if (!type) {
+          if (!type || type.length === 0) {
             return true;
           }
 
           // equal query type filter, or via typemap translation
-          return type === obj.type || type === typeMap[obj.type];
+          return type.includes(obj.type) || type.includes(typeMap[obj.type]);
         };
 
         // Store subfield names: [system, process, cpu, total] -> system.process.cpu.total
@@ -638,17 +769,17 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
             const subObj = obj[key];
 
             // Check mapping field for nested fields
-            if (_.isObject(subObj.properties)) {
+            if (isObject(subObj.properties)) {
               fieldNameParts.push(key);
               getFieldsRecursively(subObj.properties);
             }
 
-            if (_.isObject(subObj.fields)) {
+            if (isObject(subObj.fields)) {
               fieldNameParts.push(key);
               getFieldsRecursively(subObj.fields);
             }
 
-            if (_.isString(subObj.type)) {
+            if (isString(subObj.type)) {
               const fieldName = fieldNameParts.concat(key).join('.');
 
               // Hide meta-fields and check field type
@@ -668,7 +799,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
           if (index && index.mappings) {
             const mappings = index.mappings;
 
-            if (configuredEsVersion < 70) {
+            if (lt(this.esVersion, '7.0.0')) {
               for (const typeName in mappings) {
                 const properties = mappings[typeName].properties;
                 getFieldsRecursively(properties);
@@ -681,15 +812,15 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
         }
 
         // transform to array
-        return _.map(fields, (value) => {
+        return _map(fields, (value) => {
           return value;
         });
       })
     );
   }
 
-  getTerms(queryDef: any, range = getDefaultTimeRange()): Observable<MetricFindValue[]> {
-    const searchType = this.esVersion >= 5 ? 'query_then_fetch' : 'count';
+  getTerms(queryDef: TermsQuery, range = getDefaultTimeRange()): Observable<MetricFindValue[]> {
+    const searchType = gte(this.esVersion, '5.0.0') ? 'query_then_fetch' : 'count';
     const header = this.getQueryHeader(searchType, range.from, range.to);
     let esQuery = JSON.stringify(this.queryBuilder.getTermsQuery(queryDef));
 
@@ -706,7 +837,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
         }
 
         const buckets = res.responses[0].aggregations['1'].buckets;
-        return _.map(buckets, (bucket) => {
+        return _map(buckets, (bucket) => {
           return {
             text: bucket.key_as_string || bucket.key,
             value: bucket.key,
@@ -717,11 +848,17 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
   }
 
   getMultiSearchUrl() {
-    if (this.esVersion >= 70 && this.maxConcurrentShardRequests) {
-      return `_msearch?max_concurrent_shard_requests=${this.maxConcurrentShardRequests}`;
+    const searchParams = new URLSearchParams();
+
+    if (gte(this.esVersion, '7.0.0') && this.maxConcurrentShardRequests) {
+      searchParams.append('max_concurrent_shard_requests', `${this.maxConcurrentShardRequests}`);
     }
 
-    return '_msearch';
+    if (gte(this.esVersion, '6.6.0') && this.xpack && this.includeFrozen) {
+      searchParams.append('ignore_throttled', 'false');
+    }
+
+    return ('_msearch?' + searchParams.toString()).replace(/\?$/, '');
   }
 
   metricFindQuery(query: string, options?: any): Promise<MetricFindValue[]> {
@@ -729,14 +866,14 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
     const parsedQuery = JSON.parse(query);
     if (query) {
       if (parsedQuery.find === 'fields') {
-        parsedQuery.type = this.templateSrv.replace(parsedQuery.type, {}, 'lucene');
-        return this.getFields(parsedQuery.type, range).toPromise();
+        parsedQuery.type = this.interpolateLuceneQuery(parsedQuery.type);
+        return lastValueFrom(this.getFields(parsedQuery.type, range));
       }
 
       if (parsedQuery.find === 'terms') {
-        parsedQuery.field = this.templateSrv.replace(parsedQuery.field, {}, 'lucene');
-        parsedQuery.query = this.templateSrv.replace(parsedQuery.query || '*', {}, 'lucene');
-        return this.getTerms(parsedQuery, range).toPromise();
+        parsedQuery.field = this.interpolateLuceneQuery(parsedQuery.field);
+        parsedQuery.query = this.interpolateLuceneQuery(parsedQuery.query);
+        return lastValueFrom(this.getTerms(parsedQuery, range));
       }
     }
 
@@ -744,27 +881,27 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
   }
 
   getTagKeys() {
-    return this.getFields().toPromise();
+    return lastValueFrom(this.getFields());
   }
 
   getTagValues(options: any) {
-    return this.getTerms({ field: options.key, query: '*' }).toPromise();
+    return lastValueFrom(this.getTerms({ field: options.key }));
   }
 
   targetContainsTemplate(target: any) {
-    if (this.templateSrv.variableExists(target.query) || this.templateSrv.variableExists(target.alias)) {
+    if (this.templateSrv.containsTemplate(target.query) || this.templateSrv.containsTemplate(target.alias)) {
       return true;
     }
 
     for (const bucketAgg of target.bucketAggs) {
-      if (this.templateSrv.variableExists(bucketAgg.field) || this.objectContainsTemplate(bucketAgg.settings)) {
+      if (this.templateSrv.containsTemplate(bucketAgg.field) || this.objectContainsTemplate(bucketAgg.settings)) {
         return true;
       }
     }
 
     for (const metric of target.metrics) {
       if (
-        this.templateSrv.variableExists(metric.field) ||
+        this.templateSrv.containsTemplate(metric.field) ||
         this.objectContainsTemplate(metric.settings) ||
         this.objectContainsTemplate(metric.meta)
       ) {
@@ -793,7 +930,7 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
 
     for (const key of Object.keys(obj)) {
       if (this.isPrimitive(obj[key])) {
-        if (this.templateSrv.variableExists(obj[key])) {
+        if (this.templateSrv.containsTemplate(obj[key])) {
           return true;
         }
       } else if (Array.isArray(obj[key])) {
@@ -817,43 +954,50 @@ export class ElasticDatasource extends DataSourceApi<ElasticsearchQuery, Elastic
  * Modifies dataframe and adds dataLinks from the config.
  * Exported for tests.
  */
-export function enhanceDataFrame(dataFrame: DataFrame, dataLinks: DataLinkConfig[]) {
-  const dataSourceSrv = getDataSourceSrv();
+export function enhanceDataFrame(dataFrame: DataFrame, dataLinks: DataLinkConfig[], limit?: number) {
+  if (limit) {
+    dataFrame.meta = {
+      ...dataFrame.meta,
+      limit,
+    };
+  }
 
   if (!dataLinks.length) {
     return;
   }
 
   for (const field of dataFrame.fields) {
-    const dataLinkConfig = dataLinks.find((dataLink) => field.name && field.name.match(dataLink.field));
+    const linksToApply = dataLinks.filter((dataLink) => new RegExp(dataLink.field).test(field.name));
 
-    if (!dataLinkConfig) {
+    if (linksToApply.length === 0) {
       continue;
     }
 
-    let link: DataLink;
-
-    if (dataLinkConfig.datasourceUid) {
-      const dsSettings = dataSourceSrv.getInstanceSettings(dataLinkConfig.datasourceUid);
-
-      link = {
-        title: '',
-        url: '',
-        internal: {
-          query: { query: dataLinkConfig.url },
-          datasourceUid: dataLinkConfig.datasourceUid,
-          datasourceName: dsSettings?.name ?? 'Data source not found',
-        },
-      };
-    } else {
-      link = {
-        title: '',
-        url: dataLinkConfig.url,
-      };
-    }
-
     field.config = field.config || {};
-    field.config.links = [...(field.config.links || []), link];
+    field.config.links = [...(field.config.links || [], linksToApply.map(generateDataLink))];
+  }
+}
+
+function generateDataLink(linkConfig: DataLinkConfig): DataLink {
+  const dataSourceSrv = getDataSourceSrv();
+
+  if (linkConfig.datasourceUid) {
+    const dsSettings = dataSourceSrv.getInstanceSettings(linkConfig.datasourceUid);
+
+    return {
+      title: linkConfig.urlDisplayLabel || '',
+      url: '',
+      internal: {
+        query: { query: linkConfig.url },
+        datasourceUid: linkConfig.datasourceUid,
+        datasourceName: dsSettings?.name ?? 'Data source not found',
+      },
+    };
+  } else {
+    return {
+      title: linkConfig.urlDisplayLabel || '',
+      url: linkConfig.url,
+    };
   }
 }
 
