@@ -1,7 +1,5 @@
-import _ from 'lodash';
-import flatten from 'app/core/utils/flatten';
-import * as queryDef from './query_def';
-import TableModel from 'app/core/table_model';
+import { clone, filter, find, identity, isArray, keys, map, uniq, values as _values } from 'lodash';
+
 import {
   DataQueryResponse,
   DataFrame,
@@ -10,13 +8,26 @@ import {
   MutableDataFrame,
   PreferredVisualisationType,
 } from '@grafana/data';
-import { ElasticsearchAggregation, ElasticsearchQuery } from './types';
+import TableModel from 'app/core/table_model';
+import flatten from 'app/core/utils/flatten';
+
 import {
   ExtendedStatMetaType,
   isMetricAggregationWithField,
+  TopMetrics,
 } from './components/QueryEditor/MetricAggregationsEditor/aggregations';
-import { describeMetric } from './utils';
 import { metricAggregationConfig } from './components/QueryEditor/MetricAggregationsEditor/utils';
+import * as queryDef from './query_def';
+import { ElasticsearchAggregation, ElasticsearchQuery } from './types';
+import { describeMetric, getScriptValue } from './utils';
+
+const HIGHLIGHT_TAGS_EXP = `${queryDef.highlightTags.pre}([^@]+)${queryDef.highlightTags.post}`;
+type TopMetricMetric = Record<string, number>;
+interface TopMetricBucket {
+  top: Array<{
+    metrics: TopMetricMetric;
+  }>;
+}
 
 export class ElasticResponse {
   constructor(private targets: ElasticsearchQuery[], private response: any) {
@@ -101,6 +112,33 @@ export class ElasticResponse {
 
           break;
         }
+        case 'top_metrics': {
+          if (metric.settings?.metrics?.length) {
+            for (const metricField of metric.settings?.metrics) {
+              newSeries = {
+                datapoints: [],
+                metric: metric.type,
+                props: props,
+                refId: target.refId,
+                field: metricField,
+              };
+              for (let i = 0; i < esAgg.buckets.length; i++) {
+                const bucket = esAgg.buckets[i];
+                const stats = bucket[metric.id] as TopMetricBucket;
+                const values = stats.top.map((hit) => {
+                  if (hit.metrics[metricField]) {
+                    return hit.metrics[metricField];
+                  }
+                  return null;
+                });
+                const point = [values[values.length - 1], bucket.key];
+                newSeries.datapoints.push(point);
+              }
+              seriesList.push(newSeries);
+            }
+          }
+          break;
+        }
         default: {
           newSeries = {
             datapoints: [],
@@ -142,7 +180,7 @@ export class ElasticResponse {
   ) {
     // add columns
     if (table.columns.length === 0) {
-      for (const propKey of _.keys(props)) {
+      for (const propKey of keys(props)) {
         table.addColumn({ text: propKey, filterable: true });
       }
       table.addColumn({ text: aggDef.field, filterable: true });
@@ -153,11 +191,11 @@ export class ElasticResponse {
       table.addColumn({ text: metricName });
       values.push(value);
     };
-    const buckets = _.isArray(esAgg.buckets) ? esAgg.buckets : [esAgg.buckets];
+    const buckets = isArray(esAgg.buckets) ? esAgg.buckets : [esAgg.buckets];
     for (const bucket of buckets) {
       const values = [];
 
-      for (const propValues of _.values(props)) {
+      for (const propValues of _values(props)) {
         values.push(propValues);
       }
 
@@ -193,9 +231,26 @@ export class ElasticResponse {
             }
             break;
           }
+          case 'top_metrics': {
+            const baseName = this.getMetricName(metric.type);
+
+            if (metric.settings?.metrics) {
+              for (const metricField of metric.settings.metrics) {
+                // If we selected more than one metric we also add each metric name
+                const metricName = metric.settings.metrics.length > 1 ? `${baseName} ${metricField}` : baseName;
+
+                const stats = bucket[metric.id] as TopMetricBucket;
+
+                // Size of top_metrics is fixed to 1.
+                addMetricValue(values, metricName, stats.top[0].metrics[metricField]);
+              }
+            }
+
+            break;
+          }
           default: {
             let metricName = this.getMetricName(metric.type);
-            const otherMetrics = _.filter(target.metrics, { type: metric.type });
+            const otherMetrics = filter(target.metrics, { type: metric.type });
 
             // if more of the same metric type include field field name in property
             if (otherMetrics.length > 1) {
@@ -205,7 +260,7 @@ export class ElasticResponse {
 
               if (metric.type === 'bucket_script') {
                 //Use the formula in the column name
-                metricName = metric.settings?.script || '';
+                metricName = getScriptValue(metric);
               }
             }
 
@@ -226,7 +281,7 @@ export class ElasticResponse {
     const maxDepth = target.bucketAggs!.length - 1;
 
     for (aggId in aggs) {
-      aggDef = _.find(target.bucketAggs, { id: aggId });
+      aggDef = find(target.bucketAggs, { id: aggId });
       esAgg = aggs[aggId];
 
       if (!aggDef) {
@@ -242,7 +297,7 @@ export class ElasticResponse {
       } else {
         for (const nameIndex in esAgg.buckets) {
           bucket = esAgg.buckets[nameIndex];
-          props = _.clone(props);
+          props = clone(props);
           if (bucket.key !== void 0) {
             props[aggDef.field] = bucket.key;
           } else {
@@ -274,7 +329,7 @@ export class ElasticResponse {
     return metric;
   }
 
-  private getSeriesName(series: any, target: ElasticsearchQuery, metricTypeCount: any) {
+  private getSeriesName(series: any, target: ElasticsearchQuery, dedup: boolean) {
     let metricName = this.getMetricName(series.metric);
 
     if (target.alias) {
@@ -302,12 +357,12 @@ export class ElasticResponse {
 
     if (queryDef.isPipelineAgg(series.metric)) {
       if (series.metric && queryDef.isPipelineAggWithMultipleBucketPaths(series.metric)) {
-        const agg: any = _.find(target.metrics, { id: series.metricId });
+        const agg: any = find(target.metrics, { id: series.metricId });
         if (agg && agg.settings.script) {
-          metricName = agg.settings.script;
+          metricName = getScriptValue(agg);
 
           for (const pv of agg.pipelineVariables) {
-            const appliedAgg: any = _.find(target.metrics, { id: pv.pipelineAgg });
+            const appliedAgg: any = find(target.metrics, { id: pv.pipelineAgg });
             if (appliedAgg) {
               metricName = metricName.replace('params.' + pv.name, describeMetric(appliedAgg));
             }
@@ -316,7 +371,7 @@ export class ElasticResponse {
           metricName = 'Unset';
         }
       } else {
-        const appliedAgg: any = _.find(target.metrics, { id: series.field });
+        const appliedAgg: any = find(target.metrics, { id: series.field });
         if (appliedAgg) {
           metricName += ' ' + describeMetric(appliedAgg);
         } else {
@@ -327,7 +382,7 @@ export class ElasticResponse {
       metricName += ' ' + series.field;
     }
 
-    const propKeys = _.keys(series.props);
+    const propKeys = keys(series.props);
     if (propKeys.length === 0) {
       return metricName;
     }
@@ -337,19 +392,22 @@ export class ElasticResponse {
       name += series.props[propName] + ' ';
     }
 
-    if (metricTypeCount === 1) {
-      return name.trim();
+    if (dedup) {
+      return name.trim() + ' ' + metricName;
     }
 
-    return name.trim() + ' ' + metricName;
+    return name.trim();
   }
 
   nameSeries(seriesList: any, target: ElasticsearchQuery) {
-    const metricTypeCount = _.uniq(_.map(seriesList, 'metric')).length;
+    const metricTypeCount = uniq(map(seriesList, 'metric')).length;
+    const hasTopMetricWithMultipleMetrics = (
+      target.metrics?.filter((m) => m.type === 'top_metrics') as TopMetrics[]
+    ).some((m) => (m?.settings?.metrics?.length || 0) > 1);
 
     for (let i = 0; i < seriesList.length; i++) {
       const series = seriesList[i];
-      series.target = this.getSeriesName(series, target, metricTypeCount);
+      series.target = this.getSeriesName(series, target, metricTypeCount > 1 || hasTopMetricWithMultipleMetrics);
     }
   }
 
@@ -373,6 +431,7 @@ export class ElasticResponse {
         _type: hit._type,
         _index: hit._index,
         sort: hit.sort,
+        highlight: hit.highlight,
       };
 
       if (hit._source) {
@@ -391,7 +450,7 @@ export class ElasticResponse {
   }
 
   trimDatapoints(aggregations: any, target: ElasticsearchQuery) {
-    const histogram: any = _.find(target.bucketAggs, { type: 'date_histogram' });
+    const histogram: any = find(target.bucketAggs, { type: 'date_histogram' });
 
     const shouldDropFirstAndLast = histogram && histogram.settings && histogram.settings.trimEdges;
     if (shouldDropFirstAndLast) {
@@ -422,7 +481,7 @@ export class ElasticResponse {
   }
 
   getTimeSeries() {
-    if (this.targets.some((target) => target.metrics?.some((metric) => metric.type === 'raw_data'))) {
+    if (this.targets.some((target) => queryDef.hasMetricOfType(target, 'raw_data'))) {
       return this.processResponseToDataFrames(false);
     }
     return this.processResponseToSeries();
@@ -432,47 +491,79 @@ export class ElasticResponse {
     return this.processResponseToDataFrames(true, logMessageField, logLevelField);
   }
 
-  processResponseToDataFrames(
+  private processResponseToDataFrames(
     isLogsRequest: boolean,
     logMessageField?: string,
     logLevelField?: string
   ): DataQueryResponse {
     const dataFrame: DataFrame[] = [];
-
     for (let n = 0; n < this.response.responses.length; n++) {
       const response = this.response.responses[n];
       if (response.error) {
         throw this.getErrorFromElasticResponse(this.response, response.error);
       }
 
-      if (response.hits && response.hits.hits.length > 0) {
+      if (response.hits) {
         const { propNames, docs } = flattenHits(response.hits.hits);
-        if (docs.length > 0) {
-          let series = createEmptyDataFrame(
-            propNames,
-            this.targets[0].timeField!,
-            isLogsRequest,
-            logMessageField,
-            logLevelField
-          );
 
-          // Add a row for each document
-          for (const doc of docs) {
-            if (logLevelField) {
-              // Remap level field based on the datasource config. This field is then used in explore to figure out the
-              // log level. We may rewrite some actual data in the level field if they are different.
-              doc['level'] = doc[logLevelField];
-            }
+        const series = docs.length
+          ? createEmptyDataFrame(
+              propNames.map(toNameTypePair(docs)),
+              isLogsRequest,
+              this.targets[0].timeField,
+              logMessageField,
+              logLevelField
+            )
+          : createEmptyDataFrame([], isLogsRequest);
 
-            series.add(doc);
-          }
-          if (isLogsRequest) {
-            series = addPreferredVisualisationType(series, 'logs');
-          }
-          const target = this.targets[n];
-          series.refId = target.refId;
-          dataFrame.push(series);
+        if (isLogsRequest) {
+          addPreferredVisualisationType(series, 'logs');
         }
+
+        // Add a row for each document
+        for (const doc of docs) {
+          if (logLevelField) {
+            // Remap level field based on the datasource config. This field is
+            // then used in explore to figure out the log level. We may rewrite
+            // some actual data in the level field if they are different.
+            doc['level'] = doc[logLevelField];
+          }
+          // When highlighting exists, we need to collect all the highlighted
+          // phrases and add them to the DataFrame's meta.searchWords array.
+          if (doc.highlight) {
+            // There might be multiple words so we need two versions of the
+            // regular expression. One to match gobally, when used with part.match,
+            // it returns and array of matches. The second one is used to capture the
+            // values between the tags.
+            const globalHighlightWordRegex = new RegExp(HIGHLIGHT_TAGS_EXP, 'g');
+            const highlightWordRegex = new RegExp(HIGHLIGHT_TAGS_EXP);
+            const newSearchWords = Object.keys(doc.highlight)
+              .flatMap((key) => {
+                return doc.highlight[key].flatMap((line: string) => {
+                  const matchedPhrases = line.match(globalHighlightWordRegex);
+                  if (!matchedPhrases) {
+                    return [];
+                  }
+                  return matchedPhrases.map((part) => {
+                    const matches = part.match(highlightWordRegex);
+                    return (matches && matches[1]) || null;
+                  });
+                });
+              })
+              .filter(identity);
+            // If meta and searchWords already exists, add the words and
+            // deduplicate otherwise create a new set of search words.
+            const searchWords = series.meta?.searchWords
+              ? uniq([...series.meta.searchWords, ...newSearchWords])
+              : [...newSearchWords];
+            series.meta = series.meta ? { ...series.meta, searchWords } : { searchWords };
+          }
+          series.add(doc);
+        }
+
+        const target = this.targets[n];
+        series.refId = target.refId;
+        dataFrame.push(series);
       }
 
       if (response.aggregations) {
@@ -496,7 +587,7 @@ export class ElasticResponse {
 
           // When log results, show aggregations only in graph. Log fields are then going to be shown in table.
           if (isLogsRequest) {
-            series = addPreferredVisualisationType(series, 'graph');
+            addPreferredVisualisationType(series, 'graph');
           }
 
           series.refId = target.refId;
@@ -554,6 +645,7 @@ type Doc = {
   _index: string;
   _source?: any;
   sort?: Array<string | number>;
+  highlight?: Record<string, string[]>;
 };
 
 /**
@@ -575,6 +667,7 @@ const flattenHits = (hits: Doc[]): { docs: Array<Record<string, any>>; propNames
       _type: hit._type,
       _index: hit._index,
       sort: hit.sort,
+      highlight: hit.highlight,
       _source: { ...flattened },
       ...flattened,
     };
@@ -601,61 +694,66 @@ const flattenHits = (hits: Doc[]): { docs: Array<Record<string, any>>; propNames
  * @param logLevelField
  */
 const createEmptyDataFrame = (
-  propNames: string[],
-  timeField: string,
+  props: Array<[string, FieldType]>,
   isLogsRequest: boolean,
+  timeField?: string,
   logMessageField?: string,
   logLevelField?: string
 ): MutableDataFrame => {
   const series = new MutableDataFrame({ fields: [] });
 
-  series.addField({
-    config: {
-      filterable: true,
-    },
-    name: timeField,
-    type: FieldType.time,
-  });
-
-  if (logMessageField) {
-    series.addField({
-      name: logMessageField,
-      type: FieldType.string,
-    }).parse = (v: any) => {
-      return v || '';
-    };
-  }
-
-  if (logLevelField) {
-    series.addField({
-      name: 'level',
-      type: FieldType.string,
-    }).parse = (v: any) => {
-      return v || '';
-    };
-  }
-
-  const fieldNames = series.fields.map((field) => field.name);
-
-  for (const propName of propNames) {
-    // Do not duplicate fields. This can mean that we will shadow some fields.
-    if (fieldNames.includes(propName)) {
-      continue;
-    }
-    // Do not add _source field (besides logs) as we are showing each _source field in table instead.
-    if (!isLogsRequest && propName === '_source') {
-      continue;
-    }
-
+  if (timeField) {
     series.addField({
       config: {
         filterable: true,
       },
-      name: propName,
+      name: timeField,
+      type: FieldType.time,
+    });
+  }
+
+  if (logMessageField) {
+    const f = series.addField({
+      name: logMessageField,
       type: FieldType.string,
-    }).parse = (v: any) => {
+    });
+    series.setParser(f, (v: any) => {
       return v || '';
-    };
+    });
+  }
+
+  if (logLevelField) {
+    const f = series.addField({
+      name: 'level',
+      type: FieldType.string,
+    });
+    series.setParser(f, (v: any) => {
+      return v || '';
+    });
+  }
+
+  const fieldNames = series.fields.map((field) => field.name);
+
+  for (const [name, type] of props) {
+    // Do not duplicate fields. This can mean that we will shadow some fields.
+    if (fieldNames.includes(name)) {
+      continue;
+    }
+    // Do not add _source field (besides logs) as we are showing each _source field in table instead.
+    if (!isLogsRequest && name === '_source') {
+      continue;
+    }
+
+    const f = series.addField({
+      config: {
+        filterable: true,
+      },
+      name,
+      type,
+    });
+    series.setParser(f, (v: any) => {
+      return v || '';
+    });
   }
 
   return series;
@@ -668,6 +766,24 @@ const addPreferredVisualisationType = (series: any, type: PreferredVisualisation
     : (s.meta = {
         preferredVisualisationType: type,
       });
+};
 
-  return s;
+const toNameTypePair =
+  (docs: Array<Record<string, any>>) =>
+  (propName: string): [string, FieldType] =>
+    [propName, guessType(docs.find((doc) => doc[propName] !== undefined)?.[propName])];
+
+/**
+ * Trying to guess data type from its value. This is far from perfect, as in order to have accurate guess
+ * we should have access to the elasticsearch mapping, but it covers the most common use cases for numbers, strings & arrays.
+ */
+const guessType = (value: unknown): FieldType => {
+  switch (typeof value) {
+    case 'number':
+      return FieldType.number;
+    case 'string':
+      return FieldType.string;
+    default:
+      return FieldType.other;
+  }
 };
