@@ -1,8 +1,9 @@
 import { DataFrameView, FieldCache, KeyValue, MutableDataFrame } from '@grafana/data';
-import { ElasticResponse } from '../elastic_response';
 import flatten from 'app/core/utils/flatten';
-import { ElasticsearchQuery } from '../types';
+
+import { ElasticResponse } from '../elastic_response';
 import { highlightTags } from '../query_def';
+import { ElasticsearchQuery } from '../types';
 
 describe('ElasticResponse', () => {
   let targets: ElasticsearchQuery[];
@@ -603,6 +604,75 @@ describe('ElasticResponse', () => {
     });
   });
 
+  describe('with top_metrics', () => {
+    beforeEach(() => {
+      targets = [
+        {
+          refId: 'A',
+          metrics: [
+            {
+              type: 'top_metrics',
+              settings: {
+                order: 'top',
+                orderBy: '@timestamp',
+                metrics: ['@value', '@anotherValue'],
+              },
+              id: '1',
+            },
+          ],
+          bucketAggs: [{ type: 'date_histogram', id: '2' }],
+        },
+      ];
+      response = {
+        responses: [
+          {
+            aggregations: {
+              '2': {
+                buckets: [
+                  {
+                    key: new Date('2021-01-01T00:00:00.000Z').valueOf(),
+                    key_as_string: '2021-01-01T00:00:00.000Z',
+                    '1': {
+                      top: [{ sort: ['2021-01-01T00:00:00.000Z'], metrics: { '@value': 1, '@anotherValue': 2 } }],
+                    },
+                  },
+                  {
+                    key: new Date('2021-01-01T00:00:10.000Z').valueOf(),
+                    key_as_string: '2021-01-01T00:00:10.000Z',
+                    '1': {
+                      top: [{ sort: ['2021-01-01T00:00:10.000Z'], metrics: { '@value': 1, '@anotherValue': 2 } }],
+                    },
+                  },
+                ],
+              },
+            },
+          },
+        ],
+      };
+    });
+
+    it('should return 2 series', () => {
+      const result = new ElasticResponse(targets, response).getTimeSeries();
+      expect(result.data.length).toBe(2);
+
+      const firstSeries = result.data[0];
+      expect(firstSeries.target).toBe('Top Metrics @value');
+      expect(firstSeries.datapoints.length).toBe(2);
+      expect(firstSeries.datapoints).toEqual([
+        [1, new Date('2021-01-01T00:00:00.000Z').valueOf()],
+        [1, new Date('2021-01-01T00:00:10.000Z').valueOf()],
+      ]);
+
+      const secondSeries = result.data[1];
+      expect(secondSeries.target).toBe('Top Metrics @anotherValue');
+      expect(secondSeries.datapoints.length).toBe(2);
+      expect(secondSeries.datapoints).toEqual([
+        [2, new Date('2021-01-01T00:00:00.000Z').valueOf()],
+        [2, new Date('2021-01-01T00:00:10.000Z').valueOf()],
+      ]);
+    });
+  });
+
   describe('single group by with alias pattern', () => {
     let result: any;
 
@@ -1244,6 +1314,7 @@ describe('ElasticResponse', () => {
                 _source: {
                   '@timestamp': '2019-06-24T09:51:19.765Z',
                   host: 'djisaodjsoad',
+                  number: 1,
                   message: 'hello, i am a message',
                   level: 'debug',
                   fields: {
@@ -1263,6 +1334,7 @@ describe('ElasticResponse', () => {
                 _source: {
                   '@timestamp': '2019-06-24T09:52:19.765Z',
                   host: 'dsalkdakdop',
+                  number: 2,
                   message: 'hello, i am also message',
                   level: 'error',
                   fields: {
@@ -1311,7 +1383,7 @@ describe('ElasticResponse', () => {
         expect(r._source).toEqual(
           flatten(
             response.responses[0].hits.hits[i]._source,
-            (null as unknown) as { delimiter?: any; maxDepth?: any; safe?: any }
+            null as unknown as { delimiter?: any; maxDepth?: any; safe?: any }
           )
         );
       }
@@ -1344,5 +1416,56 @@ describe('ElasticResponse', () => {
       const field = fieldCache.getFieldByName('level');
       expect(field?.values.toArray()).toEqual(['debug', 'info']);
     });
+
+    it('should correctly guess field types', () => {
+      const result = new ElasticResponse(targets, response).getLogs();
+      const logResults = result.data[0] as MutableDataFrame;
+
+      const fields = logResults.fields.map((f) => {
+        return {
+          name: f.name,
+          type: f.type,
+        };
+      });
+
+      expect(fields).toContainEqual({ name: '@timestamp', type: 'time' });
+      expect(fields).toContainEqual({ name: 'number', type: 'number' });
+      expect(fields).toContainEqual({ name: 'message', type: 'string' });
+    });
+  });
+
+  describe('logs query with empty response', () => {
+    const targets: ElasticsearchQuery[] = [
+      {
+        refId: 'A',
+        metrics: [{ type: 'logs', id: '2' }],
+        bucketAggs: [{ type: 'date_histogram', settings: { interval: 'auto' }, id: '1' }],
+        key: 'Q-1561369883389-0.7611823271062786-0',
+        query: 'hello AND message',
+        timeField: '@timestamp',
+      },
+    ];
+    const response = {
+      responses: [
+        {
+          hits: { hits: [] },
+          aggregations: {
+            '1': {
+              buckets: [
+                { key_as_string: '1633676760000', key: 1633676760000, doc_count: 0 },
+                { key_as_string: '1633676770000', key: 1633676770000, doc_count: 0 },
+                { key_as_string: '1633676780000', key: 1633676780000, doc_count: 0 },
+              ],
+            },
+          },
+          status: 200,
+        },
+      ],
+    };
+
+    it('should return histogram aggregation and documents', () => {
+      const result = new ElasticResponse(targets, response).getLogs('message', 'level');
+      expect(result.data.length).toBe(2);
+    });
   });
 });
