@@ -1,27 +1,45 @@
-import _ from 'lodash';
-import { isVersionGtOrEq } from 'app/core/utils/version';
-import { InterpolateFunction } from '@grafana/data';
+import { assign, each, filter, forEach, get, includes, isString, last, map, toString, isFinite } from 'lodash';
 
-const index: any = {};
+import { InterpolateFunction } from '@grafana/data';
+import { isVersionGtOrEq } from 'app/core/utils/version';
+
+export type ParamDef = {
+  name: string;
+  type: string;
+  options?: Array<string | number>;
+  multiple?: boolean;
+  optional?: boolean;
+  version?: string;
+};
 
 export interface FuncDef {
-  name: any;
+  name: string;
+  params: ParamDef[];
+  defaultParams: Array<string | number>;
   category?: string;
-  params?: any;
-  defaultParams?: any;
   shortName?: any;
   fake?: boolean;
   version?: string;
   description?: string;
+  /**
+   * True if the function was not found on the list of available function descriptions.
+   */
+  unknown?: boolean;
 }
 
-function addFuncDef(funcDef: FuncDef) {
+export type FuncDefs = {
+  [functionName in string]: FuncDef;
+};
+
+const index: FuncDefs = {};
+
+function addFuncDef(funcDef: Partial<FuncDef> & { name: string; category: string }) {
   funcDef.params = funcDef.params || [];
   funcDef.defaultParams = funcDef.defaultParams || [];
 
-  index[funcDef.name] = funcDef;
+  index[funcDef.name] = funcDef as FuncDef;
   if (funcDef.shortName) {
-    index[funcDef.shortName] = funcDef;
+    index[funcDef.shortName] = funcDef as FuncDef;
   }
 }
 
@@ -965,21 +983,32 @@ addFuncDef({
   version: '1.1',
 });
 
-function isVersionRelatedFunction(obj: { version: string }, graphiteVersion: string) {
+function isVersionRelatedFunction(obj: { version?: string }, graphiteVersion: string) {
   return !obj.version || isVersionGtOrEq(graphiteVersion, obj.version);
 }
 
 export class FuncInstance {
-  def: any;
-  params: any;
+  def: FuncDef;
+  params: Array<string | number>;
   text: any;
-  added: boolean;
+  /**
+   * True if this function was just added and not edited yet. It's used to focus on first
+   * function param to edit it straight away after adding a function.
+   */
+  declare added: boolean;
+  /**
+   * Hidden functions are not displayed in UI but available in text editor
+   * This is used for seriesByTagUsed function which when used switches
+   * the editor to tag-only mode. Defined tags are provided to seriesByTagUsed
+   * as parameters.
+   */
+  hidden?: boolean;
 
-  constructor(funcDef: any, options?: { withDefaultParams: any }) {
+  constructor(funcDef: FuncDef, options?: { withDefaultParams: any }) {
     this.def = funcDef;
     this.params = [];
 
-    if (options && options.withDefaultParams) {
+    if (options && options.withDefaultParams && funcDef.defaultParams) {
       this.params = funcDef.defaultParams.slice(0);
     }
 
@@ -989,26 +1018,26 @@ export class FuncInstance {
   render(metricExp: string, replaceVariables: InterpolateFunction): string {
     const str = this.def.name + '(';
 
-    const parameters = _.map(this.params, (value, index) => {
+    const parameters = map(this.params, (value, index) => {
       let paramType;
 
       if (index < this.def.params.length) {
         paramType = this.def.params[index].type;
-      } else if (_.get(_.last(this.def.params), 'multiple')) {
-        paramType = _.get(_.last(this.def.params), 'type');
+      } else if (get(last(this.def.params), 'multiple')) {
+        paramType = get(last(this.def.params), 'type');
       }
 
       // param types that should never be quoted
-      if (_.includes(['value_or_series', 'boolean', 'int', 'float', 'node'], paramType)) {
+      if (includes(['value_or_series', 'boolean', 'int', 'float', 'node', 'int_or_infinity'], paramType)) {
         return value;
       }
 
-      const valueInterpolated = _.isString(value) ? replaceVariables(value) : value;
+      const valueInterpolated = isString(value) ? replaceVariables(value) : value;
 
       // param types that might be quoted
       // To quote variables correctly we need to interpolate it to check if it contains a numeric or string value
-      if (_.includes(['int_or_interval', 'node_or_tag'], paramType) && _.isFinite(+valueInterpolated)) {
-        return _.toString(value);
+      if (includes(['int_or_interval', 'node_or_tag'], paramType) && isFinite(+valueInterpolated)) {
+        return toString(value);
       }
 
       return "'" + value + "'";
@@ -1035,7 +1064,7 @@ export class FuncInstance {
       return true;
     }
 
-    if (index + 1 >= this.def.params.length && _.get(_.last(this.def.params), 'multiple')) {
+    if (index + 1 >= this.def.params.length && get(last(this.def.params), 'multiple')) {
       return true;
     }
 
@@ -1046,7 +1075,7 @@ export class FuncInstance {
     // handle optional parameters
     // if string contains ',' and next param is optional, split and update both
     if (this._hasMultipleParamsInString(strValue, index)) {
-      _.each(strValue.split(','), (partVal, idx) => {
+      each(strValue.split(','), (partVal, idx) => {
         this.updateParam(partVal.trim(), index + idx);
       });
       return;
@@ -1074,26 +1103,26 @@ export class FuncInstance {
   }
 }
 
-function createFuncInstance(funcDef: any, options?: { withDefaultParams: any }, idx?: any) {
-  if (_.isString(funcDef)) {
+function createFuncInstance(funcDef: any, options?: { withDefaultParams: any }, idx?: any): FuncInstance {
+  if (isString(funcDef)) {
     funcDef = getFuncDef(funcDef, idx);
   }
   return new FuncInstance(funcDef, options);
 }
 
-function getFuncDef(name: string, idx?: any) {
+function getFuncDef(name: string, idx?: any): FuncDef {
   if (!(idx || index)[name]) {
-    throw { message: 'Method not found ' + name };
+    return { name: name, params: [{ name: '', type: '', multiple: true }], defaultParams: [''], unknown: true };
   }
   return (idx || index)[name];
 }
 
-function getFuncDefs(graphiteVersion: string, idx?: any) {
-  const funcs: any = {};
-  _.forEach(idx || index, (funcDef) => {
+function getFuncDefs(graphiteVersion: string, idx?: any): FuncDefs {
+  const funcs: FuncDefs = {};
+  forEach(idx || index, (funcDef: FuncDef) => {
     if (isVersionRelatedFunction(funcDef, graphiteVersion)) {
-      funcs[funcDef.name] = _.assign({}, funcDef, {
-        params: _.filter(funcDef.params, (param) => {
+      funcs[funcDef.name] = assign({}, funcDef, {
+        params: filter(funcDef.params, (param) => {
           return isVersionRelatedFunction(param, graphiteVersion);
         }),
       });
@@ -1103,10 +1132,10 @@ function getFuncDefs(graphiteVersion: string, idx?: any) {
 }
 
 // parse response from graphite /functions endpoint into internal format
-function parseFuncDefs(rawDefs: any) {
-  const funcDefs: any = {};
+function parseFuncDefs(rawDefs: any): FuncDefs {
+  const funcDefs: FuncDefs = {};
 
-  _.forEach(rawDefs || {}, (funcDef, funcName) => {
+  forEach(rawDefs || {}, (funcDef, funcName) => {
     // skip graphite graph functions
     if (funcDef.group === 'Graph') {
       return;
@@ -1131,7 +1160,7 @@ function parseFuncDefs(rawDefs: any) {
     };
 
     // get rid of the first "seriesList" param
-    if (/^seriesLists?$/.test(_.get(funcDef, 'params[0].type', ''))) {
+    if (/^seriesLists?$/.test(get(funcDef, 'params[0].type', ''))) {
       // handle functions that accept multiple seriesLists
       // we leave the param in place but mark it optional, so users can add more series if they wish
       if (funcDef.params[0].multiple) {
@@ -1145,7 +1174,7 @@ function parseFuncDefs(rawDefs: any) {
       func.fake = true;
     }
 
-    _.forEach(funcDef.params, (rawParam) => {
+    forEach(funcDef.params, (rawParam) => {
       const param: any = {
         name: rawParam.name,
         type: 'string',
@@ -1155,9 +1184,13 @@ function parseFuncDefs(rawDefs: any) {
       };
 
       if (rawParam.default !== undefined) {
-        func.defaultParams.push(_.toString(rawParam.default));
+        if (rawParam.default === Infinity) {
+          func.defaultParams.push('inf');
+        } else {
+          func.defaultParams.push(toString(rawParam.default));
+        }
       } else if (rawParam.suggestions) {
-        func.defaultParams.push(_.toString(rawParam.suggestions[0]));
+        func.defaultParams.push(toString(rawParam.suggestions[0]));
       } else {
         func.defaultParams.push('');
       }
@@ -1179,12 +1212,14 @@ function parseFuncDefs(rawDefs: any) {
         param.type = 'int_or_interval';
       } else if (rawParam.type === 'seriesList') {
         param.type = 'value_or_series';
+      } else if (rawParam.type === 'intOrInf') {
+        param.type = 'int_or_infinity';
       }
 
       if (rawParam.options) {
-        param.options = _.map(rawParam.options, _.toString);
+        param.options = map(rawParam.options, toString);
       } else if (rawParam.suggestions) {
-        param.options = _.map(rawParam.suggestions, _.toString);
+        param.options = map(rawParam.suggestions, toString);
       }
 
       func.params.push(param);
