@@ -1,15 +1,16 @@
-import { GraphiteDatasource } from './datasource';
-import { GraphiteQueryCtrl } from './query_ctrl';
 import { DataSourcePlugin } from '@grafana/data';
+
+import { GraphiteQueryEditor } from './components/GraphiteQueryEditor';
+import { MetricTankMetaInspector } from './components/MetricTankMetaInspector';
 import { ConfigEditor } from './configuration/ConfigEditor';
-import { MetricTankMetaInspector } from './MetricTankMetaInspector';
+import { GraphiteDatasource } from './datasource';
 
 class AnnotationsQueryCtrl {
   static templateUrl = 'partials/annotations.editor.html';
 }
 
 export const plugin = new DataSourcePlugin(GraphiteDatasource)
-  .setQueryCtrl(GraphiteQueryCtrl)
+  .setQueryEditor(GraphiteQueryEditor)
   .setConfigEditor(ConfigEditor)
   .setMetadataInspector(MetricTankMetaInspector)
   .setAnnotationQueryCtrl(AnnotationsQueryCtrl);
