@@ -1,11 +1,24 @@
-import React from 'react';
 import { shallow } from 'enzyme';
+import React from 'react';
+
 import ConfigEditor, { Props } from './ConfigEditor';
 
+jest.mock('lodash', () => {
+  const uniqueId = (prefix: string) => `${prefix}42`;
+
+  const orig = jest.requireActual('lodash');
+
+  return {
+    ...orig,
+    uniqueId,
+  };
+});
+
 const setup = (propOverrides?: object) => {
   const props: Props = {
     options: {
       id: 21,
+      uid: 'z',
       orgId: 1,
       name: 'InfluxDB-3',
       type: 'influxdb',
@@ -13,12 +26,10 @@ const setup = (propOverrides?: object) => {
       typeLogoUrl: '',
       access: 'proxy',
       url: '',
-      password: '',
       user: '',
       database: '',
       basicAuth: false,
       basicAuthUser: '',
-      basicAuthPassword: '',
       withCredentials: false,
       isDefault: false,
       jsonData: {
