@@ -1,4 +1,6 @@
+import { uniqueId } from 'lodash';
 import React, { PureComponent } from 'react';
+
 import {
   DataSourcePluginOptionsEditorProps,
   SelectableValue,
@@ -9,8 +11,9 @@ import {
   onUpdateDatasourceSecureJsonDataOption,
   updateDatasourcePluginJsonDataOption,
 } from '@grafana/data';
-import { DataSourceHttpSettings, InfoBox, InlineField, InlineFormLabel, LegacyForms } from '@grafana/ui';
-const { Select, Input, SecretFormField } = LegacyForms;
+import { Alert, DataSourceHttpSettings, InfoBox, InlineField, InlineFormLabel, LegacyForms, Select } from '@grafana/ui';
+
+const { Input, SecretFormField } = LegacyForms;
 import { InfluxOptions, InfluxSecureJsonData, InfluxVersion } from '../types';
 
 const httpModes = [
@@ -22,12 +25,12 @@ const versions = [
   {
     label: 'InfluxQL',
     value: InfluxVersion.InfluxQL,
-    description: 'The InfluxDB SQL-like query language.  Supported in InfluxDB 1.x',
+    description: 'The InfluxDB SQL-like query language.',
   },
   {
     label: 'Flux',
     value: InfluxVersion.Flux,
-    description: 'Advanced data scripting and query language.  Supported in InfluxDB 2.x and 1.8+ (beta)',
+    description: 'Advanced data scripting and query language.  Supported in InfluxDB 2.x and 1.8+',
   },
 ] as Array<SelectableValue<InfluxVersion>>;
 
@@ -41,9 +44,12 @@ export class ConfigEditor extends PureComponent<Props, State> {
     maxSeries: '',
   };
 
+  htmlPrefix: string;
+
   constructor(props: Props) {
     super(props);
     this.state.maxSeries = props.options.jsonData.maxSeries?.toString() || '';
+    this.htmlPrefix = uniqueId('influxdb-config');
   }
 
   // 1x
@@ -83,14 +89,18 @@ export class ConfigEditor extends PureComponent<Props, State> {
     const { options } = this.props;
     const { secureJsonFields } = options;
     const secureJsonData = (options.secureJsonData || {}) as InfluxSecureJsonData;
+    const { htmlPrefix } = this;
 
     return (
       <>
         <div className="gf-form-inline">
           <div className="gf-form">
-            <InlineFormLabel className="width-10">Organization</InlineFormLabel>
+            <InlineFormLabel htmlFor={`${htmlPrefix}-org`} className="width-10">
+              Organization
+            </InlineFormLabel>
             <div className="width-10">
               <Input
+                id={`${htmlPrefix}-org`}
                 className="width-20"
                 value={options.jsonData.organization || ''}
                 onChange={onUpdateDatasourceJsonDataOption(this.props, 'organization')}
@@ -104,6 +114,7 @@ export class ConfigEditor extends PureComponent<Props, State> {
               isConfigured={(secureJsonFields && secureJsonFields.token) as boolean}
               value={secureJsonData.token || ''}
               label="Token"
+              aria-label="Token"
               labelWidth={10}
               inputWidth={20}
               onReset={this.onResetToken}
@@ -152,6 +163,7 @@ export class ConfigEditor extends PureComponent<Props, State> {
     const { options } = this.props;
     const { secureJsonFields } = options;
     const secureJsonData = (options.secureJsonData || {}) as InfluxSecureJsonData;
+    const { htmlPrefix } = this;
 
     return (
       <>
@@ -169,9 +181,12 @@ export class ConfigEditor extends PureComponent<Props, State> {
         </InfoBox>
         <div className="gf-form-inline">
           <div className="gf-form">
-            <InlineFormLabel className="width-10">Database</InlineFormLabel>
+            <InlineFormLabel htmlFor={`${htmlPrefix}-db`} className="width-10">
+              Database
+            </InlineFormLabel>
             <div className="width-20">
               <Input
+                id={`${htmlPrefix}-db`}
                 className="width-20"
                 value={options.database || ''}
                 onChange={onUpdateDatasourceOption(this.props, 'database')}
@@ -181,9 +196,12 @@ export class ConfigEditor extends PureComponent<Props, State> {
         </div>
         <div className="gf-form-inline">
           <div className="gf-form">
-            <InlineFormLabel className="width-10">User</InlineFormLabel>
+            <InlineFormLabel htmlFor={`${htmlPrefix}-user`} className="width-10">
+              User
+            </InlineFormLabel>
             <div className="width-10">
               <Input
+                id={`${htmlPrefix}-user`}
                 className="width-20"
                 value={options.user || ''}
                 onChange={onUpdateDatasourceOption(this.props, 'user')}
@@ -197,6 +215,7 @@ export class ConfigEditor extends PureComponent<Props, State> {
               isConfigured={(secureJsonFields && secureJsonFields.password) as boolean}
               value={secureJsonData.password || ''}
               label="Password"
+              aria-label="Password"
               labelWidth={10}
               inputWidth={20}
               onReset={this.onResetPassword}
@@ -207,6 +226,7 @@ export class ConfigEditor extends PureComponent<Props, State> {
         <div className="gf-form-inline">
           <div className="gf-form">
             <InlineFormLabel
+              htmlFor={`${htmlPrefix}-http-method`}
               className="width-10"
               tooltip="You can use either GET or POST HTTP method to query your InfluxDB database. The POST
           method allows you to perform heavy requests (with a lots of WHERE clause) while the GET method
@@ -215,6 +235,7 @@ export class ConfigEditor extends PureComponent<Props, State> {
               HTTP Method
             </InlineFormLabel>
             <Select
+              inputId={`${htmlPrefix}-http-method`}
               className="width-10"
               value={httpModes.find((httpMode) => httpMode.value === options.jsonData.httpMode)}
               options={httpModes}
@@ -257,6 +278,7 @@ export class ConfigEditor extends PureComponent<Props, State> {
           <div className="gf-form-inline">
             <div className="gf-form">
               <Select
+                aria-label="Query language"
                 className="width-30"
                 value={options.jsonData.version === InfluxVersion.Flux ? versions[1] : versions[0]}
                 options={versions}
@@ -279,6 +301,12 @@ export class ConfigEditor extends PureComponent<Props, State> {
           </InfoBox>
         )}
 
+        {options.access === 'direct' && (
+          <Alert title="Deprecation Notice" severity="warning">
+            Browser access mode in the InfluxDB datasource is deprecated and will be removed in a future release.
+          </Alert>
+        )}
+
         <DataSourceHttpSettings
           showAccessOptions={true}
           dataSourceConfig={options}
