@@ -1,12 +1,10 @@
-import InfluxDatasource from './datasource';
-import { InfluxQueryCtrl } from './query_ctrl';
-import InfluxStartPage from './components/InfluxStartPage';
 import { DataSourcePlugin } from '@grafana/data';
-import ConfigEditor from './components/ConfigEditor';
-import VariableQueryEditor from './components/VariableQueryEditor';
 
-// This adds a directive that is used in the query editor
-import './components/FluxQueryEditor';
+import ConfigEditor from './components/ConfigEditor';
+import InfluxStartPage from './components/InfluxStartPage';
+import { QueryEditor } from './components/QueryEditor';
+import VariableQueryEditor from './components/VariableQueryEditor';
+import InfluxDatasource from './datasource';
 
 class InfluxAnnotationsQueryCtrl {
   static templateUrl = 'partials/annotations.editor.html';
@@ -14,7 +12,7 @@ class InfluxAnnotationsQueryCtrl {
 
 export const plugin = new DataSourcePlugin(InfluxDatasource)
   .setConfigEditor(ConfigEditor)
-  .setQueryCtrl(InfluxQueryCtrl)
+  .setQueryEditor(QueryEditor)
   .setAnnotationQueryCtrl(InfluxAnnotationsQueryCtrl)
   .setVariableQueryEditor(VariableQueryEditor)
   .setQueryEditorHelp(InfluxStartPage);
