@@ -1,28 +1,32 @@
+import { css } from '@emotion/css';
 import React, { useState } from 'react';
-import { css } from 'emotion';
-import { Button, DataLinkBuiltInVars, stylesFactory, useTheme } from '@grafana/ui';
-import { GrafanaTheme, VariableOrigin } from '@grafana/data';
-import { DerivedFieldConfig } from '../types';
-import { DerivedField } from './DerivedField';
-import { DebugSection } from './DebugSection';
 
-const getStyles = stylesFactory((theme: GrafanaTheme) => ({
+import { GrafanaTheme2, VariableOrigin, DataLinkBuiltInVars } from '@grafana/data';
+import { Button, useTheme2 } from '@grafana/ui';
+
+import { DerivedFieldConfig } from '../types';
+
+import { DebugSection } from './DebugSection';
+import { DerivedField } from './DerivedField';
+
+const getStyles = (theme: GrafanaTheme2) => ({
   infoText: css`
-    padding-bottom: ${theme.spacing.md};
-    color: ${theme.colors.textWeak};
+    padding-bottom: ${theme.spacing(2)};
+    color: ${theme.colors.text.secondary};
   `,
   derivedField: css`
-    margin-bottom: ${theme.spacing.sm};
+    margin-bottom: ${theme.spacing(1)};
   `,
-}));
+});
 
 type Props = {
   value?: DerivedFieldConfig[];
   onChange: (value: DerivedFieldConfig[]) => void;
 };
+
 export const DerivedFields = (props: Props) => {
   const { value, onChange } = props;
-  const theme = useTheme();
+  const theme = useTheme2();
   const styles = getStyles(theme);
 
   const [showDebug, setShowDebug] = useState(false);
@@ -32,7 +36,7 @@ export const DerivedFields = (props: Props) => {
       <h3 className="page-heading">Derived fields</h3>
 
       <div className={styles.infoText}>
-        Derived fields can be used to extract new fields from the log message and create link from it's value.
+        Derived fields can be used to extract new fields from a log message and create a link from its value.
       </div>
 
       <div className="gf-form-group">
@@ -43,7 +47,7 @@ export const DerivedFields = (props: Props) => {
                 className={styles.derivedField}
                 key={index}
                 value={field}
-                onChange={newField => {
+                onChange={(newField) => {
                   const newDerivedFields = [...value];
                   newDerivedFields.splice(index, 1, newField);
                   onChange(newDerivedFields);
@@ -66,12 +70,12 @@ export const DerivedFields = (props: Props) => {
           })}
         <div>
           <Button
-            variant={'inverse'}
+            variant="secondary"
             className={css`
               margin-right: 10px;
             `}
-            icon="fa fa-plus"
-            onClick={event => {
+            icon="plus"
+            onClick={(event) => {
               event.preventDefault();
               const newDerivedFields = [...(value || []), { name: '', matcherRegex: '' }];
               onChange(newDerivedFields);
@@ -81,7 +85,7 @@ export const DerivedFields = (props: Props) => {
           </Button>
 
           {value && value.length > 0 && (
-            <Button variant="inverse" onClick={() => setShowDebug(!showDebug)}>
+            <Button variant="secondary" type="button" onClick={() => setShowDebug(!showDebug)}>
               {showDebug ? 'Hide example log message' : 'Show example log message'}
             </Button>
           )}
