@@ -7,18 +7,27 @@ import {
 export class MssqlConfigCtrl {
   static templateUrl = 'partials/config.html';
 
-  current: any;
+  // Set through angular bindings
+  declare current: any;
+
   onPasswordReset: ReturnType<typeof createResetHandler>;
   onPasswordChange: ReturnType<typeof createChangeHandler>;
-  showUserCredentials: boolean;
+  showUserCredentials = false;
+  showTlsConfig = false;
+  showCertificateConfig = false;
 
   /** @ngInject */
   constructor($scope: any) {
+    this.current = $scope.ctrl.current;
     this.current.jsonData.encrypt = this.current.jsonData.encrypt || 'false';
+    this.current.jsonData.sslRootCertFile = this.current.jsonData.sslRootCertFile || '';
+    this.current.jsonData.tlsSkipVerify = this.current.jsonData.tlsSkipVerify || false;
+    this.current.jsonData.serverName = this.current.jsonData.serverName || '';
     this.current.jsonData.authenticationType = this.current.jsonData.authenticationType || 'SQL Server Authentication';
     this.onPasswordReset = createResetHandler(this, PasswordFieldEnum.Password);
     this.onPasswordChange = createChangeHandler(this, PasswordFieldEnum.Password);
-    this.showUserCredentials = this.current.jsonData.authenticationType !== 'Windows Authentication';
+    this.onAuthenticationTypeChange();
+    this.onEncryptChange();
   }
 
   onAuthenticationTypeChange() {
@@ -30,4 +39,9 @@ export class MssqlConfigCtrl {
 
     this.showUserCredentials = this.current.jsonData.authenticationType !== 'Windows Authentication';
   }
+
+  onEncryptChange() {
+    this.showTlsConfig = this.current.jsonData.encrypt === 'true';
+    this.showCertificateConfig = this.showTlsConfig && this.current.jsonData.tlsSkipVerify === false;
+  }
 }
