@@ -1,9 +1,9 @@
+import { DataSourcePlugin } from '@grafana/data';
+
+import { MssqlConfigCtrl } from './config_ctrl';
 import { MssqlDatasource } from './datasource';
 import { MssqlQueryCtrl } from './query_ctrl';
-
-class MssqlConfigCtrl {
-  static templateUrl = 'partials/config.html';
-}
+import { MssqlQuery } from './types';
 
 const defaultQuery = `SELECT
     <time_column> as time,
@@ -19,18 +19,16 @@ const defaultQuery = `SELECT
 class MssqlAnnotationsQueryCtrl {
   static templateUrl = 'partials/annotations.editor.html';
 
-  annotation: any;
+  declare annotation: any;
 
-  /** @ngInject **/
-  constructor() {
+  /** @ngInject */
+  constructor($scope: any) {
+    this.annotation = $scope.ctrl.annotation;
     this.annotation.rawQuery = this.annotation.rawQuery || defaultQuery;
   }
 }
 
-export {
-  MssqlDatasource,
-  MssqlDatasource as Datasource,
-  MssqlQueryCtrl as QueryCtrl,
-  MssqlConfigCtrl as ConfigCtrl,
-  MssqlAnnotationsQueryCtrl as AnnotationsQueryCtrl,
-};
+export const plugin = new DataSourcePlugin<MssqlDatasource, MssqlQuery>(MssqlDatasource)
+  .setQueryCtrl(MssqlQueryCtrl)
+  .setConfigCtrl(MssqlConfigCtrl)
+  .setAnnotationQueryCtrl(MssqlAnnotationsQueryCtrl);
