@@ -1,6 +1,9 @@
+import { DataSourcePlugin } from '@grafana/data';
+
+import { PostgresConfigCtrl } from './config_ctrl';
 import { PostgresDatasource } from './datasource';
 import { PostgresQueryCtrl } from './query_ctrl';
-import { PostgresConfigCtrl } from './config_ctrl';
+import { PostgresQuery } from './types';
 
 const defaultQuery = `SELECT
   extract(epoch from time_column) AS time,
@@ -15,18 +18,16 @@ WHERE
 class PostgresAnnotationsQueryCtrl {
   static templateUrl = 'partials/annotations.editor.html';
 
-  annotation: any;
+  declare annotation: any;
 
   /** @ngInject */
-  constructor() {
+  constructor($scope: any) {
+    this.annotation = $scope.ctrl.annotation;
     this.annotation.rawQuery = this.annotation.rawQuery || defaultQuery;
   }
 }
 
-export {
-  PostgresDatasource,
-  PostgresDatasource as Datasource,
-  PostgresQueryCtrl as QueryCtrl,
-  PostgresConfigCtrl as ConfigCtrl,
-  PostgresAnnotationsQueryCtrl as AnnotationsQueryCtrl,
-};
+export const plugin = new DataSourcePlugin<PostgresDatasource, PostgresQuery>(PostgresDatasource)
+  .setQueryCtrl(PostgresQueryCtrl)
+  .setConfigCtrl(PostgresConfigCtrl)
+  .setAnnotationQueryCtrl(PostgresAnnotationsQueryCtrl);
