@@ -1,15 +1,21 @@
-import _ from 'lodash';
+import { cloneDeep } from 'lodash';
 import { of, throwError } from 'rxjs';
+
 import {
   CoreApp,
   DataQueryRequest,
   DataQueryResponseData,
   DataSourceInstanceSettings,
   dateTime,
+  Field,
   getFieldDisplayName,
   LoadingState,
   toDataFrame,
 } from '@grafana/data';
+import { QueryOptions } from 'app/types';
+
+import { describe } from '../../../../test/lib/common';
+import { VariableHide } from '../../../features/variables/types';
 
 import {
   alignRange,
@@ -19,9 +25,6 @@ import {
   prometheusSpecialRegexEscape,
 } from './datasource';
 import { PromOptions, PromQuery } from './types';
-import { VariableHide } from '../../../features/variables/types';
-import { describe } from '../../../../test/lib/common';
-import { QueryOptions } from 'app/types';
 
 const fetchMock = jest.fn().mockReturnValue(of(createDefaultPromResponse()));
 
@@ -54,15 +57,16 @@ beforeEach(() => {
 
 describe('PrometheusDatasource', () => {
   let ds: PrometheusDatasource;
-  const instanceSettings = ({
+  const instanceSettings = {
     url: 'proxied',
+    id: 1,
     directUrl: 'direct',
     user: 'test',
     password: 'mupp',
     jsonData: {
       customQueryParameters: '',
     } as any,
-  } as unknown) as DataSourceInstanceSettings<PromOptions>;
+  } as unknown as DataSourceInstanceSettings<PromOptions>;
 
   beforeEach(() => {
     ds = new PrometheusDatasource(instanceSettings, templateSrvStub as any, timeSrvStub as any);
@@ -101,42 +105,121 @@ describe('PrometheusDatasource', () => {
 
   describe('Datasource metadata requests', () => {
     it('should perform a GET request with the default config', () => {
-      ds.metadataRequest('/foo');
+      ds.metadataRequest('/foo', { bar: 'baz baz', foo: 'foo' });
       expect(fetchMock.mock.calls.length).toBe(1);
       expect(fetchMock.mock.calls[0][0].method).toBe('GET');
+      expect(fetchMock.mock.calls[0][0].url).toContain('bar=baz%20baz&foo=foo');
     });
-
-    it('should still perform a GET request with the DS HTTP method set to POST', () => {
-      const postSettings = _.cloneDeep(instanceSettings);
+    it('should still perform a GET request with the DS HTTP method set to POST and not POST-friendly endpoint', () => {
+      const postSettings = cloneDeep(instanceSettings);
       postSettings.jsonData.httpMethod = 'POST';
       const promDs = new PrometheusDatasource(postSettings, templateSrvStub as any, timeSrvStub as any);
       promDs.metadataRequest('/foo');
       expect(fetchMock.mock.calls.length).toBe(1);
       expect(fetchMock.mock.calls[0][0].method).toBe('GET');
     });
+    it('should try to perform a POST request with the DS HTTP method set to POST and POST-friendly endpoint', () => {
+      const postSettings = cloneDeep(instanceSettings);
+      postSettings.jsonData.httpMethod = 'POST';
+      const promDs = new PrometheusDatasource(postSettings, templateSrvStub as any, timeSrvStub as any);
+      promDs.metadataRequest('api/v1/series', { bar: 'baz baz', foo: 'foo' });
+      expect(fetchMock.mock.calls.length).toBe(1);
+      expect(fetchMock.mock.calls[0][0].method).toBe('POST');
+      expect(fetchMock.mock.calls[0][0].url).not.toContain('bar=baz%20baz&foo=foo');
+      expect(fetchMock.mock.calls[0][0].data).toEqual({ bar: 'baz baz', foo: 'foo' });
+    });
   });
 
-  describe('When using customQueryParams', () => {
-    const promDs = new PrometheusDatasource(
-      { ...instanceSettings, jsonData: { customQueryParameters: 'customQuery=123' } as any },
-      templateSrvStub as any,
-      timeSrvStub as any
-    );
-    it('added to metadata request', () => {
-      promDs.metadataRequest('/foo');
-      expect(fetchMock.mock.calls.length).toBe(1);
-      expect(fetchMock.mock.calls[0][0].url).toBe('proxied/foo?customQuery=123');
-    });
-    it('added to query', () => {
-      promDs.query({
+  describe('customQueryParams', () => {
+    const target = { expr: 'test{job="testjob"}', format: 'time_series', refId: '' };
+    function makeQuery(target: PromQuery) {
+      return {
         range: { from: time({ seconds: 63 }), to: time({ seconds: 183 }) },
-        targets: [{ expr: 'test{job="testjob"}', format: 'time_series' }],
+        targets: [target],
         interval: '60s',
-      } as any);
-      expect(fetchMock.mock.calls.length).toBe(1);
-      expect(fetchMock.mock.calls[0][0].url).toBe(
-        'proxied/api/v1/query_range?query=test%7Bjob%3D%22testjob%22%7D&start=60&end=180&step=60&customQuery=123'
+      } as any;
+    }
+
+    describe('with GET http method', () => {
+      const promDs = new PrometheusDatasource(
+        { ...instanceSettings, jsonData: { customQueryParameters: 'customQuery=123', httpMethod: 'GET' } as any },
+        templateSrvStub as any,
+        timeSrvStub as any
       );
+
+      it('added to metadata request', () => {
+        promDs.metadataRequest('/foo');
+        expect(fetchMock.mock.calls.length).toBe(1);
+        expect(fetchMock.mock.calls[0][0].url).toBe('/api/datasources/1/resources/foo?customQuery=123');
+      });
+
+      it('adds params to timeseries query', () => {
+        promDs.query(makeQuery(target));
+        expect(fetchMock.mock.calls.length).toBe(1);
+        expect(fetchMock.mock.calls[0][0].url).toBe(
+          'proxied/api/v1/query_range?query=test%7Bjob%3D%22testjob%22%7D&start=60&end=180&step=60&customQuery=123'
+        );
+      });
+      it('adds params to exemplars query', () => {
+        promDs.query(makeQuery({ ...target, exemplar: true }));
+        // We do also range query for single exemplars target
+        expect(fetchMock.mock.calls.length).toBe(2);
+        expect(fetchMock.mock.calls[0][0].url).toContain('&customQuery=123');
+        expect(fetchMock.mock.calls[1][0].url).toContain('&customQuery=123');
+      });
+
+      it('adds params to instant query', () => {
+        promDs.query(makeQuery({ ...target, instant: true }));
+        expect(fetchMock.mock.calls.length).toBe(1);
+        expect(fetchMock.mock.calls[0][0].url).toContain('&customQuery=123');
+      });
+    });
+
+    describe('with POST http method', () => {
+      const promDs = new PrometheusDatasource(
+        { ...instanceSettings, jsonData: { customQueryParameters: 'customQuery=123', httpMethod: 'POST' } as any },
+        templateSrvStub as any,
+        timeSrvStub as any
+      );
+
+      it('added to metadata request with non-POST endpoint', () => {
+        promDs.metadataRequest('/foo');
+        expect(fetchMock.mock.calls.length).toBe(1);
+        expect(fetchMock.mock.calls[0][0].url).toBe('/api/datasources/1/resources/foo?customQuery=123');
+      });
+
+      it('added to metadata request with POST endpoint', () => {
+        promDs.metadataRequest('/api/v1/labels');
+        expect(fetchMock.mock.calls.length).toBe(1);
+        expect(fetchMock.mock.calls[0][0].url).toBe('/api/datasources/1/resources/api/v1/labels');
+        expect(fetchMock.mock.calls[0][0].data.customQuery).toBe('123');
+      });
+
+      it('adds params to timeseries query', () => {
+        promDs.query(makeQuery(target));
+        expect(fetchMock.mock.calls.length).toBe(1);
+        expect(fetchMock.mock.calls[0][0].url).toBe('proxied/api/v1/query_range');
+        expect(fetchMock.mock.calls[0][0].data).toEqual({
+          customQuery: '123',
+          query: 'test{job="testjob"}',
+          step: 60,
+          end: 180,
+          start: 60,
+        });
+      });
+      it('adds params to exemplars query', () => {
+        promDs.query(makeQuery({ ...target, exemplar: true }));
+        // We do also range query for single exemplars target
+        expect(fetchMock.mock.calls.length).toBe(2);
+        expect(fetchMock.mock.calls[0][0].data.customQuery).toBe('123');
+        expect(fetchMock.mock.calls[1][0].data.customQuery).toBe('123');
+      });
+
+      it('adds params to instant query', () => {
+        promDs.query(makeQuery({ ...target, instant: true }));
+        expect(fetchMock.mock.calls.length).toBe(1);
+        expect(fetchMock.mock.calls[0][0].data.customQuery).toBe('123');
+      });
     });
   });
 
@@ -168,7 +251,7 @@ describe('PrometheusDatasource', () => {
         },
       ]);
       const result = ds.createQuery(target as any, { interval: '15s' } as any, 0, 0);
-      expect(result).toMatchObject({ expr: 'metric{job="foo",k1="v1",k2!="v2"} - metric{k1="v1",k2!="v2"}' });
+      expect(result).toMatchObject({ expr: 'metric{job="foo", k1="v1", k2!="v2"} - metric{k1="v1", k2!="v2"}' });
     });
 
     it('should add escaping if needed to regex filter expressions', () => {
@@ -186,7 +269,7 @@ describe('PrometheusDatasource', () => {
       ]);
       const result = ds.createQuery(target as any, { interval: '15s' } as any, 0, 0);
       expect(result).toMatchObject({
-        expr: `metric{job="foo",k1=~"v.*",k2=~"v\\\\'.*"} - metric{k1=~"v.*",k2=~"v\\\\'.*"}`,
+        expr: `metric{job="foo", k1=~"v.*", k2=~"v\\\\'.*"} - metric{k1=~"v.*", k2=~"v\\\\'.*"}`,
       });
     });
   });
@@ -231,8 +314,8 @@ describe('PrometheusDatasource', () => {
       await expect(ds.query(query)).toEmitValuesWith((result) => {
         const results = result[0].data;
         expect(results[0].fields[1].values.toArray()).toEqual([10, 10]);
-        expect(results[1].fields[1].values.toArray()).toEqual([10, 0]);
-        expect(results[2].fields[1].values.toArray()).toEqual([5, 0]);
+        expect(results[0].fields[2].values.toArray()).toEqual([10, 0]);
+        expect(results[0].fields[3].values.toArray()).toEqual([5, 0]);
       });
     });
 
@@ -273,7 +356,7 @@ describe('PrometheusDatasource', () => {
 
       ds.performTimeSeriesQuery = jest.fn().mockReturnValue(of(responseMock));
       await expect(ds.query(query)).toEmitValuesWith((result) => {
-        const seriesLabels = _.map(result[0].data, 'name');
+        const seriesLabels = result[0].data[0].fields.slice(1).map((field: Field) => getFieldDisplayName(field));
         expect(seriesLabels).toEqual(expected);
       });
     });
@@ -349,7 +432,7 @@ describe('PrometheusDatasource', () => {
     });
   });
 
-  describe('Prometheus regular escaping', () => {
+  describe('Prometheus regular escaping', () => {
     it('should not escape non-string', () => {
       expect(prometheusRegularEscape(12)).toEqual(12);
     });
@@ -375,7 +458,7 @@ describe('PrometheusDatasource', () => {
     });
   });
 
-  describe('Prometheus regexes escaping', () => {
+  describe('Prometheus regexes escaping', () => {
     it('should not escape simple string', () => {
       expect(prometheusSpecialRegexEscape('cryptodepression')).toEqual('cryptodepression');
     });
@@ -474,6 +557,94 @@ describe('PrometheusDatasource', () => {
     });
   });
 
+  describe('interpolateVariablesInQueries', () => {
+    it('should call replace function 2 times', () => {
+      const query = {
+        expr: 'test{job="testjob"}',
+        format: 'time_series',
+        interval: '$Interval',
+        refId: 'A',
+      };
+      const interval = '10m';
+      templateSrvStub.replace.mockReturnValue(interval);
+
+      const queries = ds.interpolateVariablesInQueries([query], { Interval: { text: interval, value: interval } });
+      expect(templateSrvStub.replace).toBeCalledTimes(2);
+      expect(queries[0].interval).toBe(interval);
+    });
+  });
+
+  describe('applyTemplateVariables', () => {
+    const originalAdhocFiltersMock = templateSrvStub.getAdhocFilters();
+    const originalReplaceMock = jest.fn((a: string, ...rest: any) => a);
+    afterAll(() => {
+      templateSrvStub.getAdhocFilters.mockReturnValue(originalAdhocFiltersMock);
+      templateSrvStub.replace = originalReplaceMock;
+    });
+
+    it('should call replace function for legendFormat', () => {
+      const query = {
+        expr: 'test{job="bar"}',
+        legendFormat: '$legend',
+        refId: 'A',
+      };
+      const legend = 'baz';
+      templateSrvStub.replace.mockReturnValue(legend);
+
+      const interpolatedQuery = ds.applyTemplateVariables(query, { legend: { text: legend, value: legend } });
+      expect(interpolatedQuery.legendFormat).toBe(legend);
+    });
+
+    it('should call replace function for interval', () => {
+      const query = {
+        expr: 'test{job="bar"}',
+        interval: '$step',
+        refId: 'A',
+      };
+      const step = '5s';
+      templateSrvStub.replace.mockReturnValue(step);
+
+      const interpolatedQuery = ds.applyTemplateVariables(query, { step: { text: step, value: step } });
+      expect(interpolatedQuery.interval).toBe(step);
+    });
+
+    it('should call replace function for expr', () => {
+      const query = {
+        expr: 'test{job="$job"}',
+        refId: 'A',
+      };
+      const job = 'bar';
+      templateSrvStub.replace.mockReturnValue(job);
+
+      const interpolatedQuery = ds.applyTemplateVariables(query, { job: { text: job, value: job } });
+      expect(interpolatedQuery.expr).toBe(job);
+    });
+
+    it('should add ad-hoc filters to expr', () => {
+      templateSrvStub.replace = jest.fn((a: string) => a);
+      templateSrvStub.getAdhocFilters.mockReturnValue([
+        {
+          key: 'k1',
+          operator: '=',
+          value: 'v1',
+        },
+        {
+          key: 'k2',
+          operator: '!=',
+          value: 'v2',
+        },
+      ]);
+
+      const query = {
+        expr: 'test{job="bar"}',
+        refId: 'A',
+      };
+
+      const result = ds.applyTemplateVariables(query, {});
+      expect(result).toMatchObject({ expr: 'test{job="bar", k1="v1", k2!="v2"}' });
+    });
+  });
+
   describe('metricFindQuery', () => {
     beforeEach(() => {
       const query = 'query_result(topk(5,rate(http_request_duration_microseconds_count[$__interval])))';
@@ -514,13 +685,13 @@ const HOUR = 60 * MINUTE;
 const time = ({ hours = 0, seconds = 0, minutes = 0 }) => dateTime(hours * HOUR + minutes * MINUTE + seconds * SECOND);
 
 describe('PrometheusDatasource', () => {
-  const instanceSettings = ({
+  const instanceSettings = {
     url: 'proxied',
     directUrl: 'direct',
     user: 'test',
     password: 'mupp',
     jsonData: { httpMethod: 'GET' },
-  } as unknown) as DataSourceInstanceSettings<PromOptions>;
+  } as unknown as DataSourceInstanceSettings<PromOptions>;
 
   let ds: PrometheusDatasource;
   beforeEach(() => {
@@ -736,26 +907,7 @@ describe('PrometheusDatasource', () => {
       },
     };
 
-    const response = {
-      status: 'success',
-      data: {
-        data: {
-          resultType: 'matrix',
-          result: [
-            {
-              metric: {
-                __name__: 'ALERTS',
-                alertname: 'InstanceDown',
-                alertstate: 'firing',
-                instance: 'testinstance',
-                job: 'testjob',
-              },
-              values: [[123, '1']],
-            },
-          ],
-        },
-      },
-    };
+    const response = createAnnotationResponse();
 
     describe('when time series query is cancelled', () => {
       it('should return empty results', async () => {
@@ -784,7 +936,7 @@ describe('PrometheusDatasource', () => {
         expect(results[0].tags).toContain('testjob');
         expect(results[0].title).toBe('InstanceDown');
         expect(results[0].text).toBe('testinstance');
-        expect(results[0].time).toBe(123 * 1000);
+        expect(results[0].time).toBe(123);
       });
     });
 
@@ -799,7 +951,7 @@ describe('PrometheusDatasource', () => {
       });
 
       it('should return annotation list', () => {
-        expect(results[0].time).toEqual(1);
+        expect(results[0].time).toEqual(456);
       });
     });
 
@@ -818,7 +970,7 @@ describe('PrometheusDatasource', () => {
         };
         ds.annotationQuery(query);
         const req = fetchMock.mock.calls[0][0];
-        expect(req.url).toContain('step=60');
+        expect(req.data.queries[0].interval).toBe('60s');
       });
 
       it('should use default step for short range when annotation step is empty string', () => {
@@ -835,7 +987,7 @@ describe('PrometheusDatasource', () => {
         };
         ds.annotationQuery(query);
         const req = fetchMock.mock.calls[0][0];
-        expect(req.url).toContain('step=60');
+        expect(req.data.queries[0].interval).toBe('60s');
       });
 
       it('should use custom step for short range', () => {
@@ -853,42 +1005,7 @@ describe('PrometheusDatasource', () => {
         };
         ds.annotationQuery(query);
         const req = fetchMock.mock.calls[0][0];
-        expect(req.url).toContain('step=10');
-      });
-
-      it('should use custom step for short range', () => {
-        const annotation = {
-          ...options.annotation,
-          step: '10s',
-        };
-        const query = {
-          ...options,
-          annotation,
-          range: {
-            from: time({ seconds: 63 }),
-            to: time({ seconds: 123 }),
-          },
-        };
-        ds.annotationQuery(query);
-        const req = fetchMock.mock.calls[0][0];
-        expect(req.url).toContain('step=10');
-      });
-
-      it('should use dynamic step on long ranges if no option was given', () => {
-        const query = {
-          ...options,
-          range: {
-            from: time({ seconds: 63 }),
-            to: time({ hours: 24 * 30, seconds: 63 }),
-          },
-        };
-        ds.annotationQuery(query);
-        const req = fetchMock.mock.calls[0][0];
-        // Range in seconds: (to - from) / 1000
-        // Max_datapoints: 11000
-        // Step: range / max_datapoints
-        const step = 236;
-        expect(req.url).toContain(`step=${step}`);
+        expect(req.data.queries[0].interval).toBe('10s');
       });
     });
 
@@ -906,21 +1023,9 @@ describe('PrometheusDatasource', () => {
         },
       };
 
-      async function runAnnotationQuery(resultValues: Array<[number, string]>) {
-        const response = {
-          status: 'success',
-          data: {
-            data: {
-              resultType: 'matrix',
-              result: [
-                {
-                  metric: { __name__: 'test', job: 'testjob' },
-                  values: resultValues,
-                },
-              ],
-            },
-          },
-        };
+      async function runAnnotationQuery(data: number[][]) {
+        let response = createAnnotationResponse();
+        response.data.results['X'].frames[0].data.values = data;
 
         options.annotation.useValueForTime = false;
         fetchMock.mockImplementation(() => of(response));
@@ -930,14 +1035,8 @@ describe('PrometheusDatasource', () => {
 
       it('should handle gaps and inactive values', async () => {
         const results = await runAnnotationQuery([
-          [2 * 60, '1'],
-          [3 * 60, '1'],
-          // gap
-          [5 * 60, '1'],
-          [6 * 60, '1'],
-          [7 * 60, '1'],
-          [8 * 60, '0'], // false --> create new block
-          [9 * 60, '1'],
+          [2 * 60000, 3 * 60000, 5 * 60000, 6 * 60000, 7 * 60000, 8 * 60000, 9 * 60000],
+          [1, 1, 1, 1, 1, 0, 1],
         ]);
         expect(results.map((result) => [result.time, result.timeEnd])).toEqual([
           [120000, 180000],
@@ -948,41 +1047,49 @@ describe('PrometheusDatasource', () => {
 
       it('should handle single region', async () => {
         const results = await runAnnotationQuery([
-          [2 * 60, '1'],
-          [3 * 60, '1'],
+          [2 * 60000, 3 * 60000],
+          [1, 1],
         ]);
         expect(results.map((result) => [result.time, result.timeEnd])).toEqual([[120000, 180000]]);
       });
 
       it('should handle 0 active regions', async () => {
         const results = await runAnnotationQuery([
-          [2 * 60, '0'],
-          [3 * 60, '0'],
-          [5 * 60, '0'],
+          [2 * 60000, 3 * 60000, 5 * 60000],
+          [0, 0, 0],
         ]);
         expect(results.length).toBe(0);
       });
 
       it('should handle single active value', async () => {
-        const results = await runAnnotationQuery([[2 * 60, '1']]);
+        const results = await runAnnotationQuery([[2 * 60000], [1]]);
         expect(results.map((result) => [result.time, result.timeEnd])).toEqual([[120000, 120000]]);
       });
     });
-  });
+    describe('with template variables', () => {
+      const originalReplaceMock = jest.fn((a: string, ...rest: any) => a);
+      afterAll(() => {
+        templateSrvStub.replace = originalReplaceMock;
+      });
 
-  describe('createAnnotationQueryOptions', () => {
-    it.each`
-      options                                | expected
-      ${{}}                                  | ${{ interval: '60s' }}
-      ${{ annotation: {} }}                  | ${{ annotation: {}, interval: '60s' }}
-      ${{ annotation: { step: undefined } }} | ${{ annotation: { step: undefined }, interval: '60s' }}
-      ${{ annotation: { step: null } }}      | ${{ annotation: { step: null }, interval: '60s' }}
-      ${{ annotation: { step: '' } }}        | ${{ annotation: { step: '' }, interval: '60s' }}
-      ${{ annotation: { step: 0 } }}         | ${{ annotation: { step: 0 }, interval: '60s' }}
-      ${{ annotation: { step: 5 } }}         | ${{ annotation: { step: 5 }, interval: '60s' }}
-      ${{ annotation: { step: '5m' } }}      | ${{ annotation: { step: '5m' }, interval: '5m' }}
-    `("when called with options: '$options'", ({ options, expected }) => {
-      expect(ds.createAnnotationQueryOptions(options)).toEqual(expected);
+      it('should interpolate variables in query expr', () => {
+        const query = {
+          ...options,
+          annotation: {
+            ...options.annotation,
+            expr: '$variable',
+          },
+          range: {
+            from: time({ seconds: 1 }),
+            to: time({ seconds: 2 }),
+          },
+        };
+        const interpolated = 'interpolated_expr';
+        templateSrvStub.replace.mockReturnValue(interpolated);
+        ds.annotationQuery(query);
+        const req = fetchMock.mock.calls[0][0];
+        expect(req.data.queries[0].expr).toBe(interpolated);
+      });
     });
   });
 
@@ -1589,17 +1696,45 @@ describe('PrometheusDatasource', () => {
       templateSrvStub.replace = jest.fn((a: string) => a);
     });
   });
+
+  it('should give back 1 exemplar target when multiple queries with exemplar enabled and same metric', () => {
+    const targetA: PromQuery = {
+      refId: 'A',
+      expr: 'histogram_quantile(0.95, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+      exemplar: true,
+    };
+    const targetB: PromQuery = {
+      refId: 'B',
+      expr: 'histogram_quantile(0.5, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+      exemplar: true,
+    };
+
+    ds.languageProvider = {
+      histogramMetrics: ['tns_request_duration_seconds_bucket'],
+    } as any;
+
+    const request = {
+      targets: [targetA, targetB],
+      interval: '1s',
+      panelId: '',
+    } as any as DataQueryRequest<PromQuery>;
+
+    const Aexemplars = ds.shouldRunExemplarQuery(targetA, request);
+    const BExpemplars = ds.shouldRunExemplarQuery(targetB, request);
+
+    expect(Aexemplars).toBe(true);
+    expect(BExpemplars).toBe(false);
+  });
 });
 
 describe('PrometheusDatasource for POST', () => {
-  //   const ctx = new helpers.ServiceTestContext();
-  const instanceSettings = ({
+  const instanceSettings = {
     url: 'proxied',
     directUrl: 'direct',
     user: 'test',
     password: 'mupp',
     jsonData: { httpMethod: 'POST' },
-  } as unknown) as DataSourceInstanceSettings<PromOptions>;
+  } as unknown as DataSourceInstanceSettings<PromOptions>;
 
   let ds: PrometheusDatasource;
   beforeEach(() => {
@@ -1681,26 +1816,39 @@ describe('PrometheusDatasource for POST', () => {
   });
 });
 
-const getPrepareTargetsContext = (target: PromQuery, app?: CoreApp, queryOptions?: Partial<QueryOptions>) => {
-  const instanceSettings = ({
+function getPrepareTargetsContext({
+  targets,
+  app,
+  queryOptions,
+  languageProvider,
+}: {
+  targets: PromQuery[];
+  app?: CoreApp;
+  queryOptions?: Partial<QueryOptions>;
+  languageProvider?: any;
+}) {
+  const instanceSettings = {
     url: 'proxied',
     directUrl: 'direct',
     user: 'test',
     password: 'mupp',
     jsonData: { httpMethod: 'POST' },
-  } as unknown) as DataSourceInstanceSettings<PromOptions>;
+  } as unknown as DataSourceInstanceSettings<PromOptions>;
   const start = 0;
   const end = 1;
   const panelId = '2';
-  const options = ({
-    targets: [target],
+  const options = {
+    targets,
     interval: '1s',
     panelId,
     app,
     ...queryOptions,
-  } as any) as DataQueryRequest<PromQuery>;
+  } as any as DataQueryRequest<PromQuery>;
 
   const ds = new PrometheusDatasource(instanceSettings, templateSrvStub as any, timeSrvStub as any);
+  if (languageProvider) {
+    ds.languageProvider = languageProvider;
+  }
   const { queries, activeTargets } = ds.prepareTargets(options, start, end);
 
   return {
@@ -1710,7 +1858,7 @@ const getPrepareTargetsContext = (target: PromQuery, app?: CoreApp, queryOptions
     end,
     panelId,
   };
-};
+}
 
 describe('prepareTargets', () => {
   describe('when run from a Panel', () => {
@@ -1718,9 +1866,10 @@ describe('prepareTargets', () => {
       const target: PromQuery = {
         refId: 'A',
         expr: 'up',
+        requestId: '2A',
       };
 
-      const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext(target);
+      const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext({ targets: [target] });
 
       expect(queries.length).toBe(1);
       expect(activeTargets.length).toBe(1);
@@ -1740,19 +1889,148 @@ describe('prepareTargets', () => {
       });
       expect(activeTargets[0]).toEqual(target);
     });
+
+    it('should give back 3 targets when multiple queries with exemplar enabled and same metric', () => {
+      const targetA: PromQuery = {
+        refId: 'A',
+        expr: 'histogram_quantile(0.95, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+        exemplar: true,
+      };
+      const targetB: PromQuery = {
+        refId: 'B',
+        expr: 'histogram_quantile(0.5, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+        exemplar: true,
+      };
+
+      const { queries, activeTargets } = getPrepareTargetsContext({
+        targets: [targetA, targetB],
+        languageProvider: {
+          histogramMetrics: ['tns_request_duration_seconds_bucket'],
+        },
+      });
+      expect(queries).toHaveLength(3);
+      expect(activeTargets).toHaveLength(3);
+    });
+
+    it('should give back 4 targets when multiple queries with exemplar enabled', () => {
+      const targetA: PromQuery = {
+        refId: 'A',
+        expr: 'histogram_quantile(0.95, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+        exemplar: true,
+      };
+      const targetB: PromQuery = {
+        refId: 'B',
+        expr: 'histogram_quantile(0.5, sum(rate(tns_request_duration_bucket[5m])) by (le))',
+        exemplar: true,
+      };
+
+      const { queries, activeTargets } = getPrepareTargetsContext({
+        targets: [targetA, targetB],
+        languageProvider: {
+          histogramMetrics: ['tns_request_duration_seconds_bucket'],
+        },
+      });
+      expect(queries).toHaveLength(4);
+      expect(activeTargets).toHaveLength(4);
+    });
+
+    it('should give back 2 targets when exemplar enabled', () => {
+      const target: PromQuery = {
+        refId: 'A',
+        expr: 'up',
+        exemplar: true,
+      };
+
+      const { queries, activeTargets } = getPrepareTargetsContext({ targets: [target] });
+      expect(queries).toHaveLength(2);
+      expect(activeTargets).toHaveLength(2);
+      expect(activeTargets[0].exemplar).toBe(true);
+      expect(activeTargets[1].exemplar).toBe(false);
+    });
+    it('should give back 1 target when exemplar and instant are enabled', () => {
+      const target: PromQuery = {
+        refId: 'A',
+        expr: 'up',
+        exemplar: true,
+        instant: true,
+      };
+
+      const { queries, activeTargets } = getPrepareTargetsContext({ targets: [target] });
+      expect(queries).toHaveLength(1);
+      expect(activeTargets).toHaveLength(1);
+      expect(activeTargets[0].instant).toBe(true);
+    });
   });
 
   describe('when run from Explore', () => {
     describe('when query type Both is selected', () => {
+      it('should give back 6 targets when multiple queries with exemplar enabled', () => {
+        const targetA: PromQuery = {
+          refId: 'A',
+          expr: 'histogram_quantile(0.95, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+          instant: true,
+          range: true,
+          exemplar: true,
+        };
+        const targetB: PromQuery = {
+          refId: 'B',
+          expr: 'histogram_quantile(0.5, sum(rate(tns_request_duration_bucket[5m])) by (le))',
+          exemplar: true,
+          instant: true,
+          range: true,
+        };
+
+        const { queries, activeTargets } = getPrepareTargetsContext({
+          targets: [targetA, targetB],
+          app: CoreApp.Explore,
+          languageProvider: {
+            histogramMetrics: ['tns_request_duration_seconds_bucket'],
+          },
+        });
+        expect(queries).toHaveLength(6);
+        expect(activeTargets).toHaveLength(6);
+      });
+
+      it('should give back 5 targets when multiple queries with exemplar enabled and same metric', () => {
+        const targetA: PromQuery = {
+          refId: 'A',
+          expr: 'histogram_quantile(0.95, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+          instant: true,
+          range: true,
+          exemplar: true,
+        };
+        const targetB: PromQuery = {
+          refId: 'B',
+          expr: 'histogram_quantile(0.5, sum(rate(tns_request_duration_seconds_bucket[5m])) by (le))',
+          exemplar: true,
+          instant: true,
+          range: true,
+        };
+
+        const { queries, activeTargets } = getPrepareTargetsContext({
+          targets: [targetA, targetB],
+          app: CoreApp.Explore,
+          languageProvider: {
+            histogramMetrics: ['tns_request_duration_seconds_bucket'],
+          },
+        });
+        expect(queries).toHaveLength(5);
+        expect(activeTargets).toHaveLength(5);
+      });
+
       it('then it should return both instant and time series related objects', () => {
         const target: PromQuery = {
           refId: 'A',
           expr: 'up',
           range: true,
           instant: true,
+          requestId: '2A',
         };
 
-        const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext(target, CoreApp.Explore);
+        const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext({
+          targets: [target],
+          app: CoreApp.Explore,
+        });
 
         expect(queries.length).toBe(2);
         expect(activeTargets.length).toBe(2);
@@ -1807,9 +2085,13 @@ describe('prepareTargets', () => {
           expr: 'up',
           instant: true,
           range: false,
+          requestId: '2A',
         };
 
-        const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext(target, CoreApp.Explore);
+        const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext({
+          targets: [target],
+          app: CoreApp.Explore,
+        });
 
         expect(queries.length).toBe(1);
         expect(activeTargets.length).toBe(1);
@@ -1839,9 +2121,13 @@ describe('prepareTargets', () => {
         expr: 'up',
         range: true,
         instant: false,
+        requestId: '2A',
       };
 
-      const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext(target, CoreApp.Explore);
+      const { queries, activeTargets, panelId, end, start } = getPrepareTargetsContext({
+        targets: [target],
+        app: CoreApp.Explore,
+      });
 
       expect(queries.length).toBe(1);
       expect(activeTargets.length).toBe(1);
@@ -1870,7 +2156,7 @@ describe('modifyQuery', () => {
       it('then the correct label should be added', () => {
         const query: PromQuery = { refId: 'A', expr: 'go_goroutines' };
         const action = { key: 'cluster', value: 'us-cluster', type: 'ADD_FILTER' };
-        const instanceSettings = ({ jsonData: {} } as unknown) as DataSourceInstanceSettings<PromOptions>;
+        const instanceSettings = { jsonData: {} } as unknown as DataSourceInstanceSettings<PromOptions>;
         const ds = new PrometheusDatasource(instanceSettings, templateSrvStub as any, timeSrvStub as any);
 
         const result = ds.modifyQuery(query, action);
@@ -1884,13 +2170,13 @@ describe('modifyQuery', () => {
       it('then the correct label should be added', () => {
         const query: PromQuery = { refId: 'A', expr: 'go_goroutines{cluster="us-cluster"}' };
         const action = { key: 'pod', value: 'pod-123', type: 'ADD_FILTER' };
-        const instanceSettings = ({ jsonData: {} } as unknown) as DataSourceInstanceSettings<PromOptions>;
+        const instanceSettings = { jsonData: {} } as unknown as DataSourceInstanceSettings<PromOptions>;
         const ds = new PrometheusDatasource(instanceSettings, templateSrvStub as any, timeSrvStub as any);
 
         const result = ds.modifyQuery(query, action);
 
         expect(result.refId).toEqual('A');
-        expect(result.expr).toEqual('go_goroutines{cluster="us-cluster",pod="pod-123"}');
+        expect(result.expr).toEqual('go_goroutines{cluster="us-cluster", pod="pod-123"}');
       });
     });
   });
@@ -1900,7 +2186,7 @@ describe('modifyQuery', () => {
       it('then the correct label should be added', () => {
         const query: PromQuery = { refId: 'A', expr: 'go_goroutines' };
         const action = { key: 'cluster', value: 'us-cluster', type: 'ADD_FILTER_OUT' };
-        const instanceSettings = ({ jsonData: {} } as unknown) as DataSourceInstanceSettings<PromOptions>;
+        const instanceSettings = { jsonData: {} } as unknown as DataSourceInstanceSettings<PromOptions>;
         const ds = new PrometheusDatasource(instanceSettings, templateSrvStub as any, timeSrvStub as any);
 
         const result = ds.modifyQuery(query, action);
@@ -1914,13 +2200,13 @@ describe('modifyQuery', () => {
       it('then the correct label should be added', () => {
         const query: PromQuery = { refId: 'A', expr: 'go_goroutines{cluster="us-cluster"}' };
         const action = { key: 'pod', value: 'pod-123', type: 'ADD_FILTER_OUT' };
-        const instanceSettings = ({ jsonData: {} } as unknown) as DataSourceInstanceSettings<PromOptions>;
+        const instanceSettings = { jsonData: {} } as unknown as DataSourceInstanceSettings<PromOptions>;
         const ds = new PrometheusDatasource(instanceSettings, templateSrvStub as any, timeSrvStub as any);
 
         const result = ds.modifyQuery(query, action);
 
         expect(result.refId).toEqual('A');
-        expect(result.expr).toEqual('go_goroutines{cluster="us-cluster",pod!="pod-123"}');
+        expect(result.expr).toEqual('go_goroutines{cluster="us-cluster", pod!="pod-123"}');
       });
     });
   });
@@ -1966,3 +2252,50 @@ function createDefaultPromResponse() {
     },
   };
 }
+
+function createAnnotationResponse() {
+  const response = {
+    data: {
+      results: {
+        X: {
+          frames: [
+            {
+              schema: {
+                name: 'bar',
+                refId: 'X',
+                fields: [
+                  {
+                    name: 'Time',
+                    type: 'time',
+                    typeInfo: {
+                      frame: 'time.Time',
+                    },
+                  },
+                  {
+                    name: 'Value',
+                    type: 'number',
+                    typeInfo: {
+                      frame: 'float64',
+                    },
+                    labels: {
+                      __name__: 'ALERTS',
+                      alertname: 'InstanceDown',
+                      alertstate: 'firing',
+                      instance: 'testinstance',
+                      job: 'testjob',
+                    },
+                  },
+                ],
+              },
+              data: {
+                values: [[123], [456]],
+              },
+            },
+          ],
+        },
+      },
+    },
+  };
+
+  return { ...response };
+}
