@@ -1,9 +1,12 @@
-import _ from 'lodash';
+import { chain, map as _map, uniq } from 'lodash';
+import { lastValueFrom } from 'rxjs';
 import { map } from 'rxjs/operators';
+
 import { MetricFindValue, TimeRange } from '@grafana/data';
+import { getTimeSrv } from 'app/features/dashboard/services/TimeSrv';
+
 import { PrometheusDatasource } from './datasource';
 import { PromQueryRequest } from './types';
-import { getTimeSrv } from 'app/features/dashboard/services/TimeSrv';
 
 export default class PrometheusMetricFindQuery {
   range: TimeRange;
@@ -40,7 +43,7 @@ export default class PrometheusMetricFindQuery {
 
     const queryResultQuery = this.query.match(queryResultRegex);
     if (queryResultQuery) {
-      return this.queryResultQuery(queryResultQuery[1]).toPromise();
+      return lastValueFrom(this.queryResultQuery(queryResultQuery[1]));
     }
 
     // if query contains full metric name, return metric name and label list
@@ -58,7 +61,7 @@ export default class PrometheusMetricFindQuery {
     const url = `/api/v1/labels`;
 
     return this.datasource.metadataRequest(url, params).then((result: any) => {
-      return _.map(result.data.data, (value) => {
+      return _map(result.data.data, (value) => {
         return { text: value };
       });
     });
@@ -79,7 +82,7 @@ export default class PrometheusMetricFindQuery {
       url = `/api/v1/label/${label}/values`;
 
       return this.datasource.metadataRequest(url, params).then((result: any) => {
-        return _.map(result.data.data, (value) => {
+        return _map(result.data.data, (value) => {
           return { text: value };
         });
       });
@@ -92,13 +95,13 @@ export default class PrometheusMetricFindQuery {
       url = `/api/v1/series`;
 
       return this.datasource.metadataRequest(url, params).then((result: any) => {
-        const _labels = _.map(result.data.data, (metric) => {
+        const _labels = _map(result.data.data, (metric) => {
           return metric[label] || '';
         }).filter((label) => {
           return label !== '';
         });
 
-        return _.uniq(_labels).map((metric) => {
+        return uniq(_labels).map((metric) => {
           return {
             text: metric,
             expandable: true,
@@ -118,7 +121,7 @@ export default class PrometheusMetricFindQuery {
     const url = `/api/v1/label/__name__/values`;
 
     return this.datasource.metadataRequest(url, params).then((result: any) => {
-      return _.chain(result.data.data)
+      return chain(result.data.data)
         .filter((metricName) => {
           const r = new RegExp(metricFilterPattern);
           return r.test(metricName);
@@ -138,12 +141,12 @@ export default class PrometheusMetricFindQuery {
     const instantQuery: PromQueryRequest = { expr: query } as PromQueryRequest;
     return this.datasource.performInstantQuery(instantQuery, end).pipe(
       map((result) => {
-        return _.map(result.data.data.result, (metricData) => {
+        return _map(result.data.data.result, (metricData) => {
           let text = metricData.metric.__name__ || '';
           delete metricData.metric.__name__;
           text +=
             '{' +
-            _.map(metricData.metric, (v, k) => {
+            _map(metricData.metric, (v, k) => {
               return k + '="' + v + '"';
             }).join(',') +
             '}';
@@ -171,7 +174,7 @@ export default class PrometheusMetricFindQuery {
     const self = this;
 
     return this.datasource.metadataRequest(url, params).then((result: any) => {
-      return _.map(result.data.data, (metric: { [key: string]: string }) => {
+      return _map(result.data.data, (metric: { [key: string]: string }) => {
         return {
           text: self.datasource.getOriginalMetricName(metric),
           expandable: true,
