@@ -1,8 +1,10 @@
-import _ from 'lodash';
+import { find } from 'lodash';
+
+import { DataFrame, dateTime, Field, FieldType, getFieldDisplayName, getTimeField, TimeRange } from '@grafana/data';
 import { colors } from '@grafana/ui';
-import { getColorFromHexRgbOrName, TimeRange, FieldType, Field, DataFrame, getTimeField } from '@grafana/data';
-import TimeSeries from 'app/core/time_series2';
+import { applyNullInsertThreshold } from '@grafana/ui/src/components/GraphNG/nullInsertThreshold';
 import config from 'app/core/config';
+import TimeSeries from 'app/core/time_series2';
 
 type Options = {
   dataList: DataFrame[];
@@ -21,29 +23,29 @@ export class DataProcessor {
     }
 
     for (let i = 0; i < dataList.length; i++) {
-      const series = dataList[i];
-      const { timeField } = getTimeField(series);
+      let series = dataList[i];
+      let { timeField } = getTimeField(series);
+
       if (!timeField) {
         continue;
       }
 
-      const seriesName = series.name ? series.name : series.refId;
+      series = applyNullInsertThreshold({ frame: series, refFieldName: timeField.name });
+      timeField = getTimeField(series).timeField!; // use updated length
+
       for (let j = 0; j < series.fields.length; j++) {
         const field = series.fields[j];
+
         if (field.type !== FieldType.number) {
           continue;
         }
-
-        let name = field.config && field.config.title ? field.config.title : field.name;
-
-        if (seriesName && dataList.length > 0 && name !== seriesName) {
-          name = seriesName + ' ' + name;
-        }
-
+        const name = getFieldDisplayName(field, series, dataList);
         const datapoints = [];
+
         for (let r = 0; r < series.length; r++) {
-          datapoints.push([field.values.get(r), timeField.values.get(r)]);
+          datapoints.push([field.values.get(r), dateTime(timeField.values.get(r)).valueOf()]);
         }
+
         list.push(this.toTimeSeries(field, name, i, j, datapoints, list.length, range));
       }
     }
@@ -52,9 +54,11 @@ export class DataProcessor {
     if (this.panel.xaxis.mode === 'histogram' && !this.panel.stack && list.length > 1) {
       const first = list[0];
       first.alias = first.aliasEscaped = 'Count';
+
       for (let i = 1; i < list.length; i++) {
         first.datapoints = first.datapoints.concat(list[i].datapoints);
       }
+
       return [first];
     }
 
@@ -76,7 +80,7 @@ export class DataProcessor {
     const series = new TimeSeries({
       datapoints: datapoints || [],
       alias: alias,
-      color: getColorFromHexRgbOrName(color, config.theme.type),
+      color: config.theme.visualization.getColorByName(color),
       unit: field.config ? field.config.unit : undefined,
       dataFrameIndex,
       fieldIndex,
@@ -87,7 +91,11 @@ export class DataProcessor {
       const from = range.from;
 
       if (last - from.valueOf() < -10000) {
-        series.isOutsideRange = true;
+        // If the data is in reverse order
+        const first = datapoints[0][1];
+        if (first - from.valueOf() < -10000) {
+          series.isOutsideRange = true;
+        }
       }
     }
     return series;
@@ -135,7 +143,7 @@ export class DataProcessor {
         }
 
         const validOptions = this.getXAxisValueOptions({});
-        const found: any = _.find(validOptions, { value: this.panel.xaxis.values[0] });
+        const found: any = find(validOptions, { value: this.panel.xaxis.values[0] });
         if (!found) {
           this.panel.xaxis.values = ['total'];
         }
