@@ -1,9 +1,11 @@
 import * as d3 from 'd3';
 import $ from 'jquery';
-import _ from 'lodash';
-import { getValueBucketBound } from './heatmap_data_converter';
+import { filter, find, isNumber, map, reduce } from 'lodash';
+
 import { getValueFormat, formattedValueToString } from '@grafana/data';
 
+import { getValueBucketBound } from './heatmap_data_converter';
+
 const TOOLTIP_PADDING_X = 30;
 const TOOLTIP_PADDING_Y = 5;
 const HISTOGRAM_WIDTH = 160;
@@ -78,7 +80,7 @@ export class HeatmapTooltip {
     let boundBottom, boundTop, valuesNumber;
     const xData = data.buckets[xBucketIndex];
     // Search in special 'zero' bucket also
-    const yData: any = _.find(xData.buckets, (bucket, bucketIndex) => {
+    const yData: any = find(xData.buckets, (bucket, bucketIndex) => {
       return bucket.bounds.bottom === yBucketIndex || bucketIndex === yBucketIndex.toString();
     });
 
@@ -87,7 +89,7 @@ export class HeatmapTooltip {
 
     // Decimals override. Code from panel/graph/graph.ts
     let countValueFormatter, bucketBoundFormatter;
-    if (_.isNumber(this.panel.tooltipDecimals)) {
+    if (isNumber(this.panel.tooltipDecimals)) {
       countValueFormatter = this.countValueFormatter(this.panel.tooltipDecimals, null);
       bucketBoundFormatter = this.panelCtrl.tickValueFormatter(this.panelCtrl.decimals, null);
     } else {
@@ -158,7 +160,7 @@ export class HeatmapTooltip {
   getXBucketIndex(x: number, data: { buckets: any; xBucketSize: number }) {
     // First try to find X bucket by checking x pos is in the
     // [bucket.x, bucket.x + xBucketSize] interval
-    const xBucket: any = _.find(data.buckets, (bucket) => {
+    const xBucket: any = find(data.buckets, (bucket) => {
       return x > bucket.x && x - bucket.x <= data.xBucketSize;
     });
     return xBucket ? xBucket.x : getValueBucketBound(x, data.xBucketSize, 1);
@@ -192,11 +194,11 @@ export class HeatmapTooltip {
       max = this.scope.ctrl.data.yAxis.max;
       ticks = this.scope.ctrl.data.yAxis.ticks;
     }
-    let histogramData = _.map(xBucket.buckets, (bucket) => {
+    let histogramData = map(xBucket.buckets, (bucket) => {
       const count = bucket.count !== undefined ? bucket.count : bucket.values.length;
       return [bucket.bounds.bottom, count];
     });
-    histogramData = _.filter(histogramData, (d) => {
+    histogramData = filter(histogramData, (d) => {
       return d[0] >= min && d[0] <= max;
     });
 
@@ -213,8 +215,8 @@ export class HeatmapTooltip {
     barWidth = Math.max(barWidth, 1);
 
     // Normalize histogram Y axis
-    const histogramDomain = _.reduce(
-      _.map(histogramData, (d) => d[1]),
+    const histogramDomain = reduce(
+      map(histogramData, (d) => d[1]),
       (sum, val) => sum + val,
       0
     );
