@@ -1,23 +1,24 @@
-import _ from 'lodash';
-import $ from 'jquery';
 import * as d3 from 'd3';
-import { contextSrv } from 'app/core/core';
-import * as ticksUtils from 'app/core/utils/ticks';
-import { HeatmapTooltip } from './heatmap_tooltip';
-import { mergeZeroBuckets } from './heatmap_data_converter';
-import { getColorScale, getOpacityScale } from './color_scale';
+import $ from 'jquery';
+import { find, isEmpty, isNaN, isNil, isString, map, max, min, toNumber } from 'lodash';
+
 import {
-  toUtc,
-  PanelEvents,
-  getValueFormat,
-  formattedValueToString,
   dateTimeFormat,
-  LegacyGraphHoverEvent,
+  formattedValueToString,
+  getValueFormat,
   LegacyGraphHoverClearEvent,
-  getColorForTheme,
+  LegacyGraphHoverEvent,
+  PanelEvents,
+  toUtc,
 } from '@grafana/data';
 import { graphTimeFormat } from '@grafana/ui';
 import { config } from 'app/core/config';
+import { contextSrv } from 'app/core/core';
+import * as ticksUtils from 'app/core/utils/ticks';
+
+import { getColorScale, getOpacityScale } from './color_scale';
+import { mergeZeroBuckets } from './heatmap_data_converter';
+import { HeatmapTooltip } from './heatmap_tooltip';
 
 const MIN_CARD_SIZE = 1,
   CARD_PADDING = 1,
@@ -62,6 +63,8 @@ export class HeatmapRenderer {
   margin: any;
   dataRangeWidingFactor: number;
 
+  hoverEvent: LegacyGraphHoverEvent;
+
   constructor(private scope: any, private elem: any, attrs: any, private ctrl: any) {
     // $heatmap is JQuery object, but heatmap is D3
     this.$heatmap = this.elem.find('.heatmap-panel');
@@ -91,6 +94,8 @@ export class HeatmapRenderer {
     this.$heatmap.on('mousedown', this.onMouseDown.bind(this));
     this.$heatmap.on('mousemove', this.onMouseMove.bind(this));
     this.$heatmap.on('mouseleave', this.onMouseLeave.bind(this));
+
+    this.hoverEvent = new LegacyGraphHoverEvent({ pos: {}, point: {}, panel: this.panel });
   }
 
   onGraphHoverClear() {
@@ -109,7 +114,7 @@ export class HeatmapRenderer {
   setElementHeight() {
     try {
       let height = this.ctrl.height || this.panel.height || this.ctrl.row.height;
-      if (_.isString(height)) {
+      if (isString(height)) {
         height = parseInt(height.replace('px', ''), 10);
       }
 
@@ -125,9 +130,14 @@ export class HeatmapRenderer {
   }
 
   getYAxisWidth(elem: any) {
+    const panelYAxisWidth = this.getPanelYAxisWidth();
+    if (panelYAxisWidth !== null) {
+      return panelYAxisWidth + Y_AXIS_TICK_PADDING;
+    }
+
     const axisText = elem.selectAll('.axis-y text').nodes();
-    const maxTextWidth = _.max(
-      _.map(axisText, (text) => {
+    const maxTextWidth = max(
+      map(axisText, (text) => {
         // Use SVG getBBox method
         return text.getBBox().width;
       })
@@ -157,7 +167,7 @@ export class HeatmapRenderer {
     const ticks = this.chartWidth / DEFAULT_X_TICK_SIZE_PX;
     const format = graphTimeFormat(ticks, this.timeRange.from.valueOf(), this.timeRange.to.valueOf());
     const timeZone = this.ctrl.dashboard.getTimezone();
-    const formatter = (date: Date) =>
+    const formatter = (date: d3.AxisDomain) =>
       dateTimeFormat(date.valueOf(), {
         format: format,
         timeZone: timeZone,
@@ -204,7 +214,7 @@ export class HeatmapRenderer {
     this.ctrl.scaledDecimals = scaledDecimals;
 
     // Set default Y min and max if no data
-    if (_.isEmpty(this.data.buckets)) {
+    if (isEmpty(this.data.buckets)) {
       yMax = 1;
       yMin = -1;
       ticks = 3;
@@ -268,7 +278,7 @@ export class HeatmapRenderer {
     yMax = this.panel.yAxis.max !== null ? this.adjustLogMax(this.panel.yAxis.max, logBase) : yMax;
 
     // Set default Y min and max if no data
-    if (_.isEmpty(this.data.buckets)) {
+    if (isEmpty(this.data.buckets)) {
       yMax = Math.pow(logBase, 2);
       yMin = 1;
     }
@@ -330,27 +340,35 @@ export class HeatmapRenderer {
       .domain([0, tsBuckets.length - 1])
       .range([this.chartHeight, 0]);
 
-    const tickValues = _.map(tsBuckets, (b, i) => i);
-    const decimalsAuto = _.max(_.map(tsBuckets, ticksUtils.getStringPrecision));
+    const tickValues = map(tsBuckets, (b, i) => i);
+    const decimalsAuto = max(map(tsBuckets, ticksUtils.getStringPrecision));
     const decimals = this.panel.yAxis.decimals === null ? decimalsAuto : this.panel.yAxis.decimals;
     this.ctrl.decimals = decimals;
 
     const tickValueFormatter = this.tickValueFormatter.bind(this);
-    function tickFormatter(valIndex: string) {
-      let valueFormatted = tsBuckets[valIndex];
-      if (!_.isNaN(_.toNumber(valueFormatted)) && valueFormatted !== '') {
-        // Try to format numeric tick labels
-        valueFormatted = tickValueFormatter(decimals)(_.toNumber(valueFormatted));
-      }
-      return valueFormatted;
+    function tickFormatter(yAxisWidth: number | null) {
+      return function (valIndex: d3.AxisDomain) {
+        let valueFormatted = tsBuckets[valIndex.valueOf()];
+        if (!isNaN(toNumber(valueFormatted)) && valueFormatted !== '') {
+          // Try to format numeric tick labels
+          valueFormatted = tickValueFormatter(decimals)(toNumber(valueFormatted));
+        } else if (valueFormatted && typeof valueFormatted === 'string' && valueFormatted !== '') {
+          if (yAxisWidth) {
+            const scale = 0.15; // how to have a better calculation for this
+            const trimmed = valueFormatted.substring(0, Math.floor(yAxisWidth * scale));
+            const postfix = trimmed.length < valueFormatted.length ? '...' : '';
+            valueFormatted = `${trimmed}${postfix}`;
+          }
+        }
+        return valueFormatted;
+      };
     }
-
-    const tsBucketsFormatted = _.map(tsBuckets, (v, i) => tickFormatter(i));
+    const tsBucketsFormatted = map(tsBuckets, (v, i) => tickFormatter(null)(i));
     this.data.tsBucketsFormatted = tsBucketsFormatted;
 
     const yAxis = d3
       .axisLeft(this.yScale)
-      .tickFormat(tickFormatter)
+      .tickFormat(tickFormatter(this.getPanelYAxisWidth()))
       .tickSizeInner(0 - this.width)
       .tickSizeOuter(0)
       .tickPadding(Y_AXIS_TICK_PADDING);
@@ -495,16 +513,16 @@ export class HeatmapRenderer {
       const logBase = this.panel.yAxis.logBase;
       const domain = this.yScale.domain();
       const tickValues = this.logScaleTickValues(domain, logBase);
-      this.data.buckets = mergeZeroBuckets(this.data.buckets, _.min(tickValues)!);
+      this.data.buckets = mergeZeroBuckets(this.data.buckets, min(tickValues)!);
     }
 
     const cardsData = this.data.cards;
     const cardStats = this.data.cardStats;
     const maxValueAuto = cardStats.max;
     const minValueAuto = Math.max(cardStats.min, 0);
-    const maxValue = _.isNil(this.panel.color.max) ? maxValueAuto : this.panel.color.max;
-    const minValue = _.isNil(this.panel.color.min) ? minValueAuto : this.panel.color.min;
-    const colorScheme: any = _.find(this.ctrl.colorSchemes, {
+    const maxValue = isNil(this.panel.color.max) ? maxValueAuto : this.panel.color.max;
+    const minValue = isNil(this.panel.color.min) ? minValueAuto : this.panel.color.min;
+    const colorScheme: any = find(this.ctrl.colorSchemes, {
       value: this.panel.color.colorScheme,
     });
     this.colorScale = getColorScale(colorScheme, contextSrv.user.lightTheme, maxValue, minValue);
@@ -641,7 +659,7 @@ export class HeatmapRenderer {
 
   getCardColor(d: { count: any }) {
     if (this.panel.color.mode === 'opacity') {
-      return getColorForTheme(this.panel.color.cardColor, config.theme);
+      return config.theme.visualization.getColorByName(this.panel.color.cardColor);
     } else {
       return this.colorScale(d.count);
     }
@@ -740,7 +758,10 @@ export class HeatmapRenderer {
     // Set minimum offset to prevent showing legend from another panel
     pos.panelRelY = Math.max(pos.offset.y / this.height, 0.001);
     // broadcast to other graph panels that we are hovering
-    this.ctrl.dashboard.events.publish(new LegacyGraphHoverEvent({ pos: pos, panel: this.panel }));
+    this.hoverEvent.payload.pos = pos;
+    this.hoverEvent.payload.panel = this.panel;
+    this.hoverEvent.payload.point['time'] = (pos as any).x;
+    this.ctrl.dashboard.events.publish(this.hoverEvent);
   }
 
   limitSelection(x2: number) {
@@ -820,7 +841,7 @@ export class HeatmapRenderer {
     }
 
     // Draw default axes and return if no data
-    if (_.isEmpty(this.data.buckets)) {
+    if (isEmpty(this.data.buckets)) {
       this.addHeatmapCanvas();
       this.addAxes();
       return;
@@ -833,4 +854,12 @@ export class HeatmapRenderer {
     this.scope.chartWidth = this.chartWidth;
     this.scope.chartTop = this.chartTop;
   }
+
+  private getPanelYAxisWidth(): number | null {
+    if (!this.panel.yAxis.width) {
+      return null;
+    }
+
+    return isNaN(this.panel.yAxis.width) ? null : parseInt(this.panel.yAxis.width, 10);
+  }
 }
