@@ -1,5 +1,5 @@
-import { BusEventBase, BusEventWithPayload, eventFactory, GrafanaTheme, TimeRange } from '@grafana/data';
-import { DashboardModel } from 'app/features/dashboard/state';
+import { AnnotationQuery, BusEventBase, BusEventWithPayload, eventFactory } from '@grafana/data';
+import { IconName } from '@grafana/ui';
 
 /**
  * Event Payloads
@@ -36,7 +36,7 @@ export interface ShowConfirmModalPayload {
   altActionText?: string;
   yesText?: string;
   noText?: string;
-  icon?: string;
+  icon?: IconName;
 
   onConfirm?: () => void;
   onAltAction?: () => void;
@@ -83,42 +83,17 @@ export interface PanelChangeViewPayload {}
  * Events
  */
 
-export const dashLinksUpdated = eventFactory('dash-links-updated');
-export const dashboardSaved = eventFactory<DashboardModel>('dashboard-saved');
-export const removePanel = eventFactory<number>('remove-panel');
-export const searchQuery = eventFactory('search-query');
-
-export const showModal = eventFactory<ShowModalPayload>('show-modal');
-export const showConfirmModal = eventFactory<ShowConfirmModalPayload>('confirm-modal');
-export const hideModal = eventFactory('hide-modal');
-export const showModalReact = eventFactory<ShowModalReactPayload>('show-modal-react');
-
 export const dsRequestResponse = eventFactory<DataSourceResponsePayload>('ds-request-response');
 export const dsRequestError = eventFactory<any>('ds-request-error');
-
-export const toggleSidemenuMobile = eventFactory('toggle-sidemenu-mobile');
 export const toggleSidemenuHidden = eventFactory('toggle-sidemenu-hidden');
-
-export const playlistStarted = eventFactory('playlist-started');
-export const playlistStopped = eventFactory('playlist-stopped');
-
-export const toggleKioskMode = eventFactory<ToggleKioskModePayload>('toggle-kiosk-mode');
-export const toggleViewMode = eventFactory('toggle-view-mode');
-
-export const timeRangeUpdated = eventFactory<TimeRange>('time-range-updated');
 export const templateVariableValueUpdated = eventFactory('template-variable-value-updated');
-export const submenuVisibilityChanged = eventFactory<boolean>('submenu-visibility-changed');
-
 export const graphClicked = eventFactory<GraphClickedPayload>('graph-click');
 
+/**
+ * @internal
+ */
 export const thresholdChanged = eventFactory<ThresholdChangedPayload>('threshold-changed');
 
-export const zoomOut = eventFactory<number>('zoom-out');
-
-export const shiftTime = eventFactory<number>('shift-time');
-
-export const routeUpdated = eventFactory('$routeUpdate');
-
 /**
  * Used for syncing queries badge count in panel edit queries tab
  * Think we can get rid of this soon
@@ -149,14 +124,83 @@ export class DashboardPanelsChangedEvent extends BusEventBase {
   static type = 'dashboard-panels-changed';
 }
 
-export class RefreshEvent extends BusEventBase {
-  static type = 'refresh';
+export class PanelDirectiveReadyEvent extends BusEventBase {
+  static type = 'panel-directive-ready';
 }
 
 export class RenderEvent extends BusEventBase {
   static type = 'render';
 }
 
-export class ThemeChangedEvent extends BusEventWithPayload<GrafanaTheme> {
-  static type = 'theme-changed';
+interface ZoomOutEventPayload {
+  scale: number;
+  updateUrl?: boolean;
+}
+
+export class ZoomOutEvent extends BusEventWithPayload<ZoomOutEventPayload> {
+  static type = 'zoom-out';
+}
+
+export enum ShiftTimeEventDirection {
+  Left = -1,
+  Right = 1,
+}
+
+interface ShiftTimeEventPayload {
+  direction: ShiftTimeEventDirection;
+  updateUrl?: boolean;
+}
+
+export class ShiftTimeEvent extends BusEventWithPayload<ShiftTimeEventPayload> {
+  static type = 'shift-time';
+}
+
+export class AbsoluteTimeEvent extends BusEventBase {
+  static type = 'absolute-time';
+}
+
+export class RemovePanelEvent extends BusEventWithPayload<number> {
+  static type = 'remove-panel';
+}
+
+/**
+ * @deprecated use ShowModalReactEvent instead that has this capability built in
+ */
+export class ShowModalEvent extends BusEventWithPayload<ShowModalPayload> {
+  static type = 'show-modal';
+}
+
+export class ShowConfirmModalEvent extends BusEventWithPayload<ShowConfirmModalPayload> {
+  static type = 'show-confirm-modal';
+}
+
+export class ShowModalReactEvent extends BusEventWithPayload<ShowModalReactPayload> {
+  static type = 'show-react-modal';
+}
+
+/**
+ * @deprecated use ShowModalReactEvent instead that has this capability built in
+ */
+export class HideModalEvent extends BusEventBase {
+  static type = 'hide-modal';
+}
+
+export class DashboardSavedEvent extends BusEventBase {
+  static type = 'dashboard-saved';
+}
+
+export class AnnotationQueryStarted extends BusEventWithPayload<AnnotationQuery> {
+  static type = 'annotation-query-started';
+}
+
+export class AnnotationQueryFinished extends BusEventWithPayload<AnnotationQuery> {
+  static type = 'annotation-query-finished';
+}
+
+export class PanelEditEnteredEvent extends BusEventWithPayload<number> {
+  static type = 'panel-edit-started';
+}
+
+export class PanelEditExitedEvent extends BusEventWithPayload<number> {
+  static type = 'panel-edit-finished';
 }
